/**
 * Component that bounces entity with the music
 * @moduleName BounceWithMusic
 * @snippet BounceWithMusic.snippet
BounceWithMusic({
    additionalScale: 0.1
})
 */
bento.define('components/bouncewithmusic', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity;
        var additionalScale = settings.additionalScale || 0.1;
        var scale = 1;
        var active = true;
        var component = {
            name: 'bounceWithMusic',
            bounce: function () {
                if (!active) {
                    return;
                }
                new Tween({
                    from: 1 + additionalScale,
                    to: 1,
                    in: 30,
                    onUpdate: function (v, t) {
                        scale = v;
                    }
                });
            },
            start: function (data) {
                EventSystem.on('songBeat', component.bounce);
            },
            destroy: function (data) {
                EventSystem.off('songBeat', component.bounce);
            },
            update: function (data) {},
            draw: function (data) {
                data.renderer.scale(scale, scale);
            },
            attached: function (data) {
                entity = data.entity;
                entity.moveComponentTo(component, 0);
            }
        };

        Object.defineProperty(component, 'active', {
            get: function () {
                return active;
            },
            set: function (value) {
                active = value;
            }
        });
        return component;
    };
});