/**
 * Force an entity to stick to a wall.
 * Follows the wall if it moves, useful for sideways spikes etc.
 * @moduleName WallSnapper
 * @snippet WallSnapper.snippet
WallSnapper({
    searchDirection: 'left',
    searchAttempts: 2,
    searchDistance: 256,
})
 */
bento.define('components/wallsnapper', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var searchDirection = settings.searchDirection ||  'left';
        var searchAttempts = settings.searchAttempts ||  2;
        var searchDistance = settings.searchDistance ||  256;
        var entity;

        var wall = null;
        var wallOffset = null;

        var axis, step;
        switch (searchDirection) {
            case 'right':
                axis = 'x';
                step = 2;
                break;
            case 'left':
                axis = 'x';
                step = -2;
                break;
            case 'down':
                axis = 'y';
                step = 2;
                break;
            case 'up':
                axis = 'y';
                step = -2;
                break;
        }

        var findWall = function () {
            var distance = 0;
            var offset = Vector2(0, 0);
            searchAttempts--;

            while (!wall && distance < searchDistance) {
                offset[axis] += step;
                distance += Math.abs(step);
                wall = entity.collidesWith({ family: 'solid', offset:offset });
            }
            if (wall) {
                entity.position.addTo(offset);
                wallOffset = entity.position.subtract(wall.position);
            }
        };

        var component = {
            name: 'wallSnapper',
            start: function (data) {
                findWall();
            },
            destroy: function (data) {},
            update: function (data) {
                if (wall !== null) {
                    entity.position.x = wall.position.x + wallOffset.x;
                    entity.position.y = wall.position.y + wallOffset.y;
                } else if (searchAttempts >= 0) {
                    findWall();
                }
            },
            draw: function (data) {},
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});