/**
 * Parallax scrolling background (with clouds and stuff)
 * @moduleName Background
 * @snippet Background.snippet
Background({})
 */
bento.define('entities/background', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'entities/cloud'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    Cloud
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = Vector2(0, 0);
        var sky = new Entity({
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            scale: new Vector2(128, 1),
            components: [
                new Sprite({
                    imageName: 'bg-sky',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });
        var sky2 = new Entity({
            position: new Vector2(viewport.width / 2, viewport.height * -0.5),
            scale: new Vector2(128, -1),
            components: [
                new Sprite({
                    imageName: 'bg-sky',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
            },
            destroy: function (data) {},
            update: function (data) {
                // position.x = viewport.x * 0.7;
                // position.y = viewport.y * 0.7;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.BACKGROUND,
            name: 'background',
            position: position,
            updateWhenPaused: 0,
            float: true,
            components: [
                behavior,
                sky,
                sky2,
                new Cloud({
                    image: 'bg-cloud',
                    position: new Vector2(viewport.width * 0.5, viewport.height * 0.25),
                    scale: 1.0,
                    scrollSpeed: 1.7,
                }),
                new Cloud({
                    image: 'bg-cloud2',
                    position: new Vector2(viewport.width * 0.1, viewport.height * 0.1),
                    scale: 1.0,
                    scrollSpeed: 0.7,
                }),
                new Cloud({
                    image: 'bg-cloud3',
                    position: new Vector2(viewport.width * 0.8, viewport.height * 0.35),
                    scale: 1.0,
                    scrollSpeed: 1.3,
                }),
                new Cloud({
                    image: 'bg-cloud4',
                    position: new Vector2(viewport.width * -0.2, viewport.height * 0.17),
                    scale: 1.0,
                    scrollSpeed: 1.1,
                }),
                new Cloud.Strip({
                    image: 'bg-tilablecloud',
                    position: new Vector2(0, viewport.height * 0.5),
                    scale: 1.0,
                    scrollSpeed: 0.7,
                    count: 2.0
                }),
                new Cloud.Strip({
                    image: 'bg-tilablecloud',
                    position: new Vector2(0, viewport.height * 0.5 + 950),
                    scale: new Vector2(1, -1),
                    opacity: 0.5,
                    scrollSpeed: 0.7,
                    count: 2.0
                }),
            ]
        });
        return entity;
    };
});