/**
 * Block
 * @moduleName Block
 * @snippet Block.snippet
Block({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/block', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/debug',
    'components/planetwrap',
    'globals',
    'components/trianglerenderer'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Debug,
    PlanetWrap,
    Globals,
    TriangleRenderer
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var size = Utils.getTileProperty(settings, 'size', 128);

        var sprite = new Sprite({
            imageName: 'wall-middle',
            originRelative: new Vector2(0.5, 1.0),
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.getComponent('planetWrap', function (planetWrap) {
                    planetWrap.manageFamily('solid');
                    planetWrap.setWall(true);
                });

                // make boundingBox match Tiled size
                var w = Utils.getTiledObject(settings).width * 128/16;
                var h = 128;
                entity.boundingBox = new Rectangle(-128*0.5, -h, w, h);
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var triangleRenderer = new TriangleRenderer({
            sprite: sprite,
            imageSize: Utils.getTileProperty(settings, 'size', 128),
            objectWidth: Utils.getTiledObject(settings).width,
            objectHeight: Utils.getTiledObject(settings).height,
            tileWidth: ((settings.tiled || {}).tileSet || {}).tilewidth || 16,
            tileHeight: ((settings.tiled || {}).tileSet || {}).tileheight || 16,
        });
        var entity = new Entity({
            z: Globals.Layers.BLOCKS,
            name: 'block',
            family: ['solid'],
            position: position,
            updateWhenPaused: 0,
            float: false,
            scale: Vector2(size / 128, size / 128),
            components: [
                behavior,
                triangleRenderer,
                sprite,
            ]
        });

        return entity;
    };
});