/**
 * parallax scrolling cloud in the background
 * @moduleName Cloud
 * @snippet Cloud.snippet
Cloud({
    image: 'bg-cloud',
    position: new Vector2(0, 0),
    scale: 1.0,
    scrollSpeed: 0.5,
    wrapMultiplier: 1.0,  // optional (for tilable clouds)
                          // how many times our width should we leap after going off-screen?
})
 */
bento.define('entities/cloud', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    var Cloud = function (settings) {
        var viewport = Bento.getViewport();
        var wrapDistance;
        var scale = settings.scale || new Vector2(1, 1);

        // allow scalar parameters for scale
        if (typeof scale === 'number') scale = new Vector2(scale, scale);

        var entity = new Entity({
            z: Globals.Layers.BACKGROUND_DECOR,
            name: 'cloud',
            position: settings.position || new Vector2(0, 0),
            scale: scale,
            alpha: settings.opacity || 1.0,
            components: [
                {
                    start: function () {
                        if (settings.wrapMultiplier) {
                            wrapDistance = entity.dimension.width * settings.wrapMultiplier;
                        } else {
                            wrapDistance = viewport.width + entity.dimension.width;
                        }
                    },
                    update: function () {
                        entity.position.x -= settings.scrollSpeed;
                        if (entity.position.x > wrapDistance - entity.dimension.width) {
                            entity.position.x -= wrapDistance;
                        } else if (entity.position.x < 0 - entity.dimension.width) {
                            entity.position.x += wrapDistance;
                        }
                    }
                },
                new Sprite({
                    imageName: settings.image,
                })
            ]
        });

        return entity;
    };

    // a strip of seamless, tilable clouds!
    Cloud.Strip = function (settings) {
        var i;
        var settingsClone;
        var cloud;
        var strip = new Entity({
            z: Globals.Layers.BACKGROUND_DECOR,
            name: 'cloud-strip',
            position: settings.position || new Vector2(0, 0),
        });

        for (i = 0; i < settings.count; i++) {
            settingsClone = Utils.copyObject(settings);
            settingsClone.position = new Vector2(0, 0);
            settings.wrapMultiplier = settings.count;
            cloud = new Cloud(settingsClone);
            cloud.position.x = i * cloud.dimension.width;
            strip.attach(cloud);
        }

        return strip;
    };

    return Cloud;
});