/**
 * Coin
 * @moduleName Coin
 * @snippet Coin.snippet
Coin({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/coin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'bento/components/eventlistener',
    'components/planetwrap',
    'globals',
    'bento/components/spine',
    'entities/effect',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    EventListener,
    PlanetWrap,
    Globals,
    Spine,
    Effect,
    SaveManager
) {
    'use strict';
    var Coin = function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        // var sprite = new Spine({
        //     spine: 'coin',
        //     animation: 'animation',
        //     scale: 1,
        //     triangleRendering: false
        // });
        var sprite = new Sprite({
            imageName: 'coin',
            originRelative: new Vector2(0.5, 0.5),
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.position.y -= 64;
            },
            destroy: function () {
                songBeatTween.removeSelf();
            },
            update: function () {
                entity.collidesWith({
                    name: 'player',
                    onCollide: function (player) {
                        if (player.isAlive()) {
                            Bento.audio.stopSound('sfx_coin');
                            Bento.audio.playSound('sfx_coin');
                            entity.removeSelf();

                            Coin.createSparkleEffect(entity);

                            SaveManager.add('coins', 1);
                            SaveManager.add('gainedCoins', 1);
                            EventSystem.fire('updateCoins', {
                                entity: entity,
                                coins: 1
                            });
                        }
                    }
                });
            }
        };

        var songBeatTween = new Tween({
            from: 1.2,
            to: 1,
            in: 30,
            ease: 'elastic',
            decay: 5,
            oscillations: 1,
            stay: true,
            onUpdate: function (v, t) {
                entity.scale.x = v;
                entity.scale.y = v;
            }
        });

        var entity = new Entity({
            z: Globals.Layers.COINS,
            name: 'coin',
            family: ['coins'],
            position: position,
            updateWhenPaused: 0,
            float: false,
            components: [
                sprite,
                behavior,
                new EventListener({
                    eventName: 'songBeat',
                    onEvent: function () {
                        songBeatTween.stop();
                        songBeatTween.begin();
                    }
                })
            ]
        });
        return entity;
    };
    Coin.createSparkleEffect = function (entity) {
        var planet;
        var sparkles = new Effect({
            z: Globals.Layers.COINS,
            position: entity.position.clone(),
            spriteSheet: 'effects/coin'
        });
        var glowBehavior = {
            name: 'glowBehavior',
            start: function (data) {
                var parent = data.entity;
                var duration = 20;
                new Tween({
                    from: 0.5,
                    to: 2,
                    delay: 10,
                    applyOnDelay: true,
                    in: duration,
                    ease: 'exponential',
                    decay: 5,
                    onStart: function () {
                        Bento.objects.attach(sparkles);
                    },
                    onUpdate: function (v, t) {
                        parent.scale.x = v;
                        parent.scale.y = v;
                    }
                });
                new Tween({
                    from: 1,
                    to: 0,
                    in: duration,
                    delay: 10,
                    applyOnDelay: true,
                    decay: 1,
                    ease: 'quadratic',
                    onUpdate: function (v, t) {
                        parent.alpha = v;
                    },
                    onComplete: function () {
                        parent.removeSelf();
                    }
                });
            }
        };
        var glow = new Entity({
            z: Globals.Layers.COINS,
            position: entity.position.clone(),
            components: [
                new Sprite({
                    imageName: 'coin-glow',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                glowBehavior
            ]
        });
        var glowBehavior2 = {
            name: 'glowBehavior',
            start: function (data) {
                var parent = data.entity;
                var duration = 30;
                new Tween({
                    from: 1,
                    to: 1.25,
                    in: 5,
                    ease: 'exponential',
                    decay: 1,
                    onUpdate: function (v, t) {
                        parent.scale.x = v;
                        parent.scale.y = v;
                    },
                    onComplete: function () {
                        new Tween({
                            from: 1.25,
                            to: 0,
                            in: 10,
                            ease: 'exponential',
                            // ease: 'easeInBack',
                            decay: 5,
                            onUpdate: function (v, t) {
                                parent.scale.x = v;
                                parent.scale.y = v;
                            }
                        });
                    }
                });

            }
        };
        var glow2 = new Entity({
            z: Globals.Layers.COINS,
            position: entity.position.clone(),
            components: [
                new Sprite({
                    imageName: 'coin-glow2',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                glowBehavior2
            ]
        });
        if (entity.parent) {
            entity.parent.attach(glow);
            entity.parent.attach(glow2);
        } else {
            Bento.objects.attach(glow);
            Bento.objects.attach(glow2);
        }

        // add new planetwraps
        entity.getComponent('planetWrap', function (planetWrap) {
            planet = planetWrap.planet;
        });
        if (planet) {
            sparkles.attach(new PlanetWrap({
                planet: planet
            }));
            glow.attach(new PlanetWrap({
                planet: planet
            }));
            glow2.attach(new PlanetWrap({
                planet: planet
            }));
        }
    };
    return Coin;
});