/**
 * Coin block
 * @moduleName CoinBlock
 * @snippet CoinBlock.snippet
CoinBlock({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/coinblock', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/planetwrap',
    'entities/coin',
    'components/movable',
    'managers/save',
    'entities/glassparticle',
    'modules/tapticengine'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    PlanetWrap,
    Coin,
    Movable,
    SaveManager,
    GlassParticle,
    TapticEngine
) {
    'use strict';

    var bumpableHeight = 0;

    return function (settings) {
        var viewport = Bento.getViewport();
        var sprite = new Sprite({
            imageName: 'glass',
            originRelative: new Vector2(0.5, 0.5),
        });
        var player;
        var bumpVel = 0;
        var isBumping = false;
        var hasBumped = false;
        var transformY = 0;

        var bumpableRegion;

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                player = Bento.objects.get('player');

                // match with coin offset
                entity.position.y -= 64;

                entity.getComponent('planetWrap', function (planetWrap) {
                    bumpableRegion = new Entity({
                        name: 'bumpableRegion',
                        position: entity.position.add(new Vector2(0,64)),
                        boundingBox: new Rectangle(-60, -32, 120, 38),
                        components: [
                            planetWrap.clone()
                        ]
                    });
                    Bento.objects.attach(bumpableRegion);
                });
            },
            destroy: function (data) {
                Bento.objects.remove(bumpableRegion);
            },
            update: function (data) {
                if (isBumping) {
                    transformY += bumpVel;
                    bumpVel += 1;
                    if (transformY > 0) {
                        transformY = 0;
                        isBumping = false;
                    }
                }
                if (!player) {
                    player = Bento.objects.get('player');
                    return;
                }

                // block can only be bumped if the player is moving upwards
                player.getComponent('movable', function (movable) {
                    if (!hasBumped && player.isAlive() && movable.velocity.y <= 0) {
                        player.collidesWith({
                            entity: bumpableRegion,
                            onCollide: function () {
                                behavior.bump();
                                movable.velocity.y = 0;
                                while (player.collidesWith({ entity: bumpableRegion })) {
                                    player.position.y += 1;
                                }
                            }
                        });
                    }
                });
            },
            draw: function (data) {
                data.renderer.translate(0, transformY);
            },
            bump: function () {
                isBumping = true;
                hasBumped = true;
                bumpVel = -10;
                createCoin();
                
                for (var i=0; i<6; i++) {
                    Bento.objects.attach(new GlassParticle(entity));
                }
                entity.removeByName('sprite');
                
                SaveManager.add('coins', 1);
                EventSystem.fire('updateCoins', {
                    coins: 1
                });

                Bento.audio.stopSound('sfx_bump');
                Bento.audio.playSound('sfx_bump');
                Bento.audio.stopSound('sfx_glass');
                Bento.audio.playSound('sfx_glass');

                entity.removeSelf();

                TapticEngine.impact({
                  type: "medium"
                });
            }
        };
        var entity = new Entity({
            z: Globals.Layers.BLOCKS,
            name: 'coinBlock',
            family: ['solid'],
            updateWhenPaused: 0,
            float: false,
            boundingBox: new Rectangle(-64,-64,128,96),
            components: [
                behavior,
                sprite,
            ]
        });
        var createCoin = function () {
            var vel = -35;
            var jumpBehavior = {
                start: function () {},
                update: function () {
                    vel += 2;
                    jumpingCoin.position.y += vel;
                    if (jumpingCoin.timer > 20) {
                        Coin.createSparkleEffect(jumpingCoin);
                        jumpingCoin.removeSelf();
                        Bento.audio.stopSound('sfx_coin');
                        Bento.audio.playSound('sfx_coin');
                    }
                },
            };
            var jumpingCoin = new Entity({
                z: entity.z - 1,
                name: 'jumpingCoins',
                position: entity.position.clone(),
                components: [
                    new Sprite({
                        imageName: 'coin',
                        originRelative: new Vector2(0.5, 0.5),
                    }),
                    jumpBehavior
                ]
            });
            entity.getComponent('planetWrap', function (planetWrap) {
                jumpingCoin.attach(new PlanetWrap({
                    planet: planetWrap.planet
                }));
            });
            Bento.objects.attach(jumpingCoin);
        };
        return entity;
    };
});