/**
 * Causes the direction of movement to change when the player hits it
 * @moduleName DirectionSwitcher
 * @snippet DirectionSwitcher.snippet
DirectionSwitcher({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/directionswitcher', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'bento/eventsystem',
    'globals',
    'components/planetwrap'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    Text,
    Utils,
    Tween,
    EventSystem,
    Globals,
    PlanetWrap
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var size = settings.tiled.tileProperties.size || 128;
        var active = true;

        var sprite = new Sprite({
            imageName: 'direction-changer',
            originRelative: new Vector2(0.5, 1.0),
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                if (!active) return;
                entity.collidesWith({
                    name: 'player',
                    onCollide: function (player) {
                        EventSystem.fire('setDirection', {
                            direction: Utils.oppositeDirection(player.getDirection()),
                            animate: true
                        });
                        active = false;
                        Utils.timeout(120, function () {
                            active = true;
                        });
                    }
                });
            },
            draw: function (data) {}
        };

        var entity = new Entity({
            z: Globals.Layers.ITEMS,
            name: 'directionSwitcher',
            position: position,
            scale: Vector2(size/128, size/128),
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});