/**
 * Entity that disappears on its own after an animation
 * @moduleName Effect
 * @snippet Effect|constructor
Effect({
    z: 0, // default: Globals.Layers.EFFECT
    position: ${1:new Vector2(0, 0)},
    spriteSheet: '$2'
});
 */
bento.define('entities/effect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    return function (settings) {
        /*settings = {
            position: Vector2 // positions the entity
            spriteSheet: '' // name of spritesheet
        }*/
        var viewport = Bento.getViewport();
        var components = settings.components || [];
        var spriteSheet = settings.spriteSheet;
        var sprite = new Sprite({});
        var behavior = {
            name: 'effectBehavior',
            start: function (data) {
                sprite.setSpriteSheet(spriteSheet, function () {
                    entity.removeSelf();
                });
            }
        };
        var entity = new Entity({
            z: settings.z || Globals.Layers.EFFECTS,
            name: 'effect',
            family: ['effects'].concat(settings.family || []),
            position: settings.position,
            updateWhenPaused: settings.updateWhenPaused || Bento.objects.isPaused(),
            components: components.concat([
                sprite,
                behavior
            ])
        });
        return entity;
    };
});