/**
 * A simple enemy
 * @moduleName Acorn
 */
bento.define('entities/enemies/acorn', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/physics',
    'globals',
    'bento/components/eventlistener',
    'components/movable',
    'components/planetwrap',
    'bento/components/spine',
    'entities/effect',
    'components/enemy'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Physics,
    Globals,
    EventListener,
    Movable,
    PlanetWrap,
    Spine,
    Effect,
    Enemy
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var sprite = new Spine({
            spine: 'acorn',
            animation: 'idle',
            scale: 1,
            triangleRendering: false
        });
        var size = Utils.getTileProperty(settings, 'size', 128);
        var physics = new Movable({
            velocity: new Vector2(0, 0),
            gravity: Globals.GRAVITY,
            bounce: new Vector2(1, 0),
            maxVel: new Vector2(63, 63)
        });
        var enemy = new Enemy({
            physics: physics,
            spine: sprite
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.position.y -= size * 0.1;
                // start moving
                // physics.velocity.x = -1;
            },
            update: function (data) {
                
            }
        };
        var startListener = new EventListener({
            name: 'eventListener',
            eventName: 'startLevel',
            ignorePause: false,
            onEvent: function (data) {
                // start moving
                // physics.velocity.x = -2;
            }
        });
        var entity = new Entity({
            z: Globals.Layers.ENEMIES,
            name: 'acorn',
            family: ['enemies'],
            position: position,
            scale: Vector2(size / 128, size / 128),
            boundingBox: new Rectangle(-64, -100, 128, 100),
            components: [
                sprite,
                behavior,
                enemy,
                physics,
                startListener
            ]
        });

        return entity;
    };
});