/**
 * A simple enemy
 * @moduleName Crab
 */
bento.define('entities/enemies/crab', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/physics',
    'globals',
    'components/squisher',
    'bento/components/eventlistener',
    'components/movable',
    'components/planetwrap',
    'bento/components/spine',
    'components/enemy'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Physics,
    Globals,
    Squisher,
    EventListener,
    Movable,
    PlanetWrap,
    Spine,
    Enemy
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        // var sprite = new Sprite({
        //     imageName: 'badfish',
        //     originRelative: new Vector2(0.5, 0.75),
        // });
        var sprite = new Spine({
            spine: 'crab',
            animation: 'idle',
            scale: 1,
            triangleRendering: false
        });
        var size = Utils.getTileProperty(settings, 'size', 128);
        var speed = parseFloat(Utils.getObjectProperty(settings, 'speed', -1));
        var vulnerableOffset = Vector2(0, -2);

        var physics = new Movable({
            velocity: new Vector2(speed, 0),
            gravity: Globals.GRAVITY,
            bounce: new Vector2(1, 0),
            maxVel: new Vector2(63, 63)
        });
        var enemy = new Enemy({
            spine: sprite,
            physics: physics
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            update: function (data) {

            }
        };
        var started = false;
        var startListener = new EventListener({
            name: 'eventListener',
            eventName: 'startLevel',
            ignorePause: false,
            onEvent: function (data) {
                started = true;
                // start moving
                physics.velocity.x = -2;
            }
        });
        var entity = new Entity({
            z: Globals.Layers.ENEMIES,
            name: 'crab',
            family: ['enemies'],
            position: position,
            scale: Vector2(size / 128, size / 128),
            boundingBox: new Rectangle(-64, -100, 128, 100),
            components: [
                sprite,
                behavior,
                enemy,
                physics,
                startListener,
            ]
        });

        return entity;
    };
});