/**
 * A flying enemy
 * @moduleName Fly
 */
bento.define('entities/enemies/fly', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/physics',
    'globals',
    'bento/components/eventlistener',
    'components/movable',
    'components/planetwrap',
    'bento/components/spine',
    'entities/effect',
    'components/enemy'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Physics,
    Globals,
    EventListener,
    Movable,
    PlanetWrap,
    Spine,
    Effect,
    Enemy
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var sprite = new Spine({
            spine: 'fly',
            animation: 'idle',
            scale: 1,
            triangleRendering: false
        });
        var size = Utils.getTileProperty(settings, 'size', 128);
        var flightTimer = 0;
        var flightUpwardsVel = 0;

        var physics = new Movable({
            velocity: new Vector2(0, 0),
            gravity: 0,
            bounce: new Vector2(1, 0),
            maxVel: new Vector2(63, 63)
        });
        var enemy = new Enemy({
            physics: physics,
            spine: sprite,
            onDead: function () {
                physics.gravity = Globals.GRAVITY;
                physics.velocity.y = 0;
            }
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.position.y -= size * 0.25;

                // start moving (higher = slower to account for polar coordinates)
                physics.velocity.x = -3 - (64 / entity.position.y);
            },
            update: function (data) {
                if (enemy.dead) {
                    return;
                }
                // sinusoidal flight path (bobbing up and down)
                flightTimer += Bento.getGameSpeed();
                flightUpwardsVel = Math.sin(flightTimer / 20) * 3;
                physics.velocity.y = flightUpwardsVel;

            },
        };
        var entity = new Entity({
            z: Globals.Layers.ENEMIES,
            name: 'fly',
            family: ['enemies'],
            scale: Vector2(size / 128, size / 128),
            boundingBox: new Rectangle(-64, -100, 128, 100),
            components: [
                sprite,
                behavior,
                enemy,
                physics,
            ]
        });

        return entity;
    };
});