/**
 * Floating text to signify experience gained when you collect a coin, kill an enemy, etc.
 * @moduleName ExpBonusText
 * @snippet ExpBonusText.snippet
ExpBonusText({
    planet: planet,
    posInPlanet: new Vector2(0, 0),
    value: 10
})
 */
bento.define('entities/expbonustext', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'gui/gametext',
    'globals',
    'components/planetwrap'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    GameText,
    Globals,
    PlanetWrap
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var value = settings.value;
        var planet = settings.planet;
        var text = settings.text || '+' + value;
        var posInPlanet = settings.posInPlanet;
        // var position = Utils.wrapPoint(posInPlanet, planet.radius, planet.angle)
        //                     .addTo(planet.position)
        //                     .addTo(posInPlanet);
        var position = posInPlanet.clone();
        var planetWrap = new PlanetWrap({
            planet: planet,
            correctBoundingBox: false
        });

        var scale = new Vector2(1, 1);

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                new Tween({
                    from: 1,
                    to: 0,
                    in: 40,
                    ease: 'easeInQuart',
                    onUpdate: function (v, t) {
                        scale.x = scale.y = v;
                        entity.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });
                new Tween({
                    from: position.y,
                    to: position.y - 200,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        position.y = v;
                    }
                });
                new Tween({
                    from: position.x,
                    to: position.x + 100,
                    in: 40,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        position.x = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };

        var entity = new GameText({
            text: text,
            fontSize: 100,
            z: Globals.Layers.PARTICLES,
            name: 'expBonusText',
            position: position,
            scale: scale,
            components: [behavior, planetWrap]
        });
        return entity;
    };
});