/**
 * Infinitely-wide solid object - one belongs to each planet
 * @moduleName Floor
 * @snippet Floor.snippet
Floor({
    planet: planet,
    yPosition: 0
})
 */
bento.define('entities/floor', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'bento/components/fill',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PlanetWrap,
    Fill,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var planet = settings.planet;
        var yPosition = settings.yPosition || 0;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var planetWrap = new PlanetWrap({
            planet: planet,
            managedFamilies: ['solid'],
            correctBoundingBox: false
        });
        var entity = new Entity({
            name: 'floor',
            family: ['solid'],
            position: new Vector2(0, yPosition),
            // note: x:-Infinity and width:Infinity can cause problems with collision 
            // because x + width becomes NaN
            boundingBox: new Rectangle(-Math.pow(2, 52), 0, Infinity, 128),
            components: [ behavior, planetWrap ]
        });
        
        if (Globals.FLAT_RENDERING) {
            entity.attach(new Fill({
                dimension: entity.boundingBox
            }));
        }

        return entity;
    };
});