/**
 * Glass particle
 * @moduleName GlassParticle
 * @snippet GlassParticle.snippet
GlassParticle({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/glassparticle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/movable',
    'components/planetwrap'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Movable,
    PlanetWrap
) {
    'use strict';
    var GlassParticle = function (coinBlock) {
        var position = coinBlock.position.clone();
        var rotationSpeed = Utils.getRandomRangeFloat(-0.4, 0.4);
        var movableSettings = {
            velocity: new Vector2(
                Utils.getRandomRangeFloat(-20, 20),
                Utils.getRandomRangeFloat(-30, -10)
            ),
            gravity: 2,
            bounce: new Vector2(0.8, 0.8),
            ignoreSolids: true,
        };
        
        var behavior = {
            start: function () {
                new Tween({
                    from: 1, to: 0, in: 30,
                    delay: 40,
                    onUpdate: function (v, t) {
                        particle.alpha = v;
                    },
                    onComplete: function () {
                        particle.removeSelf();
                    }
                });
                Utils.timeout(5, function () {
                    // allow collision after a few frames
                    movableSettings.ignoreSolids = false;
                });
            },
            update: function (data) {
                particle.rotation += rotationSpeed * data.speed;
            }
        };
        var particle = new Entity({
            z: Globals.Layers.PARTICLES,
            name: 'glassParticle',
            position: position,
            boundingBox: new Rectangle(-10, -10, 20, 20),
            components: [
                behavior,
                new Movable(movableSettings),
                new Sprite({
                    imageName: 'glass-gibs',
                    originRelative: new Vector2(0.5, 0.5),
                    frameCountX: 4,
                    animations: {
                        default: {
                            speed: 0,
                            frames: [Utils.getRandom(4)]
                        }
                    }
                })

            ]
        });

        coinBlock.getComponent('planetWrap', function (planetWrap) {
            particle.attach(new PlanetWrap({
                planet: planetWrap.planet,
                correctBoundingBox: false,
            }));
        });
        return particle;
    };
    return GlassParticle;
});