/**
 * Glow particle
 * @moduleName GlowParticle
 * @snippet GlowParticle.snippet
GlowParticle({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/glowparticle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'globals',
    'entities/coin'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    PlanetWrap,
    Globals,
    Coin
) {
    'use strict';
    return function (settings) {
        var planet = settings.planet;
        var glowBehavior = {
            name: 'glowBehavior',
            start: function (data) {
                var parent = data.entity;
                var duration = 20;
                new Tween({
                    from: 0.5,
                    to: 2.5,
                    in: duration,
                    ease: 'exponential',
                    decay: 5,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        parent.scale.x = v;
                        parent.scale.y = v;
                    }
                });
                new Tween({
                    from: 0.25,
                    to: 0,
                    in: duration,
                    // delay: 10,
                    // applyOnDelay: true,
                    decay: 1,
                    ease: 'quadratic',
                    onUpdate: function (v, t) {
                        parent.alpha = v;
                    },
                    onComplete: function () {
                        parent.removeSelf();
                    }
                });
            }
        };
        var glow = new Entity({
            z: Globals.Layers.COINS,
            position: settings.position,
            family: ['glow'],
            components: [
                new Sprite({
                    imageName: 'glow',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                glowBehavior
            ]
        });
        if (planet) {
            glow.attach(new PlanetWrap({
                planet: planet
            }));
        }
        return glow;
    };
});