/**
 * A particle of dust/smoke.
 * All settings are optional, except position.
 * @moduleName Particle
 * @snippet Particle.snippet
Particle({
    position: new Vector2(0, 0),
    // imageName: 'circle',
    // velocity: new Vector2(0, 0),
    // scale: Utils.getRandomRangeFloat(0.75, 1.5),
    // fadeSpeed: Utils.getRandomRangeFloat(0.01, 0.02),
    // planet: null
})
 */
bento.define('entities/particle', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PlanetWrap,
    Globals
) {
    'use strict';
    return function (settings) {
        var velocity = settings.velocity || new Vector2(0, 0);
        var fadeSpeed = settings.fadeSpeed || Utils.getRandomRangeFloat(0.01, 0.02);
        var scale = settings.scale || Utils.getRandomRangeFloat(0.75, 1.5);
        var shouldScale = Utils.getDefault(settings.shouldScale, true); 
        var sprite = new Sprite({
            imageName: settings.imageName,
            originRelative: new Vector2(0.5, 0.5),
        });
        var fadeout = {
            start: function () {
                if (settings.spriteSheet) {
                    sprite.setSpriteSheet(settings.spriteSheet, function () {
                        entity.removeSelf();
                    });
                }
            },
            update: function () {
                entity.position.x += velocity.x;
                entity.position.y += velocity.y;

                if (shouldScale) {
                    entity.scale.x = Utils.approach(entity.scale.x, 0, fadeSpeed);
                    entity.scale.y = Utils.approach(entity.scale.y, 0, fadeSpeed);

                    if (entity.scale.x <= 0) {
                        entity.removeSelf();
                    }
                }
            }
        };
        var entity = new Entity({
            z: Globals.Layers.PARTICLES,
            name: 'particle',
            position: settings.position,
            scale: new Vector2(scale, scale),
            components: [
                sprite,
                fadeout,
            ]
        });
        if (settings.planet) {
            entity.attach(new PlanetWrap({
                planet: settings.planet,
                correctBoundingBox: false
            }));
        }
        return entity;
    };
});