/**
 * Round spinning planet.
 * Use PlanetWrap components to bind other entities to this planet's polar coorinate system.
 * Contains some useful numbers like radius, perimeter, tileSize and playerSize,
 *  which may vary from one planet to the next.
 * @moduleName Planet
 * @snippet Planet.snippet
Planet({
    imageName: 'planet',
    radius: 64,
    position: new Vector2(0, 0)
})
 */
bento.define('entities/planet', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'entities/spike',
    'components/planetwrap',
    'entities/player',
    'bento/components/fill',
    'globals',
    'modules/planetskins',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Spike,
    PlanetWrap,
    Player,
    Fill,
    Globals,
    PlanetSkins,
    SaveManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var skin = settings.skin || SaveManager.load('planetSkin') ||Globals.currentWorld || 'world-grass';
        var graphic;

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // since the background might heavily depend on what skin was chosen for the planet
                // we spawn the background entity here, but only if no background exists yet
                // we don't want to spawn multiple backgrounds after all
                if (!Bento.objects.get('background')) {
                    PlanetSkins.createBackground(skin, entity);
                }
            },
            destroy: function (data) {},
            update: function (data) {
                if (!Globals.FLAT_RENDERING) {
                    graphic.rotation = entity.angle;
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.PLANET,
            name: 'planet',
            position: position,
            updateWhenPaused: 0,
            float: false,
            components: [
                behavior
            ]
        });

        // angle is used to transform all the things that have PlanetWrap components
        entity.angle = 0;
        entity.radius = settings.radius;
        entity.perimeter = 2 * Math.PI * entity.radius;
        entity.dimension.width = entity.perimeter;
        entity.tileSize = settings.tileSize;
        entity.playerSize = settings.playerSize;
        
        // initial direction of travel when the player spawns on this planet
        entity.flip = settings.flip;

        // will generally be overriden if a player spawn is specified in the Tiled map
        entity.spawnPosition = new Vector2(entity.flip ? 80 : -80, -200);

        graphic = PlanetSkins.create(skin, entity);
        entity.attach(graphic);

        return entity;
    };

});