/**
 * Portal that beats the level
 * @moduleName Portal
 * @snippet Portal.snippet
Portal({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/portal', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PlanetWrap,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var size = Utils.getTileProperty(settings, 'size', 128);

        var sprite = new Sprite({
            imageName: 'portal',
            originRelative: new Vector2(0.45, 0.52),
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                entity.position.y -= size * 0.45;
            },
            destroy: function (data) {},
            update: function (data) {
                entity.rotation += 0.1;
                entity.collidesWith({
                    name: 'player',
                    firstOnly: true,
                    onCollide: function (player) {
                        if (Utils.samePlanet(entity, player)) {
                            EventSystem.fire('playerInPortal', entity);
                        }
                    }
                });
            }
        };
        var entity = new Entity({
            z: Globals.Layers.PORTALS,
            name: 'portal',
            position: position,
            updateWhenPaused: 0,
            float: false,
            scale: Vector2(size/128, size/128),
            components: [
                sprite,
                behavior
            ]
        });
        entity.boundingBox = entity.dimension.clone().grow(-10);
        
        return entity;
    };
});