/**
 * A rocket which triggers the event to carry the player to another planet
 * @moduleName Rocket
 * @snippet Rocket.snippet
Rocket({
    destination: 'entityName',
    position: new Vector2(0, 0)
})
 */
bento.define('entities/rocket', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals',
    'entities/rockettarget',
    'components/planetwrap'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals,
    RocketTarget,
    PlanetWrap
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var size = Utils.getTileProperty(settings, 'size', 128);
        var destinationId = Utils.getTiledObject(settings).name || Utils.getObjectProperty(settings, 'destination', null);
        var sprite = new Sprite({
            imageName: 'rocket',
            originRelative: new Vector2(0.5, 1.0),
        });
        var player;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (!player) {
                    player = Bento.objects.get('player');
                }
            },
            destroy: function (data) {},
            update: function (data) {
                if (!player) {
                    player = Bento.objects.get('player');
                    return;
                }
                entity.collidesWith({
                    entity: player,
                    onCollide: function () {
                        if (Utils.samePlanet(entity, player)) {

                            // locks the player
                            EventSystem.fire('playerInRocket', entity);

                            // begins the rocket flying animation to go to the next planet
                            RocketTarget.getById(destinationId, function (dest) {
                                EventSystem.fire('nextPlanet', {
                                    source: entity,
                                    destination: dest
                                });
                            });

                            // this rocket can't be used anymore
                            Bento.objects.remove(entity);


                            var respawnTime = 30 +  Globals.ROCKET_FLIGHT_DELAY + Globals.ROCKET_FLIGHT_DURATION;

                            Utils.timeout(respawnTime, function () {
                                Bento.objects.attach(entity);
                            });
                        }
                    }
                });
            },
            draw: function (data) {}
        };

        var entity = new Entity({
            z: Globals.Layers.ROCKETS,
            name: 'rocket',
            position: position,
            scale: Vector2(size/128, size/128),
            components: [
                sprite,
                behavior
            ]
        });

        return entity;
    };
});