/**
 * Target for rocket to land
 * @moduleName RocketTarget
 * @snippet RocketTarget.snippet
RocketTarget({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/rockettarget', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    var RocketTarget = function (settings) {
        var viewport = Bento.getViewport();
        var targetId = Utils.getTiledObject(settings).name || '';
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // add to targets
                if (targetId) {
                    RocketTarget.targets[targetId] = entity;
                }
                // don't need planetwrap
                entity.removeByName('planetWrap');

                // prevent the player from spawning in the floor and dying
                entity.position.y -= 3;
            },
            destroy: function (data) {
                // remove from targets
                if (targetId) {
                    delete RocketTarget.targets[targetId];
                }
            },
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            name: 'rocketTarget',
            components: [
                behavior
            ]
        }).extend({
            planet: null,
            setPlanet: function (p) {
                entity.planet = p;
            }
        });
        return entity;
    };
    RocketTarget.targets = {};
    RocketTarget.getById = function (id, callback) {
        var target = RocketTarget.targets[id];
        if (callback) {
            callback(target);
        }
        return target;
    };

    return RocketTarget;
});