/**
 * Spinning blade that kills the player on contact.
 * @moduleName Sawblade
 * @snippet Sawblade.snippet
Sawblade({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/sawblade', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/planetwrap'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    PlanetWrap
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var size = Utils.getTileProperty(settings, 'size', 256); // sawblades are twice as big as other tiles
        var secondaryHitbox;

        var sprite = new Sprite({
            imageName: 'sawblade',
            originRelative: new Vector2(0.5, 0.5)
        });
        var behavior = {
            start: function (data) {
                entity.getComponent('planetWrap', function (planetWrap) {
                    planetWrap.manageFamily('deadly');
                    secondaryHitbox = new Entity({
                        name: 'secondaryHitbox',
                        family: ['deadly'],
                        position: entity.position,  // shared position (not cloned)
                        boundingBox: new Rectangle(-60, -90, 120, 200),
                        components: [
                            new PlanetWrap({
                                planet: planetWrap.planet,
                                managedFamilies: ['deadly']
                            })
                        ]
                    });
                    Bento.objects.attach(secondaryHitbox);
                });
                entity.position.y -= size * 0.25;

            },
            destroy: function (data) {
                Bento.objects.remove(secondaryHitbox);
            },
            update: function (data) {
                entity.rotation += 0.5;
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.SAWBLADES,
            name: 'sawblade',
            family: ['deadly'],
            boundingBox: new Rectangle(-90, -64, 180, 128),
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});