/**
 * Invisible entity that mimicks the bounding box, position and family of another entity.
 * This is useful for preventing collision weirdness at the seams of the level.
 * @moduleName ShadowEntity
 * @snippet ShadowEntity.snippet
ShadowEntity({
    other: myEntity,
    offset: new Vector2(0, 0)
})
 */
bento.define('entities/shadowentity', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/components/fill',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Fill,
    Globals
) {
    'use strict';

    // keep the following families in sync
    // for example, if we are shadowing a block whose planet gets deactivated
    // then we would want the solid family to be removed
    var familiesToCheck = ['solid', 'deadly'];
    var numFamiliesToCheck = familiesToCheck.length;

    return function (settings) {
        var other = settings.other;
        var position = other.position.add(settings.offset);
        var viewport = Bento.getViewport();
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                var i, family;
                for (i = 0; i < numFamiliesToCheck; i++) {
                    family = familiesToCheck[i];
                    if (entity.isFamily(family) && !other.isFamily(family)) {
                        entity.removeFromFamily(family);
                    } else if (!entity.isFamily(family) && other.isFamily(family)) {
                        entity.addToFamily(family);
                    }
                }
                position.x = other.position.x + settings.offset.x;
                position.y = other.position.y + settings.offset.y;
                entity.boundingBox = other.boundingBox;
            },
        };
        var entity = new Entity({
            z: Globals.Layers.SHADOW_ENTITIES,
            name: 'shadowEntity',
            family: other.family.slice(),
            position: position,
            updateWhenPaused: other.updateWhenPaused,
            float: other.float,
            dimension: other.dimension,
            boundingBox: other.boundingBox,
            components: [
                behavior
            ]
        });
        if (Globals.SHOW_SHADOW_COPIES) {
            entity.attach(new Fill({
                dimension: other.dimension
            }));
        }

        return entity;
    };
});