/**
 * Manages game speed during endless mode
 * @moduleName SpeedManager
 */
bento.define('entities/speedmanager', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    var SpeedManager = function (settings) {
        var player = settings.player;
        var planet = settings.planet;
        var chunkLoader = settings.chunkLoader;
        var revolutions = 0;
        var gameSpeed = 1;
        var speedManager = {
            name: 'speedManager',
            reset: SpeedManager.reset,
            tweenBackToNormal: function () {
                new Tween({
                    from: gameSpeed,
                    to: 1,
                    in: 60,
                    ignoreGameSpeed: true,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        gameSpeed = v;
                        Bento.setGameSpeed(gameSpeed);
                    },
                    onComplete: function () {

                    }
                });
            },
            increaseSpeed: function () {
                if (gameSpeed < 3 && player.isAlive()) {
                    if (gameSpeed < 1.5) {
                        gameSpeed += 0.05;
                    } else if (gameSpeed < 2) {
                        gameSpeed += 0.025;
                    } else {
                        gameSpeed += 0.01;
                    }
                    Bento.setGameSpeed(gameSpeed);
                }
            },
            update: function () {
                // check when player revolved around planet and increase speed
                if (!chunkLoader.isBossFight && player.position.x > planet.perimeter * revolutions * gameSpeed) {
                    revolutions += 1;

                    // increase speed
                    speedManager.increaseSpeed();
                }
            },
            onBoss: function () {
                // pause game to show boss chunks
                new Tween({
                    from: Bento.getGameSpeed(),
                    to: 0,
                    in: 30,
                    ignoreGameSpeed: true,
                    onStart: function () {},
                    onUpdate: function (v, t) {
                        Bento.setGameSpeed(v);
                    },
                    onComplete: function () {
                        // restore speed
                        new Tween({
                            ignoreGameSpeed: true,
                            from: 0,
                            to: gameSpeed,
                            in: 30,
                            delay: 120,
                            onStart: function () {},
                            onUpdate: function (v, t) {
                                Bento.setGameSpeed(v);
                            },
                            onComplete: function () {}
                        });
                    }
                });
            },
            start: function () {
                EventSystem.on('startBoss', speedManager.onBoss);
            },
            destroy: function () {
                EventSystem.off('startBoss', speedManager.onBoss);
            }
        };
        return speedManager;
    };
    SpeedManager.reset = function () {
        Bento.setGameSpeed(1);
    };
    return SpeedManager;
});