/**
 * Aaah, a spike!
 * @moduleName Spike
 * @snippet Spike.snippet
Spike({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/spike', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/debug',
    'bento/components/fill',
    'components/planetwrap',
    'globals',
    'components/wallsnapper',
    'modules/planetskins'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Debug,
    Fill,
    PlanetWrap,
    Globals,
    WallSnapper,
    PlanetSkins
) {
    'use strict';
    return function (settings) {
        var size = Utils.getTileProperty(settings, 'size', 128);
        var viewport = Bento.getViewport();

        var planetSkin = PlanetSkins.getCurrentSkin();
        var imageName = settings.imageName;
        if (!imageName) {
            imageName = 'spike';
            if (planetSkin.spikeStyle) {
                imageName += '-' + planetSkin.spikeStyle;
            }
        }
        var sprite = new Sprite({
            imageName: imageName,
            originRelative: new Vector2(0.5, 0.9)
        });
        var direction = Utils.getTileProperty(settings, 'direction', 'up');
        var snap = Utils.getObjectProperty(settings, 'snap', null);
        
        var innerSpikeRegion;

        // used to check when the player crosses the spike
        var playerX, oldPlayerX;

        // once the crosses the spike, stays true until the player hits the ground.
        var isBeingCrossed = false;

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                var position = entity.position;
                var dimension = entity.dimension;
                var baseBoundingBox;

                switch (direction) {
                    case 'up':
                        baseBoundingBox = new Rectangle(-35, -2, 70, 10);
                        entity.boundingBox = new Rectangle(-1, -90, 2, 110);
                        break;
                    case 'down':
                        entity.rotation = Math.PI;
                        position.y -= 128;
                        baseBoundingBox = new Rectangle(-35, -2, 70, 10);
                        entity.boundingBox = new Rectangle(-1, -10, 2, 140);
                        break;
                    case 'left': 
                        entity.rotation = Math.PI * 1.5;
                        position.y -= 64;
                        baseBoundingBox = new Rectangle(-2, -35, 10, 70);
                        entity.boundingBox = new Rectangle(-90, -6, 110, 2);
                        break;
                    case 'right': 
                        entity.rotation = Math.PI * 0.5;
                        position.y -= 64;
                        baseBoundingBox = new Rectangle(-6, -35, 10, 70);
                        entity.boundingBox = new Rectangle(-20, -6, 110, 2);
                        break;
                }

                if (snap) {
                    // TODO: this approach kinda sucks for static spikes, should revisit that polar maths and calculate the desired position offset manually
                    entity.attach(new WallSnapper({
                        searchDirection: snap,
                        searchAttempts: 100,
                        searchDistance: 256,
                    }));
                }

                entity.getComponent('planetWrap', function (planetWrap) {
                    planetWrap.manageFamily('deadly');

                    innerSpikeRegion = new Entity({
                        family: ['deadly'],
                        position: entity.position,  // shared position (not cloned)
                        boundingBox: baseBoundingBox,
                        components: [
                            new PlanetWrap({ planet: planetWrap.planet })
                        ]
                    });
                    Bento.objects.attach(innerSpikeRegion);
                });
            },
            destroy: function (data) {
                if (innerSpikeRegion) {
                    Bento.objects.remove(innerSpikeRegion);
                }
            },
            update: function (data) {
                var player = Bento.objects.get('player');
                var position = entity.position;
                if (!player || !Utils.samePlanet(player, entity)) {
                    return;
                }
                if (isBeingCrossed) {
                    if (player.canJump()) {
                        EventSystem.fire('crossedSpike', entity);
                        isBeingCrossed = false;
                    }
                } else {
                    playerX = player.position.x;
                    if (oldPlayerX !== undefined && player.hasStarted() && player.isAlive()) {
                        // TODO: check that there's not a block between us and the player
                        // (in which case it shouldn't count as crossing the spike)
                        if ((oldPlayerX < position.x && playerX >= position.x)
                        || (oldPlayerX >= position.x && playerX < position.x)) {
                            isBeingCrossed = true;
                        }
                    }
                    oldPlayerX = playerX;
                }
                
                if (innerSpikeRegion.isFamily('deadly')) {
                    if (!entity.isFamily('deadly')) {
                        innerSpikeRegion.removeFromFamily('deadly');
                    }
                } else if (entity.isFamily('deadly')) {
                    innerSpikeRegion.addToFamily('deadly');
                }
            },
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.SPIKES,
            name: 'spike',
            family: ['deadly'],
            updateWhenPaused: 0,
            scale: Vector2(size/128 * 0.8, size/128 * 0.8),
            float: false,
            components: [
                behavior,
                sprite,
                // new Debug({})
            ]
        });

        return entity;
    };
});