/**
 * Spring pad that bounces you higher
 * @moduleName SpringPad
 * @snippet SpringPad.snippet
SpringPad({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/springpad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'globals',
    'bento/components/spine'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PlanetWrap,
    Globals,
    Spine
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var size = Utils.getTileProperty(settings, 'size', 128);

        var cooldown = false;

        var spine = new Spine({
            spine: 'springpad',
            animation: 'idle',
            scale: 1,
            triangleRendering: false
        });

        var behavior = {
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                entity.collidesWith({
                    name: 'player',
                    firstOnly: true,
                    onCollide: function (player) {
                        if (cooldown || !Utils.samePlanet(entity, player) || !player.isAlive()) {
                            return;
                        }
                        spine.setAnimation('animation', function () {
                            spine.setAnimation('idle');
                        });

                        // we delay the actual bouncing action, to line up with the spring animation
                        cooldown = true;
                        Utils.timeout(3, function () {
                            player.bounceUpForced(1.4);
                        });
                        Utils.timeout(20, function () {
                            cooldown = false;
                        });
                        // TODO: add a proper spring pad sound
                        Bento.audio.stopSound('sfx_enemy');
                        Bento.audio.playSound('sfx_enemy');
                    }
                });
            }
        };

        var entity = new Entity({
            z: Globals.Layers.ITEMS,
            name: 'springpad',
            updateWhenPaused: 0,
            float: false,
            scale: Vector2(size/128, size/128),
            components: [
                spine,
                behavior
            ]
        });
        
        return entity;
    };
});