/**
 * Collectible star!
 * May trigger in-game events
 * @moduleName Star
 * @snippet Star.snippet
Star({
    position: new Vector2(0, 0),
    size: 128,
})
 */
bento.define('entities/star', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'components/planetwrap',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween,
    PlanetWrap,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var size = Utils.getTileProperty(settings, 'size', 128);
        var scale = size / 128;

        var sprite = new Sprite({
            imageName: 'collectible',
            originRelative: new Vector2(0.5, 1),
        });
        
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                EventSystem.on('songBeat', onMusicBeat);
            },
            destroy: function (data) {
                EventSystem.off('songBeat', onMusicBeat);
            },
            update: function (data) {
                entity.collidesWith({
                    name: 'player',
                    onCollide: function (player) {
                        if (player.isAlive() && Utils.samePlanet(entity, player)) {
                            EventSystem.fire('collectStar', entity);
                            Bento.audio.stopSound('sfx_coin');
                            Bento.audio.playSound('sfx_coin');
                            entity.removeSelf();
                        }
                    }
                });
            },
            draw: function (data) {}
        };

        var onMusicBeat = function() {
            new Tween({
                from: scale * 1.1,
                to: scale * 0.9,
                in: 30,
                delay: 0,
                applyOnDelay: 0,
                ease: 'linear',
                decay: 1,
                oscillations: 1,
                onUpdate: function (v, t) {
                    entity.scale.x = entity.scale.y = v;
                },
            });
        };

        var entity = new Entity({
            z: Globals.Layers.COINS,
            name: settings.name || '',
            family: ['star'],
            position: position,
            updateWhenPaused: 0,
            scale: new Vector2(scale, scale),
            float: false,
            components: [
                sprite,
                behavior
            ]
        });
        return entity;
    };
});