/**
 * A simple moving rectangle, for collision testing purposes.
 * @moduleName TestPlayer
 * @snippet TestPlayer.snippet
TestPlayer({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/testplayer', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/debug',
    'components/planetwrap',
    'components/movable'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Debug,
    PlanetWrap,
    Movable
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var planet = settings.planet;
        var velocity = new Vector2(0, 0);
        var movable = new Movable({velocity: velocity});
        var speed = 8;
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
                velocity.x = 0;
                velocity.y = 0;
                if (Bento.input.isKeyDown('up')) {
                    velocity.y -= speed;
                }
                if (Bento.input.isKeyDown('down')) {
                    velocity.y += speed;
                }
                if (Bento.input.isKeyDown('left')) {
                    velocity.x -= speed;
                }
                if (Bento.input.isKeyDown('right')) {
                    velocity.x += speed;
                }
            },
            draw: function (data) {
                var box = entity.boundingBox;
                if (box) {
                    data.renderer.fillRect([0, 0, 1, 0.5], box.x, box.y, box.width, box.height);
                }
            }
        };

        var planetWrap = new PlanetWrap({
            planet: planet,             // the planet to wrap around (mandatory)
            correctBoundingBox: true,  // adjust bounding box based on spawn height and movement direction
            debug: true,
            //managedFamilies: ['solid']  // automatically add/remove entity from these families based on whether our planet is active.
        });

        var entity = new Entity({
            z: 0,
            name: '',
            family: [''],
            position: position,
            updateWhenPaused: 0,
            float: false,
            components: [
                movable,
                planetWrap,
                // new Debug({}),
                behavior
            ]
        });

        entity.boundingBox = new Rectangle(-64, -128, 128, 128);

        return entity;
    };
});