/**
 * Tutorial overlay
 * @moduleName TutorialHoldToJump
 * @snippet TutorialHoldToJump.snippet
TutorialHoldToJump({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/tutorials/holdtojump', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/modal',
    'globals',
    'bento/components/eventlistener',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Modal,
    Globals,
    EventListener,
    SaveManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var background = new Entity({
            name: 'background',
            alpha: 0.95,
            components: [
                new Sprite({
                    imageName: 'tutorial/background',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var tap = new Entity({
            name: 'tap',
            position: new Vector2(viewport.width / 2, viewport.height * 0.25 - 128),
            components: [
                new Sprite({
                    imageName: 'tutorial/tap',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var hold = new Entity({
            name: 'hold',
            position: new Vector2(viewport.width / 2, viewport.height * 0.60 - 128),
            components: [
                new Sprite({
                    imageName: 'tutorial/hold',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonOk = new ClickButton({
            name: 'buttonOk',
            sfx: 'sfx_button',
            imageName: 'tutorial/button-ok',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width / 2, viewport.height * 0.9 - 128),
            scale: new Vector2(0.8, 0.8),
            onClick: function () {
                entity.removeSelf();
                // save tutorial state
                SaveManager.add('tutorialHoldToJump', 1);
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName()+':gui:holdToJumpOk'
                });
            },
            onButtonDown: function () {
                buttonOk.transform.y = 16;
            },
            onButtonUp: function () {
                buttonOk.transform.y = 0;
            },
            components: [
                new EventListener({
                    eventName: 'songBeat',
                    onEvent: function () {
                        new Tween({
                            from: 0.85,
                            to: 0.8,
                            in: 30,
                            delay: 0,
                            applyOnDelay: 0,
                            ease: 'elastic',
                            decay: 5,
                            oscillations: 1,
                            onUpdate: function (v, t) {
                                buttonOk.scale.x = v;
                                buttonOk.scale.y = v;
                            }
                        });
                    }
                })
            ]
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // stretch background to fill screen
                background.scale.x = viewport.width / background.dimension.width * 1.1;
                background.scale.y = viewport.height / background.dimension.height;

                // tween in alpha
                new Tween({
                    from: 0,
                    to: 1,
                    delay: 20,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });

                // tween in buttonOk
                new Tween({
                    from: buttonOk.position.y + viewport.height / 2,
                    to: buttonOk.position.y,
                    in: 30,
                    delay: 90,
                    applyOnDelay: true,
                    ease: 'elastic',
                    decay: 8,
                    oscillations: 1,
                    onUpdate: function (v, t) {
                        buttonOk.position.y = v;
                    }
                });
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var entity = new Entity({
            z: Globals.Layers.TUTORIAL,
            name: 'tutorial',
            position: position,
            float: true,
            alpha: 0,
            components: [
                new Modal({}),
                background,
                behavior,
                tap,
                hold,
                buttonOk
            ]
        });
        return entity;
    };
});