bento.define('entities/zoom', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport(),
            renderer = Bento.getRenderer(),
            zoomValue = settings.zoom || 1,
            rotation = 0,
            module = {
                /**
                 * Rotates screen.
                 * @function
                 * @param {Number} value - Angle in radians
                 * @instance
                 * @name rotate
                 */
                rotate: function (value) {
                    rotation = value;
                },
                /**
                 * Zooms screen in or out depending on passed value.
                 * @function
                 * @param {Number} value - 0 < 1 to zoom out, > 1 to zoom in. Keep number positive to prevent point reflection.
                 * @instance
                 * @name zoom
                 */
                zoom: function (value) {
                    zoomValue = value;
                },
                /**
                 * Adds two objects that ensure only certain layers get zoomed.
                 */
                start: function () {
                    // check if not added yet
                    if (Bento.objects.get('zoomStart')) {
                        return;
                    }

                    Bento.objects.attach({
                        z: Globals.Layers.CAMERA - 1,
                        name: 'zoomStart',
                        destroy: function () {
                            zoomValue = 1;
                            rotation = 0;
                        },
                        draw: function (data) {
                            renderer.save();
                            renderer.translate(
                                viewport.width * ((1 - zoomValue) / 2),
                                viewport.height * ((1 - zoomValue) / 2)
                            );
                            renderer.scale(zoomValue, zoomValue);
                            // rotate
                            renderer.translate(
                                viewport.width / 2,
                                viewport.height / 2
                            );
                            renderer.rotate(rotation);
                            renderer.translate(-viewport.width / 2, -viewport.height / 2);
                        }
                    });
                    Bento.objects.attach({
                        z: Globals.Layers.GUI - 1,
                        name: 'zoomEnd',
                        draw: function (data) {
                            renderer.restore();
                        }
                    });
                }
            };

        return module;
    };
});