window.VERSION = '0.0.0';
window.BUILD = 0;
window.VERSION = "1.0.12";
window.BUILD = 34;

// undefine require
if (window.require) {
    window.require = undefined;
}
if (window.define) {
    window.define = undefined;
}

if (bento.require.config) {
    // requirejs config
    bento.require.config({
        baseUrl: 'js',
        waitSeconds: 0
    });
}

window.startGame = function () {
    bento.require([
        'bento',
        'bento/math/vector2',
        'bento/math/rectangle',
        'bento/tween',
        'bento/autoresize',
        'utils',
        'init'
    ], function (
        Bento,
        Vector2,
        Rectangle,
        Tween,
        AutoResize,
        Utils,
        Init
    ) {
        // for 'pixi', include pixi.js manually in index.html
        var renderer = 'canvas2d';

        // // set up a dynamic resolution, e.g. 180~240 x 320
        // var landscape = false;
        // var baseSize = new Rectangle(0, 0, 180, 320);
        // var minWidth = 180;
        // var maxWidth = 240;
        // // we lock the width by setting min and max the same
        // var minHeight = 320;
        // var maxHeight = 320; 
        // var pixelSize = 3;

        var landscape = false;
        var baseSize = new Rectangle(0, 0, 1080, 1920);
        var minWidth = 1080,
            maxWidth = 1440; // 1440 for iPad
        var minHeight = 1920,
            maxHeight = 1920;
        var pixelSize = 1;

        var canvasDimension = new AutoResize(baseSize, minHeight, maxHeight, landscape);

        // responsive resize callback (useful for development)
        var onResize = function () {
            var viewport = Bento.getViewport();
            var canvas = document.getElementById('canvas');
            var context;
            var screenRatio = 1;
            var wentOverMinimum = false;
            var isWideScreen = false;

            if (renderer !== 'canvas2d') {
                // responsive resizing only works for canvas2d for now
                return;
            }
            if (!canvas) {
                return;
            }

            context = canvas.getContext('2d');

            canvasDimension = new AutoResize(
                baseSize,
                minHeight,
                maxHeight,
                landscape
            );
            // max/min width
            if (canvasDimension.width > maxWidth) {
                canvasDimension.width = maxWidth;
            }
            if (Math.ceil(canvasDimension.width) < minWidth) {
                canvasDimension.width = minWidth;
                wentOverMinimum = true;
                isWideScreen = true;
            }

            canvas.width = canvasDimension.width * pixelSize;
            canvas.height = canvasDimension.height * pixelSize;
            viewport.width = canvasDimension.width;
            viewport.height = canvasDimension.height;

            // iPhone X: pad extra pixels
            if (wentOverMinimum) {
                screenRatio = window.innerWidth / window.innerHeight;
                // append to height
                canvasDimension.height = Math.round(canvasDimension.width / screenRatio);
                canvas.height = canvasDimension.height * pixelSize;
                viewport.height = canvasDimension.height;
                Utils.isWideScreen = true;
            } else {
                Utils.isWideScreen = false;
            }

            // fit to height
            canvas.style.height = window.innerHeight + 'px';
            canvas.style.width = (viewport.width / viewport.height * window.innerHeight) + 'px';

            // Bento input updates its viewport before this event, so call this manually
            if (Bento.input) {
                Bento.input.updateCanvas();
            }
        };
        if (renderer === 'canvas2d') {
            onResize();
        }

        // setup game
        Bento.setup({
            name: 'Empty Project',
            canvasId: 'canvas',
            canvasDimension: canvasDimension,
            renderer: renderer,
            pixelSize: pixelSize,
            manualResize: (renderer === 'canvas2d') ? true : false,
            subPixel: true,
            useDeltaT: true,
            smoothing: true,
            preventContextMenu: true,
            reload: {
                // DEV ONLY! right mouse click refreshes the current screen with new javascript code
                simple: 'mouseDown-right',
                assets: 'buttonDown-1',
                jump: 'buttonDown-2'
            },
            // DEV ONLY! q downloads a screenshot
            screenshot: 'buttonDown-q',
            // set to false with build script
            dev: false
        }, function () {
            // trigger responsive resize with event listeners
            if (!Utils.isCocoonJs()) {
                window.addEventListener('resize', onResize, false);
                window.addEventListener('orientationchange', onResize, false);
            }
            Init();
        });
    });
};

window.patchUrl = 'http://d3q8vwl9xlq0g.cloudfront.net/fishout/data';
window.patchGame = function () {
    var hasStarted = false;

    window.patcher(window.patchUrl, 'v' + window.VERSION, function (res) {
        if (res) {
            console.log('Ended with', res);
        }
        if (!hasStarted) {
            window.startGame();
            hasStarted = true;
        }
    });
};

// entry points for cordova apps
document.addEventListener('deviceready', function () {
    window.patchGame();
}, false);

// since browsers don't fire the deviceready event, we simulate one here
// remove this part during the build process for cordova apps!
