/**
 * Global values
 * @moduleName Globals
 */
bento.define('globals', [
    'bento',
    'bento/utils',
    'managers/save'
], function (
    Bento,
    Utils,
    SaveManager
) {
    'use strict';

    // define a chunk category (use data to override the default values)
    var category = function (name, data) {
        return Utils.extend({
            name: name, // corresponds to subfolder in this category's world e.g. 'world-disco/easy'
            afterDistance: 0, // these chunks begin to appear after a certain distance has been travelled
            afterBosses: 0, // these chunks begin to appear after a number of bosses have been cleared
            beforeDistance: Infinity, // these chunks only appear before a certain distance has been travelled
            beforeBosses: Infinity, // these chunks only appear before a number of bosses have been cleared
            likeliness: 1.0, // weighting factor when deciding whether to pick a chunk from this category amongst others
            cooldown: 0, // number of chunks to wait before considering this category again
            consecutive: 0, // once this category has been selected, force it to be selected again (can specify a random range e.g. [2,5])
            firstBoss: false, // if this is a boss - true means we have to have defeated this at least once before different bosses will appear
        }, data || {});
    };

    var Globals = {
        // DEMO mode will reset the tutorial flag back on app start (see screens/preloader)
        DEMO: false,
        /**
         * @snippet Globals.SOFTLAUNCH
            Globals.SOFTLAUNCH
         */
        SOFTLAUNCH: true,
        /**
         * Balancing values 
         */
        // amount of coins gift
        getGiftAmount: function () {
            var gifts = SaveManager.load('receivedGifts');
            if (gifts === 0) {
                return 500 + Globals.extraFreeGiftCoins;
            } else if (gifts === 1) {
                return 500 + Globals.extraFreeGiftCoins;
            } else if (gifts === 2) {
                return 400 + Globals.extraFreeGiftCoins;
            } else if (gifts === 3) {
                return 300 + Globals.extraFreeGiftCoins;
            } else if (gifts === 4) {
                return 200 + Globals.extraFreeGiftCoins;
            }
            return 100 + Globals.extraFreeGiftCoins;
        },
        // amount of coins after ad
        getAdAmount: function () {
            return 80 + Globals.extraAdCoins;
        },
        // ads needed to watch to unlock legendary skins
        getTotalAdsForSkin: function (legendarySkinsUnlocked) {
            // number of ads increases 
            return 3 + legendarySkinsUnlocked;
        },
        // time between free gifts, in ms
        getGiftTime: function (gifts) {
            // return time difference (based on number of gifts?) 
            if (Utils.isUndefined(gifts)) {
                return 6 * 60 * 60 * 1000;
            }
            var hours = 1;
            switch (gifts) {
            case 0:
                // 0 should not be possible
                return 2 * 60 * 1000;
            case 1:
                return 2 * 60 * 1000;
            case 2:
                return 5 * 60 * 1000;
            case 3:
                return 15 * 60 * 1000;
            case 4:
                return 30 * 60 * 1000;
            case 5:
                return 45 * 60 * 1000;
            case 6:
                return 1 * 60 * 60 * 1000;
            case 7:
                return 2 * 60 * 60 * 1000;
            case 8:
                return 3 * 60 * 60 * 1000;
            case 9:
                return 4 * 60 * 60 * 1000;
            case 10:
                return 5 * 60 * 60 * 1000;
            default:
                return 6 * 60 * 60 * 1000;
            }

            return 6 * 60 * 60 * 1000;
        },
        getPlanetSkins: function () {
            var arr = [
                ['world-grass', 0],
                ['world-forest', 500],
                ['world-disco', 600],
                ['world-agrume', 750],
                ['world-ice', 1000],
                ['world-maki', 1250],
                ['world-pizza', 1500],
                ['world-vinyl', 1750],
                ['world-space', 2000]
            ];
            if (SaveManager.load('purchased-' + Utils.IapToId['worlds1'])) {
                arr.push(['world-cog', 0]);
                arr.push(['world-coin', 0]);
                arr.push(['world-watermelon', 0]);
            } else {
                arr.push(['iap-worlds1', '$2.99']);
            }
            return arr;
        },
        // values that may be modified by exp items
        extraTrickJumpMultiplier: 1.0,
        secondChanceProbability: 0,
        extraAdCoins: 0,
        extraFreeGiftCoins: 0,

        /**
         * Layer constants
         */
        Layers: {
            BACKGROUND: -50,
            BACKGROUND_DECOR: -40,
            CAMERA: -30,
            SAWBLADES: -25,
            PLANET: -20,
            SHADOW_ENTITIES: -2,
            ONE_WAY_PLATFORMS: -1,
            BLOCKS: 0,
            SPIKES: 150,
            SWITCHES: 200,
            PORTALS: 250,
            COINS: 300,
            ITEMS: 350,
            ROCKETS: 400,
            ENEMIES: 500,
            TNT_BLOCKS: 550,
            PARTICLES: 600,
            PLAYER: 700,
            GUI: 800,
            OVERLAY: 810,
            CONFETTI: 820,
            HUD: 830,
            TUTORIAL: 840,
            FADE: 850,
            PAUSE: 860,
            COINSOVERLAY: 870,
            DIALOG: 880,
            IPHONE_X_OVERLAY: 900,
            DEBUG: 1000,
        },

        // example of pauselevel definitions
        PauseLevels: {
            NONE: 0,
            PAUSE: 1,
            SYSTEM: 2
        },

        // player finetuning properties
        GRAVITY: 1.0 * 1.28,
        SPEED: 5 * 1.28,
        ACCEL: 0.5,
        JUMP_POWER: 23 * 1.28,
        JUMP_LEEWAY: 7, // coyote time
        JUMP_BUFFER: 4, // input buffering
        TRICK_JUMP_BASE_SCORE: 10,

        // when tilt is 0, player is directly in the centre of the screen
        // increase tilt to rotate the world so that the player is closer to the left edge of the screen
        VIEW_TILT: 0.2,

        ROCKET_FLIGHT_DELAY: 50,
        ROCKET_FLIGHT_DURATION: 120,

        // various debug properties / cheats
        STARTING_SCORE: 0,
        STARTING_DISTANCE: 0,
        STARTING_BOSSES_CLEARED: 0,
        INVINCIBLE: false,
        DISABLE_PLANET_ROTATION: false,
        FLAT_RENDERING: false,
        SKIP_BUTTON: false,
        SHOW_BOUNDING_BOXES: false,
        DISABLE_BOX_CORRECTION: false,

        // debug draw for the solid 'shadow entities' which exist at the seams of the planet
        SHOW_SHADOW_COPIES: false,

        // true when player watches rewarded ad
        justWatchedRewarded: false,

        // for when your eyes get tired and you need to preserve your sanity
        // FORCE_SPIN_DIRECTION: 'left',

        // level definitions

        // Chunk conventions:
        // harder chunks should appear later (by requiring more score or bosses-cleared)
        // 'rare' chunks have a <1% chance of appearing. They're usually nice or interesting.
        // 'fancy' chunks are those that:
        //   a) have no hazards on the floor (so the player can beat them without pressing anything)
        //   b) showcase a new mechanic earlier than usual (e.g. balloons/springpads)
        // We give them a cooldown so you won't get 2 in a row.

        levels: [{
            mapName: 'world-grass',
            chunkCategories: [

                category('grass-obstacles', {
                    afterDistance: 200,
                    beforeBosses: 2
                }), // some chunks with one-way platforms (i think we should group these all together for a section in grassland that 'feels' differen)
                category('wip2', {
                    likeliness: 0.3,
                    afterBosses: 1
                }), // some chunks with one-way platforms (i think we should group these all together for a section in grassland that 'feels' differen)
                category('wip', {
                    likeliness: 0.3,
                    afterBosses: 1
                }), // some chunks that encourage trick jumps

                category('grass-easy', {
                    beforeBosses: 2
                }),
                category('grass-medium', {
                    afterDistance: 200,
                    beforeBosses: 3
                }),
                category('grass-hard', {
                    afterDistance: 300
                }),
                category('grass-harder', {
                    afterDistance: 2000
                }),
                category('grass-variations', {
                    afterDistance: 1500
                }),
                category('grass-fancy', {
                    afterDistance: 400,
                    likeliness: 0.5,
                    cooldown: 4
                }),
                category('grass-hard-fancy', {
                    afterDistance: 1000,
                    likeliness: 0.1,
                    cooldown: 4
                }),
                category('rare', {
                    likeliness: 0.01,
                    afterBosses: 1,
                    cooldown: 20
                }),
                category('forest-easy', {
                    afterDistance: 500,
                    beforeBosses: 2
                }),
                category('forest-medium', {
                    afterDistance: 1000
                }),
                category('forest-hard', {
                    afterDistance: 2000
                }),
                category('tnt-passage', {
                    afterDistance: 1000,
                    likeliness: 0.02,
                    consecutive: [3, 5],
                    cooldown: 30
                }),
            ],
            bossCategories: [
                category('grass-boss-first', {
                    firstBoss: true
                }),
                // category('boss-test',   { afterBosses: 0 }),
                category('grass-boss-easy', {
                    beforeBosses: 2,
                    likeliness: 3
                }),
                category('grass-boss-medium', {
                    afterBosses: 0,
                    likeliness: 2
                }),
                category('grass-boss-hard', {
                    afterBosses: 1,
                    likeliness: 1.5
                }),
                category('grass-boss-harder', {
                    afterBosses: 1,
                    likeliness: 1
                }),
                // category('forest-boss-easy',  { afterBosses: 2, likeliness: 3 }),
            ]
        }, {
            mapName: 'world-forest',
            chunkCategories: [
                category('grass-medium'),
                category('forest-easy'),
                category('forest-medium'),
                category('forest-hard'),
                // category('forest-easy',   { beforeBosses: 2 }),
                // category('forest-medium', { afterDistance: 500 }),
                // category('forest-hard',   { afterBosses: 1 }),
                // category('rare',   { likeliness: 0.01 }),
            ],
            bossCategories: [
                category('forest-boss-easy', {
                    beforeBosses: 1
                }),
                category('grass-boss-medium', {
                    afterBosses: 1
                }),
                // category('forest-boss-hard',   { afterBosses: 2, likeliness: 2 }),
            ]
        }, {
            mapName: 'world-disco'
        }, {
            mapName: 'moving-spiked-platform'
        }, {
            mapName: 'balloons'
        }, {
            mapName: 'tnt'
        }, {
            mapName: 'tnt-stairs'
        }, {
            mapName: 'one-way-platforms'
        }, {
            mapName: 'conveyor'
        }, {
            mapName: 'path-test-world'
        }, {
            mapName: 'coinblock-test'
        }, {
            mapName: 'sawblades'
        }, {
            mapName: 'enemy'
        }, {
            mapName: 'chase-the-button'
        }, {
            mapName: 'world-1-1'
        }, {
            mapName: 'pillar-climbing'
        }, {
            mapName: 'paths-test-2'
        }, {
            mapName: 'build-a-bridge'
        }, {
            mapName: 'paths-test'
        }, {
            mapName: 'multiplanet-self-contained-2'
        }, {
            mapName: 'multiplanet-self-contained'
        }, {
            mapName: 'loop'
        }, {
            mapName: 'basic-toggle-switch'
        }, {
            mapName: 'toggle-button-2'
        }, {
            mapName: 'toggle-button'
        }, {
            mapName: 'flipping-tower'
        }, {
            mapName: 'flipping-tower-2'
        }, {
            mapName: 'lots-of-spikes'
        }, {
            mapName: 'easy-stars'
        }, {
            mapName: 'tall-jump'
        }, {
            mapName: 'obvious-loop'
        }, {
            mapName: 'high-loop'
        }, ],
        currentLevel: 0,

        currentWorld: SaveManager.load('currentWorld')
    };
    return Globals;
});