/**
 * big text that says e.g. 'BOSS FIGHT' 'BOSS DEFEATED'
 * @moduleName BossIntroOutroText
 * @snippet BossIntroOutroText.snippet
BossIntroOutroText({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/bossintrooutrotext', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'gui/gametext',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    GameText,
    Globals
) {
    'use strict';

    var animations = {
        'drop-in': function (entity, onComplete) {
            new Tween({
                ignoreGameSpeed: true,
                from: 0,
                to: 1,
                in: 40,
                ease: 'easeInQuart',
                onUpdate: function (v, t) {
                    entity.alpha = v;
                }
            });
            new Tween({
                ignoreGameSpeed: true,
                from: 3,
                to: 1,
                in: 40,
                ease: 'easeInQuart',
                onUpdate: function (v, t) {
                    entity.scale.x = entity.scale.y = v;
                }
            });
            new Tween({
                ignoreGameSpeed: true,
                delay: 100 + entity.extraDuration,
                from: 1,
                to: 0,
                in: 30,
                ease: 'easeInQuad',
                onUpdate: function (v, t) {
                    entity.alpha = v;
                }
            });
            new Tween({
                ignoreGameSpeed: true,
                delay: 100 + entity.extraDuration,
                from: entity.position.x,
                to: entity.position.x - 1000,
                in: 30,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    entity.position.x = v;
                },
                onComplete: onComplete
            });
        },
        'slide-from-right': function (entity, onComplete) {
            var restingX = entity.position.x;
            var startX = restingX + 1000;
            var endX = restingX - 1000;
            new Tween({
                ignoreGameSpeed: true,
                from: startX,
                to: restingX,
                in: 40,
                ease: 'easeOutBack',
                onStart: function () {
                    entity.alpha = 1;
                },
                onUpdate: function (v, t) {
                    entity.position.x = v;
                },
            });
            new Tween({
                ignoreGameSpeed: true,
                delay: 80 + entity.extraDuration,
                from: restingX,
                to: endX,
                in: 40,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    entity.position.x = v;
                },
                onComplete: onComplete
            });
        }
    };

    return function (settings) {
        var viewport = Bento.getViewport();

        var extraDuration = settings.extraDuration || 0;

        var entity = new GameText({
            z: Globals.Layers.OVERLAY,
            position: settings.position || new Vector2(0, 0),
            text: settings.text,
            alpha: 0,
            fontSize: 120,
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(4, 6),
            components: [{
                start: function () {
                    var anim = animations[settings.animation];
                    if (!anim) {
                        Utils.log('Text anim "' + settings.animation + '" does not exist!');
                    } else {
                        settings.position.addTo(viewport);
                        anim(entity, function () {
                            entity.removeSelf();
                        });
                    }

                }
            }]
        });

        entity.extraDuration = extraDuration;

        return entity;
    };
});