/**
 * Button to get coins for watching ad
 * @moduleName CoinsForAdButton
 * @snippet CoinsForAdButton.snippet
CoinsForAdButton({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/buttons/coinsforad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/overlays/coins',
    'managers/save',
    'gui/overlays/dialog',
    'modules/mopub'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    CoinsOverlay,
    SaveManager,
    Dialog,
    MoPub
) {
    'use strict';
    var CoinsForAdButton = function (settings) {
        var viewport = Bento.getViewport();
        var onWatched = settings.onWatched;
        var canShow = MoPub.canShowRewardVideo();

        var buttonAd = new ClickButton({
            name: 'buttonAd',
            sfx: 'sfx_button',
            imageName: canShow ? 'ui/button-ad' : 'ui/button-ad-grey',
            alpha: canShow ? 1 : 0.8,
            frameCountX: 1,
            frameCountY: 1,
            position: settings.position,
            scale: settings.scale,
            onButtonDown: function () {
                if (!canShow) return;
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                if (!canShow) return;
                this.transform.y = 0;
            },
            onClick: function () {
                if (!canShow) return;
                var amount = Globals.getAdAmount();
                CoinsForAdButton.watchRewarded({
                    onWatched: onWatched,
                    amount: amount
                });
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName()+':gui:coinsForAd',
                    value: amount
                });
            },
            components: [
                new Text({
                    position: new Vector2(16, 64),
                    text: '+' + Globals.getAdAmount(),
                    font: 'FredokaOne-Regular',
                    fontSize: 50,
                    fontColor: '#ffffff',
                    align: 'right',
                    textBaseline: 'middle',
                    sharpness: 1,
                    scale: new Vector2(4, 4),
                    maxWidth: 112,
                    maxHeight: 50,
                    linebreaks: false,
                    components: []
                }),
                new Entity({
                    name: 'coin',
                    position: new Vector2(24, 66),
                    scale: new Vector2(0.5, 0.5),
                    alpha: canShow ? 1 : 0.7,
                    components: [
                        new Sprite({
                            imageName: 'coin',
                            originRelative: new Vector2(0, 0.5),
                        })
                    ]
                })
            ].concat(settings.components || [])
        });
        return buttonAd;
    };
    CoinsForAdButton.watchRewarded = function (settings) {
        var onWatched = settings.onWatched;
        var onComplete = function (didComplete) {
            // restore mute state
            Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
            Bento.audio.muteSound(Bento.saveState.load('muteSound', false));
            EventSystem.fire('onMute', {
                muted: Bento.saveState.load('muteMusic', false)
            });
            if (!didComplete) {
                // video was turned off early
                return;
            }
            var amount = settings.amount;
            var coinsOverlay = new CoinsOverlay({
                amount: amount,
                onComplete: function () {
                    EventSystem.fire('updateCoins', {
                        coins: amount
                    });
                }
            });
            SaveManager.add('coins', amount);

            Bento.objects.attach(coinsOverlay);

            if (onWatched) {
                onWatched();
            }
        };
        if (Bento.isDev()) {
            onComplete(true);
            return;
        }
        if (MoPub.canShowRewardVideo()) {
            // mute before showing
            Bento.audio.muteMusic(true);
            MoPub.showRewardVideo(onComplete);
        } else {
            Bento.objects.attach(new Dialog({
                name: 'noAds',
                text: 'Ads not loaded. Try again later.',
                onComplete: function () {}
            }));
        }
    };
    return CoinsForAdButton;
});