/**
 * Button to go to homescreen.
 * @moduleName HomeButton
 * @snippet HomeButton.snippet
HomeButton({
    position: new Vector2(128, 146)
})
 */
bento.define('gui/buttons/home', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/eventlistener',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    EventListener,
    BackButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // to replace click behaviour, pass your own function in settings
        // to extend click behaviour, attach a clickable component
        var onClick = settings.onClick || function () {
            EventSystem.fire('GameAnalytics-custom', {
                id: Utils.currentScreenName()+':gui:home'
            });
            Bento.screens.show('screens/titlescreen');
        };

        var backButtonComponent = new BackButton({
            removeSelf: true,
            onPressed: function (data) {
                data.entity.doCallback();
            }
        });

        var entity = new ClickButton({
            name: settings.name || 'home',
            sfx: 'sfx_button',
            imageName: 'shop/button-home',
            frameCountY: 1,
            frameCountX: 1,
            position: settings.position || new Vector2(128, 146),
            scale: settings.scale || new Vector2(0.6, 0.6),
            float: true,
            visible: settings.visible,
            onButtonDown: function () {
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: onClick,
            components: [
                backButtonComponent
            ]
        });

        return entity;
    };
});