/**
 * Pause button
 * @moduleName PauseButton
 * @snippet PauseButton.snippet
PauseButton({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/buttons/pause', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/eventlistener',
    'gui/overlays/pause',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    EventListener,
    PauseOverlay,
    BackButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var entity = new ClickButton({
            z: settings.z || Globals.Layers.GUI,
            name: 'pauseButton',
            sfx: 'sfx_button',
            imageName: 'ui/button-pause',
            frameCountX: 1,
            frameCountY: 1,
            position: settings.position || new Vector2(128, 146 - 32),
            scale: settings.scale || new Vector2(0.44, 0.44),
            float: true,
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                var pauseOverlay = new PauseOverlay({});
                Bento.objects.attach(pauseOverlay);
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName()+':gui:pause'
                });
            },
            components: [
                new EventListener({
                    name: 'eventListener',
                    eventName: 'playerDead',
                    ignorePause: false,
                    onEvent: function (data) {
                        entity.active = false;
                    }
                }),
                new EventListener({
                    name: 'eventListener',
                    eventName: 'onPlayerRevive',
                    ignorePause: false,
                    onEvent: function (data) {
                        entity.active = true;
                    }
                }),
                new BackButton({
                    removeSelf: true,
                    onPressed: function (data) {
                        data.entity.doCallback();
                    }
                })
            ]
        });
        return entity;
    };
});