/**
 * Shop button (skin select)
 * @moduleName ShopButton
 * @snippet ShopButton.snippet
ShopButton({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/buttons/shop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'gui/overlays/shop',
    'managers/save',
    'gui/badge',
    'components/bouncewithmusic',
    'bento/components/eventlistener',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    ShopOverlay,
    SaveManager,
    Badge,
    BounceWithMusic,
    EventListener,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var behavior = {
            name: 'behavior',
            start: function () {
                checkBadge();
            },
            update: function () {
                if (badge.visible) {
                    entity.rotation = Math.PI / 20 * Math.sin(entity.timer / 10);
                }
            }
        };
        var badge = new Badge({});
        var bounceWithMusic = new BounceWithMusic({
            additionalScale: 0.1
        });
        var entity = new ClickButton({
            name: 'shop',
            sfx: 'sfx_button',
            imageName: 'title/button-skin',
            frameCountX: 1,
            frameCountY: 1,
            scale: settings.scale,
            position: settings.position,
            visible: settings.visible,
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                EventSystem.fire('GameAnalytics-custom', Utils.currentScreenName() + ':gui:shop');
                Bento.screens.show('screens/shop');

                if (SaveManager.load('totalDeaths-' + Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld + ':firstDeathOverlay:buttonShop'
                    });
                }
            },
            components: [
                behavior,
                badge,
                bounceWithMusic,
                new EventListener({
                    eventName: 'updateCoins',
                    ignorePause: true,
                    onEvent: function (data) {
                        checkBadge();
                    }
                })
            ]
        });
        var isSkinUnlocked = function (skinName) {
            var unlocked = SaveManager.load('unlockedSkins');
            return unlocked.indexOf(skinName) >= 0;
        };
        var anySkinUnlockable = function (array) {
            var anySkin = false;
            Utils.forEach(array, function (skinName, i, l, breakLoop) {
                if (!isSkinUnlocked(skinName)) {
                    anySkin = true;
                    breakLoop();
                }
            });
            return anySkin;
        };
        var checkBadge = function () {
            // check if any fish can be unlocked
            // player needs enough coins and not all skins are unlocked yet
            var rarities = ShopOverlay.rarities;
            var common = rarities['common'];
            var rare = rarities['rare'];
            var canUnlock = false;
            var costCommon = ShopOverlay.getCost('common');
            var costRare = ShopOverlay.getCost('rare');
            var coins = SaveManager.load('coins');

            // enough coins to unlock?
            if (coins >= costCommon) {
                // find any skin that can be unlocked
                if (anySkinUnlockable(common)) {
                    canUnlock = true;
                } else {
                    // check coins for rare
                    if (coins >= costRare) {
                        if (anySkinUnlockable(rare)) {
                            canUnlock = true;
                        }
                    }
                }
            }
            if (canUnlock) {
                badge.visible = true;
                bounceWithMusic.active = true;
            } else {
                badge.visible = false;
                bounceWithMusic.active = false;
            }
        };
        return entity;
    };
});