/**
 * Module for toggling haptics
 * @moduleName ToggleHaptics
 */
bento.define('gui/buttons/togglehaptics', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/gui/togglebutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    ToggleButton,
    Counter,
    Text,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        /*settings = {
            // describe your settings object parameters
            position: Vector2 // positions the entity
        }*/
        var viewport = Bento.getViewport();
        var toggleSound = new ToggleButton({
            name: 'toggleSound',
            sfx: 'sfx_button',
            z: settings.z || 0,
            image: Bento.assets.getImage('settings/button-haptic'),
            position: settings.position || new Vector2(0, 0),
            frameCountX: 2,
            frameCountY: 1,
            updateWhenPaused: settings.updateWhenPaused || 0,
            toggled: Bento.saveState.load('disableTapticEngine', false),
            family: settings.family || ['gui'],
            onToggle: function () {
                // we don't include the taptic engine module here because it will disable reloading
                EventSystem.fire('TapticEngine.disable', this.isToggled());
                Bento.saveState.save('disableTapticEngine', this.isToggled());

                EventSystem.fire('GameAnalytics-custom', {
                    id: 'settings:toggleHaptics',
                    value: this.isToggled() ? 0 : 1
                });
            }
        });
        return toggleSound;
    };
});