/**
 * World select button. During softlaunch this button acts as leaderboards button
 * @moduleName WorldSelectButton
 * @snippet WorldSelectButton.snippet
WorldSelectButton({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/buttons/worldselect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'modules/planetskins',
    'modules/gamecenter',
    'managers/save',
    'gui/badge',
    'components/bouncewithmusic',
    'bento/components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    PlanetSkins,
    GameCenter,
    SaveManager,
    Badge,
    BounceWithMusic,
    EventListener
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var scale = settings.scale;
        var position = settings.position;
        var behavior = {
            name: 'behavior',
            start: function () {
                checkBadge();
            },
            update: function () {
                if (badge.visible) {
                    buttonLevelSelect.rotation = Math.PI / 20 * Math.sin(buttonLevelSelect.timer / 10);
                }
            }
        };
        var badge = new Badge({});
        var bounceWithMusic = new BounceWithMusic({
            additionalScale: 0.1
        });
        var buttonLevelSelect = new ClickButton({
            name: 'levelselect',
            sfx: 'sfx_button',
            imageName: /*Globals.SOFTLAUNCH ? 'title/button-leaderboards2' : */ 'title/button-levelselect',
            frameCountX: 1,
            frameCountY: 1,
            scale: scale,
            position: position,
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (false && Globals.SOFTLAUNCH) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Utils.currentScreenName() + ':gui:leaderboards'
                    });
                    GameCenter.show('com.luckykat.fishhop.leaderboards.grass');
                } else {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Utils.currentScreenName() + ':gui:worldSelect'
                    });
                    Bento.screens.show('screens/shopworld');
                }

                if (SaveManager.load('totalDeaths-' + Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld + ':firstDeathOverlay:buttonWorldSelect'
                    });
                }
            }
        });
        var currentWorld = SaveManager.load('planetSkin');
        var planet = PlanetSkins.create(currentWorld, buttonLevelSelect);
        var isUnlocked = function (worldName) {
            var unlockedPlanets = SaveManager.load('unlockedPlanetSkins');
            return unlockedPlanets.indexOf(worldName) >= 0;
        };
        var checkBadge = function () {
            var canUnlock = false;
            var skins = Globals.getPlanetSkins();
            var coins = SaveManager.load('coins');

            // enough coins to any world?
            Utils.forEach(skins, function (skinData, i, l, breakLoop) {
                var skin = skinData[0];
                var cost = skinData[1];

                if (!isUnlocked(skin) && coins >= cost) {
                    canUnlock = true;
                    breakLoop();
                }
            });

            if (canUnlock) {
                badge.visible = true;
                bounceWithMusic.active = true;
            } else {
                badge.visible = false;
                bounceWithMusic.active = false;
            }
        };

        planet.scale.x = 0.52;
        planet.scale.y = 0.52;

        // make face bigger
        Utils.forEach(planet.components, function (component, i, l, breakLoop) {
            var faceName;
            if (component.name === 'sprite' && component.animationSettings.imageName) {
                if (component.animationSettings.imageName.indexOf('face') > 0) {
                    faceName = component.animationSettings.imageName;
                    planet.remove(component);
                    planet.attach(new Entity({
                        name: 'bigFace',
                        scale: new Vector2(2, 2),
                        components: [
                            new Sprite({
                                imageName: faceName,
                                originRelative: new Vector2(0.5, 0.5),
                                frameCountX: 2,
                                frameCountY: 2,
                                animations: {
                                    default: {
                                        speed: 0.2,
                                        frames: [0, 1, 2, 3]
                                    }
                                }
                            })
                        ]
                    }));
                    breakLoop();
                }
            } else if (component.name === 'face') {
                component.scale.x = 2;
                component.scale.y = 2;
            }
        });

        if (true /*!Globals.SOFTLAUNCH*/ ) {
            buttonLevelSelect.removeByName('sprite');
            buttonLevelSelect.attach(planet);
        }

        buttonLevelSelect
            .attach(behavior)
            .attach(badge)
            .attach(bounceWithMusic)
            .attach(new EventListener({
                eventName: 'updateCoins',
                ignorePause: true,
                onEvent: function (data) {
                    checkBadge();
                }
            }));

        return buttonLevelSelect;
    };
});