/**
 * XP button
 * @moduleName XPButton
 * @snippet XPButton.snippet
XPButton({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/buttons/xp', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'managers/save',
    'gui/badge',
    'components/bouncewithmusic',
    'globals'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SaveManager,
    Badge,
    BounceWithMusic,
    Globals
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var badge = new Badge({});
        var bounce = new BounceWithMusic({
            additionalScale: 0.1
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                if (SaveManager.load('unlockedXpRecently')) {
                    badge.visible = true;
                    bounce.active = true;
                } else {
                    badge.visible = false;
                    bounce.active = false;
                }
            },
            update: function () {
                if (badge.visible) {
                    entity.rotation = Math.PI / 20 * Math.sin(entity.timer / 10);
                }
            }
        };
        var entity = new ClickButton({
            name: 'xp',
            sfx: 'sfx_button',
            imageName: 'title/button-xp',
            frameCountX: 1,
            frameCountY: 1,
            scale: settings.scale,
            position: settings.position,
            visible: settings.visible,
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName() + ':gui:xp'
                });
                Bento.screens.show('screens/expscreen');
                SaveManager.save('unlockedXpRecently', false);

                if (SaveManager.load('totalDeaths-' + Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld + ':firstDeathOverlay:buttonXp'
                    });
                }
            },
            components: [
                behavior,
                badge,
                bounce
            ]
        });
        return entity;
    };
});