/**
 * Icon in the top corner which counts coins
 * @moduleName CoinCounter
 * @snippet CoinCounter.snippet
CoinCounter({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/coincounter', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/eventlistener',
    'entities/scoremanager',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    EventListener,
    ScoreManager,
    SaveManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position || new Vector2(viewport.width - 128, 146);

        var coinsChanged = false;

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {},
            update: function (data) {
            },
            draw: function (data) {}
        };

        var coinCounter = new Counter({
            name: 'coinCounter',
            value: SaveManager.load('coins'),
            imageName: 'numbers',
            frameCountX: 10,
            frameCountY: 2,
            padding: 0,
            align: 'center',
            spacing: new Vector2(-16, 0),
            position: new Vector2(0, 32),
            scale: new Vector2(0.65, 0.65)
        });

        var coinIcon = new Entity({
            position: new Vector2(0, -32),
            components: [
                new Sprite({
                    imageName: 'coin',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });


        // hidden cheats menu
        if (Utils.isDev() || Bento.saveState.load('showDeveloperCheats', false)) {
            coinIcon.attach(new Clickable({
                onClick: function (data) {
                    Bento.screens.show('screens/cheatscreen');
                }
            }));
        }

        var entity = new Entity({
            z: Globals.Layers.GUI,
            name: 'coinCounter',
            position: position,
            updateWhenPaused: 0,
            float: true,
            components: [
                behavior,
                coinIcon,
                coinCounter,
                new EventListener({
                    eventName: 'updateCoins',
                    ignorePause: true,
                    onEvent: function (data) {
                        coinCounter.setValue(SaveManager.load('coins'));
                        new Tween({
                            from: 1.2,
                            to: 1,
                            in: 30,
                            ease: 'easeOutQuad',
                            onUpdate: function (v, t) {
                                entity.scale.x = entity.scale.y = v;
                            }
                        });
                    }
                })
            ]
        });

        return entity;
    };
});