/**
 * Overlay that shows when the player dies (as an alternative to having a separate death screen)
 * @moduleName DeathOverlay
 * @snippet DeathOverlay.snippet
DeathOverlay({})
 */
bento.define('gui/deathoverlay', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'gui/gametext',
    'entities/scoremanager',
    'gui/confetti',
    'gui/centeredlist',
    'gui/buttons/gift',
    'gui/coincounter',
    'gui/buttons/coinsforad',
    'components/bouncewithmusic',
    'gui/buttons/worldselect',
    'gui/overlays/iap',
    'gui/loadingspinner',
    'gui/overlays/shop',
    'managers/save',
    'modules/localization',
    'modules/gamecenter',
    'gui/buttons/shop',
    'gui/buttons/xp',
    'gui/buttons/home',
    'modules/mopub',
    'modules/googleplay'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    GameText,
    ScoreManager,
    Confetti,
    CenteredList,
    GiftButton,
    CoinCounter,
    CoinsForAdButton,
    BounceWithMusic,
    WorldSelectButton,
    IapOverlay,
    LoadingSpinner,
    ShopOverlay,
    SaveManager,
    Localization,
    GameCenter,
    ShopButton,
    XPButton,
    HomeButton,
    MoPub,
    GooglePlay
) {
    'use strict';
    var adCounter = 0;
    var noAdsId = Utils.IapToId['noads'];
    return function (settings) {
        var viewport = Bento.getViewport();
        var tweenIn = Utils.getDefault(settings.tweenIn, true);
        var position = settings.position;
        var didShowAd = false;

        var prevFrameScore = 0;
        var oldHighScore, highScore = 0;

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {

                // increase the death count for this world
                SaveManager.add('totalDeaths-'+Globals.currentWorld, 1);

                // if it was our first death in this world, send firstDeath to GameAnalytics
                if (SaveManager.load('totalDeaths-'+Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld+':firstDeath',
                        value: ScoreManager.getScore()
                    });
                }

                EventSystem.fire('GameAnalytics-custom', {
                    id: Globals.currentWorld+':death',
                    value: ScoreManager.getScore()
                });

                // TODO: find a proper usage for progression events

                // EventSystem.fire('GameAnalytics-endProgression', {
                //     id: Globals.currentWorld,        // categorise the progression by current world name
                //     didSucceed: false,               // the player dies, so it doesn't count as 'success'
                //     score: ScoreManager.getScore()
                // });


                EventSystem.fire('GameAnalytics-source', {
                    currency: 'coins',
                    amount: SaveManager.load('gainedCoins'),
                    sourceType: 'gameplay',
                    sourceId: Globals.currentWorld
                });

                mainButtons.iterate(function (item, i) {
                    item.visible = false;
                    new Tween({
                        from: item.position.y + viewport.height / 2,
                        to: item.position.y,
                        in: 100,
                        delay: 30 + i * 10,
                        applyOnDelay: true,
                        ease: 'elastic',
                        decay: 15,
                        oscillations: 3,
                        onStart: function () {
                            item.visible = true;
                        },
                        onUpdate: function (v, t) {
                            item.position.y = v;
                        }
                    });
                });
                // buttonHome
                new Tween({
                    from: buttonHome.position.y - 256,
                    to: buttonHome.position.y,
                    in: 40,
                    delay: 60,
                    applyOnDelay: true,
                    ease: 'elastic',
                    decay: 10,
                    oscillations: 1,
                    onStart: function () {
                        buttonHome.visible = true;
                    },
                    onUpdate: function (v, t) {
                        buttonHome.position.y = v;
                    }
                });
                // coinCounter
                new Tween({
                    from: coinCounter.position.y - 256,
                    to: coinCounter.position.y,
                    in: 40,
                    ease: 'elastic',
                    delay: 60,
                    decay: 10,
                    oscillations: 1,
                    onStart: function () {
                        entity.attach(coinCounter);
                    },
                    onUpdate: function (v, t) {
                        coinCounter.position.y = v;
                    }
                });
                if (tweenIn) {
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 50,
                        delay: 20,
                        ease: 'easeInOutQuad',
                        onUpdate: function (v, t) {
                            fillColor[3] = v * 0.75; // set overlay background alpha
                        }
                    });
                }
                // scoreText
                new Tween({
                    from: scoreText.position.x,
                    to: viewport.width * 0.5,
                    in: 100,
                    delay: 50,
                    ease: 'elastic',
                    decay: 10,
                    oscillations: 4,
                    onUpdate: function (v, t) {
                        scoreText.position.x = v;
                    }
                });
                // highScoreText
                new Tween({
                    from: highScoreText.position.x,
                    to: viewport.width * 0.506,
                    in: 120,
                    delay: 70,
                    ease: 'elastic',
                    decay: 10,
                    oscillations: 4,
                    onStart: function () {
                        loadHighScore();
                        saveHighScore();
                    },
                    onUpdate: function (v, t) {
                        highScoreText.position.x = v;
                    },
                    onComplete: function () {}
                });
                // exp
                new Tween({
                    from: gainedExpText.position.x,
                    to: viewport.width * 0.8,
                    in: 20,
                    delay: 140,
                    ease: 'easeOutQuad',
                    onUpdate: function (v, t) {
                        gainedExpText.position.x = v;
                    },
                    onComplete: function () {
                        new Tween({
                            from: 0,
                            to: ScoreManager.getGainedExp(),
                            in: 50,
                            ease: 'easeOutQuad',
                            onUpdate: function (v, t) {
                                gainedExpText.setText(Math.floor(v) + ' exp?');
                            },
                        });
                    }
                });

                buttonContainer.iterate(function (button, i) {
                    new Tween({
                        from: button.position.y + viewport.height / 2,
                        to: button.position.y,
                        in: 100,
                        delay: 60 + 5 * i,
                        applyOnDelay: true,
                        ease: 'elastic',
                        decay: 20,
                        oscillations: 3,
                        onStart: function () {
                            buttonContainer.visible = true;
                            button.visible = true;
                        },
                        onUpdate: function (v, t) {
                            button.position.y = v;
                        }
                    });
                });

                // remove the hud
                Bento.objects.get('progressBar', function (progressBar) {
                    new Tween({
                        from: progressBar.alpha,
                        to: 0,
                        in: 60,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            progressBar.alpha = v;
                        },
                        onComplete: function () {
                            progressBar.removeSelf();
                        }
                    });
                });
                Bento.objects.get('coinCounter', function (coinCounter) {
                    new Tween({
                        from: coinCounter.alpha,
                        to: 0,
                        in: 60,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            coinCounter.alpha = v;
                        },
                        onComplete: function () {
                            coinCounter.removeSelf();
                        }
                    });
                });

                if (SaveManager.load('receivedGifts') > 0) {
                    showIapPack();
                }
            },
            destroy: function (data) {
                ScoreManager.saveScore();
            },
            update: function (data) {
                var score = ScoreManager.getScore();
                if (prevFrameScore !== score) {
                    scoreText.setText(score);
                    prevFrameScore = score;
                }
            },
            draw: function (data) {}
        };

        var loadHighScore = function () {
            highScore = ScoreManager.getHighScore();
            highScoreText.setText('best: ' + highScore);
        };
        var saveHighScore = function () {
            ScoreManager.saveScore();
            oldHighScore = highScore;
            highScore = ScoreManager.getHighScore();
            if (highScore !== oldHighScore) {
                newRecord();
            }

            // submit score to leaderboards
            var score = ScoreManager.getScore();
            if (Utils.isNativeIos()) {
                GameCenter.post(score, 'com.luckykat.fishhop.leaderboards.grass');
            } else {
                GooglePlay.post(score, 'CgkI39Wq6N8fEAIQAA');
            }
        };
        var newRecord = function () {
            var newRecordText = new GameText({
                position: new Vector2(viewport.width * 0.62, viewport.height * 0.2 + 260),
                rotation: -0.2,
                text: Localization.getText('newRecord'),
                fontSize: 70,
                fontColor: '#ffeb65',
                scale: new Vector2(0, 0)
            });
            new Tween({
                from: 0,
                to: 1,
                in: 60,
                delay: 20,
                ease: 'easeOutElastic',
                onStart: spawnConfetti,
                onUpdate: function (v, t) {
                    newRecordText.scale.x = newRecordText.scale.y = v;
                }
            });
            new Tween({
                from: oldHighScore,
                to: highScore,
                in: 60,
                delay: 40,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    highScoreText.setText(Localization.getText('best').replace('{NUMBER}', Math.floor(v)).toLowerCase());
                }
            });
            entity.attach(newRecordText);
        };
        var spawnConfetti = function () {
            var confetti = new Confetti({
                speedY: 4,
                particleScale: 0.2,
                z: Globals.Layers.CONFETTI,
            });
            Bento.objects.attach(confetti);
            Bento.audio.playSound('sfx_cheer');
        };

        var fillColor = tweenIn ? [0, 0, 0, 0] : [0, 0, 0, 0.75];
        var fill = new Fill({
            color: fillColor
        });
        /**
         * Text
         */
        var scoreText = new GameText({
            position: new Vector2(viewport.width * -0.2, viewport.height * 0.2),
            text: '0',
            fontSize: 180,
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(6, 8)
        });
        var highScoreText = new GameText({
            position: new Vector2(viewport.width * 1.2, viewport.height * 0.2 + 180),
            text: 'best: 0',
            fontSize: 80,
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(6, 8)
        });
        var gainedExpText = new GameText({
            position: new Vector2(viewport.width * 1.2, viewport.height * 0.18),
            text: '0 exp',
            fontSize: 80,
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(4, 6)
        });
        var coinCounter = new CoinCounter({});

        /**
         * Buttons
         */
        var buttonContainer = new CenteredList({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63 + 320),
            spacing: -16
        });

        var buttonXp = new XPButton({
            scale: new Vector2(0.8, 0.8),
            position: new Vector2(0, 0),
            visible: false,
        });
        var buttonLevelSelect = new WorldSelectButton({
            scale: new Vector2(0.8, 0.8),
            position: new Vector2(0, 100)
        });
        var buttonShop = new ShopButton({
            position: new Vector2(0, 100),
            scale: new Vector2(0.8, 0.8),
            visible: false,
        });
        var buttonGift = new GiftButton({
            position: new Vector2(0, 0),
            scale: new Vector2(0.8, 0.8),
            visible: false
        });
        var buttonAd = new CoinsForAdButton({
            onWatched: function () {
                mainButtons.removeItem(buttonAd);
            }
        });
        if (MoPub.canShowRewardVideo()) {
            buttonAd.attach(new BounceWithMusic({}));
        }
        var buttonIap = new ClickButton({
            name: 'iap',
            sfx: 'sfx_button',
            imageName: 'settings/button-noad',
            frameCountX: 1,
            frameCountY: 1,
            float: true,
            scale: Vector2(1.1, 1.1),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                var overlay = IapOverlay.Purchase({
                    z: entity.z + 1,
                    id: noAdsId,
                    fontSettings: {
                        font: 'FredokaOne-Regular',
                        fontSize: 100,
                        maxWidth: viewport.width - 256
                    },
                    onPurchaseComplete: function (success, id, purchaseInfo) {
                        // purchased-com.luckykat.fishhop.noads is set to true in global event listener
                        if (success && id === noAdsId) {
                            mainButtons.removeItem(buttonIap);
                        }
                        overlay.removeByName('loadingSpinner');
                    }
                });
                overlay.attach(new LoadingSpinner({
                    radius: 128,
                    fadeSpeed: 1 / 60,
                    ballsToSpawn: 8,
                    rotateBalls: false,
                    imageName: 'dot',
                    position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
                    scale: new Vector2(0.5, 0.5),
                    components: []
                }));
            },
            components: [
                new BounceWithMusic({})
            ]
        });
        var buttonReplay = new ClickButton({
            name: 'replay',
            sfx: 'sfx_button',
            imageName: 'ui/button-replay',
            frameCountX: 1,
            frameCountY: 1,
            float: true,
            visible: false,
            scale: Vector2(1.1, 1.1),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                Bento.screens.show('screens/main');

                if (SaveManager.load('totalDeaths-' + Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld + ':firstDeathOverlay:buttonReplay'
                    });
                }
            }
        });
        var mainButtons = new CenteredList({
            spacing: 128,
            direction: 'x',
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.62),
            useItemDimensions: true,
            items: [buttonReplay]
        });
        var buttonHome = new HomeButton({
            visible: false,
            onClick: function () {
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName()+':gui:home'
                });
                Bento.screens.show('screens/titlescreen');

                if (SaveManager.load('totalDeaths-' + Globals.currentWorld) === 1) {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Globals.currentWorld + ':firstDeathOverlay:buttonHome'
                    });
                }
            }
        });
        var showIapPack = function () {
            // var skins = ShopOverlay.rarities['epicpack'];
            // some skins aren't fit to be displayed here
            var skins = ['blockyfish', 'toyfish', 'cartoonfish', 'narwhalfish', 'godfish', 'retrofish'];

            // remove skins from packs already purchased
            skins = skins.filter(function (skin) {
                var rarity = ShopOverlay.getRarityBySkin(skin);
                return !ShopOverlay.hasPurchased(rarity);
            });

            // already bought everything
            if (skins.length === 0) {
                return;
            }

            var randomSkin = skins[Utils.getRandom(skins.length)];

            // display iap button on the right
            var glow = new Entity({
                name: 'glow',
                scale: new Vector2(0.3, 0.3),
                alpha: 0.25,
                components: [
                    new Sprite({
                        imageName: 'boss-ui/fx-tada',
                        originRelative: new Vector2(0.5, 0.5),
                    }), {
                        name: 'rotateBehavior',
                        update: function () {
                            glow.rotation += 0.01;
                            pack.rotation = Math.PI / 80 * Math.sin(pack.timer / 10);
                        }
                    }
                ]
            });
            var pack = new Entity({
                name: 'pack',
                components: [
                    new Sprite({
                        imageName: 'shop/skins/' + randomSkin,
                        originRelative: new Vector2(0.5, 0.5),
                    })
                ]
            });
            var buyButton = new Entity({
                name: 'button',
                position: new Vector2(0, 128),
                scale: new Vector2(0.25, 0.25),
                components: [
                    new Sprite({
                        imageName: 'shop/button-purchase',
                        originRelative: new Vector2(0.5, 0.5),
                    }),
                    new GameText({
                        position: new Vector2(0, 0),
                        text: 'GET',
                        fontSize: 160,
                        textBaseline: 'middle'
                    })
                ]
            });
            var iapContainer = new ClickButton({
                name: 'iapContainer',
                boundingBox: new Rectangle(-128, -128, 256, 256),
                frameCountX: 1,
                frameCountY: 1,
                position: new Vector2(viewport.width - 128, viewport.height / 2 - 128),
                components: [
                    glow,
                    pack,
                    buyButton
                ],
                onButtonDown: function () {
                    this.transform.y = 8;
                },
                onButtonUp: function () {
                    this.transform.y = 0;
                },
                onClick: function () {
                    EventSystem.fire('GameAnalytics-custom', {
                        id: Utils.currentScreenName()+':gui:iap'
                    });
                    // open shop to IAP
                    Bento.screens.show('screens/shop', {
                        target: ShopOverlay.getRarityBySkin(randomSkin)
                    });
                }
            });

            new Tween({
                from: iapContainer.position.x + 512,
                to: iapContainer.position.x,
                in: 30,
                delay: 60,
                ease: 'elastic',
                decay: 5,
                oscillations: 1,
                onStart: function () {
                    entity.attach(iapContainer);
                },
                onUpdate: function (v, t) {
                    iapContainer.position.x = v;
                }
            });

            // TODO: check if not already purchased
        };
        var entity = new Entity({
            z: Globals.Layers.OVERLAY,
            name: 'deathOverlay',
            updateWhenPaused: 0,
            float: true,
            components: [
                fill,
                behavior,
                mainButtons,
                buttonHome,
                buttonContainer,
                scoreText,
                highScoreText,
                // gainedExpText,
            ]
        });

        // button container
        buttonContainer.addItem(buttonXp);
        buttonContainer.addItem(buttonLevelSelect);
        buttonContainer.addItem(buttonShop);
        buttonContainer.addItem(buttonGift);
        buttonContainer.visible = false;


        if (SaveManager.load('gameOvers') > 0) {
            mainButtons.addItem(buttonAd);
        }

        if (Globals.justWatchedRewarded) {
            Globals.justWatchedRewarded = false;
            adCounter = 0;
        }

        adCounter += 1;
        adCounter %= 3;

        if (Bento.saveState.load('purchased-' + noAdsId, false) === false) {
            if (adCounter === 0 && MoPub.canShowInterstitial()) {
                // mute before showing
                
                Bento.audio.muteMusic(true);
                didShowAd = true;
                MoPub.showInterstitial(function () {
                    // restore mute state
                    Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                    Bento.audio.muteSound(Bento.saveState.load('muteSound', false));

                    EventSystem.fire('onMute', {
                        muted: Bento.saveState.load('muteMusic', false)
                    });
                });
                

                // prepare to show no ads IAP
                mainButtons.removeItem(buttonAd);
                mainButtons.addItem(buttonIap);
            }
        }

        // show banner first time
        if (!SaveManager.load('showBanner') &&
            !MoPub.isShowingBanner() &&
            !SaveManager.load('purchased-' + Utils.IapToId['noads'])
        ) {
            SaveManager.save('showBanner', true);
            MoPub.showBanner();
        }

        SaveManager.add('gameOvers', 1);

        return entity;
    };
});