/**
 * Text with default settings for this game
 * The text is wrapped in an entity for more control, and to avoid problems with scaling, alpha, etc.
 * @moduleName GameText
 * @snippet GameText.snippet
GameText({
    position: new Vector2(0, 0),
    text: 'tap to continue',
    fontSize: 100,
})
 */
bento.define('gui/gametext', [
    'bento',
    'bento/math/vector2',
    'bento/gui/text',
    'bento/utils',
    'bento/entity',
    'globals'
], function (
    Bento,
    Vector2,
    Text,
    Utils,
    Entity,
    Globals
) {
    'use strict';
    return function (settings) {
        // unpack and delete settings that we should apply to the entity itself
        var z = settings.z !== undefined ? settings.z : Globals.Layers.GUI;
        var name = settings.name || 'gameText';
        var position = settings.position;
        var scale = settings.scale;
        var alpha = settings.alpha;
        var components = settings.components;
        var float = settings.float;
        delete settings.z;
        delete settings.name;
        delete settings.position;
        delete settings.scale;
        delete settings.alpha;
        delete settings.components;
        delete settings.float;

        // extend text object with all other settings
        var text = new Text(Utils.extend({
            font: 'FredokaOne-Regular',
            fontSize: 100,
            fontColor: '#fff',
            align: settings.align || 'center',
            textBaseline: settings.textBaseline || 'top',
            sharpness: 1,
            scale: new Vector2(4, 4),
            ySpacing: 0,
            innerStroke: false,
            pixelStroke: true, // workaround for Cocoon bug
            maxWidth: settings.maxWidth,
            maxHeight: settings.maxHeight,
            linebreaks: Utils.getDefault(settings.linebreaks, true),
            drawDebug: false
        }, settings));

        if (components) {
            components.push(text);
        } else {
            components = [text];
        }

        return new Entity({
            z: z,
            name: name,
            position: position,
            alpha: alpha,
            scale: scale,
            components: components,
            float: float
        }).extend({
            getCanvas: text.getCanvas,
            getStrings: text.getStrings,
            getText: text.getText,
            setText: text.setText,
        });
    };
});