/**
 * Credits
 * @moduleName Credits
 * @snippet Credits.snippet
Credits({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/overlays/credits', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/modal',
    'gui/gametext',
    'modules/gameanalytics',
    'gui/buttons/home'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Modal,
    GameText,
    GameAnalytics,
    HomeButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var background = new Entity({
            name: 'background',
            components: [
                new Sprite({
                    imageName: 'tutorial/background',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var tartine = new Entity({
            name: 'tartine',
            position: new Vector2(viewport.width / 2, viewport.height / 4),
            components: [
                new Sprite({
                    imageName: 'tartine',
                    originRelative: new Vector2(0.5, 0.5),
                }),
                new Clickable({
                    pointerDown: function (data) {
                        if (Bento.input.getPointers().length > 1) {
                            tartine.isHolding = 1;
                        }
                    },
                    pointerUp: function (data) {
                        tartine.isHolding = 0;
                    }
                }), {
                    timer: 0,
                    update: function () {
                        if (tartine.isHolding > 0) {
                            this.timer += 1;
                        } else {
                            this.timer = 0;
                        }
                        if (this.timer === 180) {
                            ask();
                        }
                    }
                }
            ]
        });
        /**
         * Ask about cheat
         */
        var ask = function () {
            var res;
            if (Utils.isCocoonJS() && window.Cocoon) {
                window.Cocoon.Dialog.prompt({
                    title: 'Hello',
                    message: 'What would you like?'
                }, {
                    success: function (text) {
                        showDeveloperCheats(text);
                    }
                });
            } else {
                res = window.prompt("What would you like?");
                showDeveloperCheats(res);
            }
        };
        var showDeveloperCheats = function (code) {
            if (code === 'tartinedev') {
                Bento.saveState.save('showDeveloperCheats', true);
                Bento.screens.show('screens/titlescreen');
                GameAnalytics.disable();
            } else if (code === 'reset') {
                var cheats = Bento.saveState.load('showDeveloperCheats', false);
                Bento.saveState.clear();
                Bento.saveState.save('showDeveloperCheats', cheats);
                Bento.screens.show('screens/titlescreen');
                GameAnalytics.enable();
            } else {
                Bento.saveState.save('showDeveloperCheats', false);
                GameAnalytics.enable();
            }
        };
        var buttonHome = new HomeButton({
            position: new Vector2(128, 128),
            scale: new Vector2(0.6, 0.6),
            onClick: function () {
                entity.removeSelf();
            }
        });
        var text = new GameText({
            position: new Vector2(viewport.width / 2, viewport.height * 2 / 4),
            text: '-Producer-\nHerdjie Zhou\n\n-Programming-\nJeremy Clarke\nHernan Zhou\n\n-Art-\nGregory Parisi',
            textBaseLine: 'middle',
            fontSize: 80,
            maxWidth: viewport.width - 64,
            maxHeight: viewport.height * 3 / 4
        });
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                // stretch background to fill screen
                background.scale.x = viewport.width / background.dimension.width * 1.1;
                background.scale.y = viewport.height / background.dimension.height;
            }
        };
        var modal = new Modal({});
        var entity = new Entity({
            z: settings.z,
            name: 'credits',
            float: true,
            components: [
                modal,
                background,
                // buttonHome,
                tartine,
                text,
                behavior,
                new Clickable({
                    pointerUp: function (data) {
                        entity.removeSelf();
                    },
                })
            ]
        });
        return entity;
    };
});