/**
 * Simple dialog with text and an OK button
 * @moduleName Dialog
 * @snippet Dialog.snippet
Dialog({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/overlays/dialog', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'bento/components/fill',
    'components/modal',
    'globals',
    'components/backbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Fill,
    Modal,
    Globals,
    BackButton
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var onComplete = settings.onComplete;
        var onClose = settings.onClose;
        var onYes = settings.onYes;
        var onNo = settings.onNo;
        var position = settings.position;
        var text = settings.text;
        var type = settings.type || 'yes';
        var canClose = settings.canClose || false;
        var fill = new Entity({
            name: 'fill',
            components: [
                new Fill({
                    color: [0, 0, 0, 0.75]
                })
            ]
        });
        var modal = new Modal({});
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {},
            destroy: function (data) {
                if (onComplete) {
                    onComplete();
                }
            },
            update: function (data) {},
            draw: function (data) {}
        };
        var textEntity = new Text({
            position: new Vector2(0, 0),
            text: text,
            font: 'FredokaOne-Regular',
            fontSize: 80,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            maxWidth: 800 - 32,
            maxHeight: 800 - 32,
            linebreaks: true,
            drawDebug: false,
            components: []
        });
        var box = new Entity({
            name: 'box',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            components: [
                new Sprite({
                    imageName: 'ui/dialog',
                    originRelative: new Vector2(0.5, 0.5)
                }),
                textEntity
            ]
        });
        var buttonOk = new ClickButton({
            name: 'buttonOk',
            sfx: 'sfx_button',
            imageName: type === 'yesno' ? 'ui/button-v' : 'ui/button-ok',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(
                viewport.width / 2 + (type === 'yesno' ? 192 : 0),
                viewport.height / 2 + 400
            ),
            scale: new Vector2(0.8, 0.8),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (onYes) {
                    onYes();
                }
                entity.removeSelf();
            }
        });
        var buttonNo = new ClickButton({
            name: 'buttonNo',
            sfx: 'sfx_button',
            imageName: 'ui/button-x',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(
                viewport.width / 2 - 192,
                viewport.height / 2 + 400
            ),
            scale: new Vector2(0.8, 0.8),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (onNo) {
                    onNo();
                }
                entity.removeSelf();
            }
        });
        buttonNo.attach(new BackButton({
            removeSelf: true,
            onPressed: function (data) {
                data.entity.doCallback();
            }
        }));
        var buttonClose = new ClickButton({
            name: 'buttonClose',
            sfx: 'sfx_button',
            imageName: 'ui/button-x',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(
                viewport.width / 2 - 400,
                viewport.height / 2 - 400
            ),
            scale: new Vector2(0.4, 0.4),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (onClose) {
                    onClose();
                }
                entity.removeSelf();
            }
        });
        var entity = new Entity({
            z: settings.z || Globals.Layers.DIALOG,
            name: settings.name || 'dialog',
            float: true,
            components: [
                modal,
                fill,
                box,
                buttonOk,
                behavior
            ]
        });

        if (type === 'yesno') {
            entity.attach(buttonNo);
        }
        if (canClose) {
            entity.attach(buttonClose);
        }
        return entity;
    };
});