/**
 * Pause screen
 * @moduleName PauseOverlay
 * @snippet PauseOverlay.snippet
PauseOverlay({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/overlays/pause', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/modal',
    'bento/components/fill',
    'globals',
    'gui/overlays/dialog',
    'modules/localization',
    'components/backbutton',
    'gui/buttons/home',
    'modules/mopub',
    'managers/save'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Modal,
    Fill,
    Globals,
    Dialog,
    Localization,
    BackButton,
    HomeButton,
    MoPub,
    SaveManager
) {
    'use strict';
    return function (settings) {
        var noAds = SaveManager.load('purchased-' + Utils.IapToId['noads']);
        var viewport = Bento.getViewport();
        var position = settings.position;
        var fill = new Entity({
            name: 'fill',
            components: [
                new Fill({
                    color: [0, 0, 0, 0.75]
                })
            ]
        });
        var modal = new Modal({});
        /**
         * Buttons
         */
        var buttonPlay = new ClickButton({
            name: 'buttonPlay',
            sfx: 'sfx_button',
            imageName: 'title/button-play',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width*0.5, viewport.height*(noAds ? 0.5 : 0.88)),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                // remove everything except the fill
                entity.remove(buttonPlay);

                // give player 10 frames to respond
                new Tween({
                    from: 1,
                    to: 0,
                    in: 10,
                    ignoreGameSpeed: true,
                    onUpdate: function (v, t) {
                        fill.alpha = v;
                    },
                    onComplete: function () {
                        entity.removeSelf();
                    }
                });

                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName()+':gui:play'
                });
            }
        });

        var goHome = function () {
            EventSystem.fire('GameAnalytics-custom', {
                id: Utils.currentScreenName()+':gui:home'
            });
            Bento.screens.show('screens/titlescreen');
        };

        var buttonHome = new HomeButton({
            onClick: function () {
                MoPub.hideNative();

                // ask first
                var dialog = new Dialog({
                    name: 'goHomeDialog',
                    text: Localization.getText('askQuit'),
                    type: 'yesno',
                    onYes: function () {
                        // display interstitial ad when quitting to homescreen.
                        if (!noAds) {
                            if (MoPub.canShowInterstitial()) {
                                MoPub.showInterstitial();
                            }
                        }

                        // more seamless transition if we switch screens immediately?
                        // but this assumes the ad will appear immediately so the homescreen won't be visible.
                        goHome();
                    },
                    onNo: function () {
                        if (!noAds) {
                            MoPub.showNative();
                        }
                    }
                });
                Bento.objects.attach(dialog);
            }
        });

        var behavior = {
            start: function () {
                MoPub.hideBanner();
                if (!noAds) {
                    MoPub.showNative();
                }
            },
            destroy: function () {
                // now it's OK to dispose of the native ad and load a new one
                MoPub.destroyNative();
                if (!noAds) {
                    MoPub.showBanner();
                }
            }
        };

        var entity = new Entity({
            z: Globals.Layers.PAUSE,
            name: 'pauseOverlay',
            float: true,
            components: [
                modal,
                fill,
                buttonPlay,
                buttonHome,
                behavior
            ]
        });
        return entity;
    };
});