/**
 * Overlay asking user to revive by watching ad
 * @moduleName ReviveForAd
 * @snippet ReviveForAd.snippet
ReviveForAd({})
 */
bento.define('gui/overlays/reviveforad', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/fill',
    'gui/gametext',
    'components/modal',
    'entities/player',
    'bento/components/eventlistener',
    'gui/deathoverlay',
    'managers/save',
    'components/bouncewithmusic',
    'gui/overlays/dialog',
    'modules/mopub'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Fill,
    GameText,
    Modal,
    Player,
    EventListener,
    DeathOverlay,
    SaveManager,
    BounceWithMusic,
    Dialog,
    MoPub
) {
    'use strict';
    var CircleCounter = function (settings) {
        var parts = [];
        var twoPi = Math.PI * 2;
        var radius = settings.radius || 192;
        var count = settings.count || 30;
        var value = settings.value || 1;
        var maxValue = settings.maxValue || 1;
        var container = new Entity({
            z: settings.z,
            name: 'circleCounter',
            position: settings.position,
            components: []
        }).extend({
            setValue: function (v) {
                value = v;
                Utils.forEach(parts, function (part, i, l, breakLoop) {
                    var sx = i * maxValue / l;
                    var dy = 1;
                    var dx = (maxValue / l);
                    part.alpha = Utils.clamp(
                        0,
                        dy / dx * (v - sx),
                        1
                    );
                });
            }
        });
        var makePart = function () {
            var circlePartSprite = new Sprite({
                imageName: 'circlepart',
                originRelative: new Vector2(0.5, 0.5)
            });
            var circlePart = new Entity({
                name: 'circlePart',
                position: new Vector2(0, 0),
                scale: new Vector2(0.5, 1),
                components: [
                    circlePartSprite
                ]
            });
            parts.push(circlePart);
            container.attach(circlePart);
            return circlePart;
        };
        var align = function () {
            Utils.forEach(parts, function (part, i, l, breakLoop) {
                part.position.x = radius * Math.cos(-i * twoPi / l - Math.PI / 2);
                part.position.y = radius * Math.sin(-i * twoPi / l - Math.PI / 2);
                part.rotation = -i * twoPi / l;
            });
        };

        Utils.repeat(count, makePart);
        align();
        return container;
    };
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var fill = new Entity({
            name: 'fill',
            components: [
                new Fill({
                    color: [0, 0, 0, 0.75]
                })
            ]
        });
        var isCanceled = false;
        var isWatchingVid = false;
        var fish = new Player({
            planet: null,
            position: new Vector2(viewport.width / 2, viewport.height / 2 + 64 - 128),
        });
        var continueText = new GameText({
            position: new Vector2(viewport.width / 2, viewport.height / 4 - 128),
            text: 'CONTINUE?',
            fontSize: 100,
            align: 'center',
            textBaseline: 'middle'
        });
        var noThanksText = new GameText({
            text: 'NO THANKS',
            fontSize: 64,
            align: 'center',
            textBaseline: 'middle',
            maxWidth: viewport.width / 2,
            linebreaks: false
        });
        var noThanksButton = new Entity({
            name: 'noThanks',
            position: new Vector2(viewport.width / 2, viewport.height / 4 * 3 + 256 - 128),
            boundingBox: new Rectangle(-viewport.width / 4, -64, viewport.width / 2, 128),
            components: [
                new Clickable({
                    onClick: function (data) {
                        EventSystem.fire('GameAnalytics-custom', {
                            id: Utils.currentScreenName() + ':gui:noReviveForAd'
                        });
                        Bento.audio.playSound('sfx_button');
                        cancel();
                    }
                }),
                noThanksText
            ]
        });
        var buttonAd = new ClickButton({
            name: 'buttonAd',
            sfx: 'sfx_button',
            imageName: 'ui/button-ad',
            frameCountX: 1,
            frameCountY: 1,
            scale: new Vector2(1.2, 1.2),
            position: new Vector2(viewport.width / 2, viewport.height / 4 * 3 - 128),
            onButtonDown: function () {
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (isCanceled) {
                    return;
                }
                if (Bento.isDev()) {
                    countDown();
                    return;
                }
                if (MoPub.canShowRewardVideo()) {
                    isWatchingVid = true;

                    // mute before showing
                    Bento.audio.muteMusic(true);

                    MoPub.showRewardVideo(function (didComplete) {
                        if (didComplete) {
                            countDown();
                        }
                        // restore mute state
                        Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                        Bento.audio.muteSound(Bento.saveState.load('muteSound', false));
                        EventSystem.fire('onMute', {
                            muted: Bento.saveState.load('muteMusic', false)
                        });
                        Globals.justWatchedRewarded = true;
                    });
                } else {
                    Bento.objects.attach(new Dialog({
                        name: 'noAds',
                        text: 'Ads not loaded. Try again later.',
                        onComplete: function () {}
                    }));
                }

                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName() + ':gui:reviveForAd'
                });
            },
            components: [
                new Text({
                    position: new Vector2(0, 56),
                    text: 'CONTINUE',
                    font: 'FredokaOne-Regular',
                    fontSize: 50,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    sharpness: 1,
                    scale: new Vector2(4, 4),
                    maxWidth: 180,
                    maxHeight: 50,
                    linebreaks: false,
                    components: []
                }),
                new BounceWithMusic({})
            ]
        });
        var responseTime = 10;
        var circleCounter = new CircleCounter({
            position: new Vector2(viewport.width / 2, viewport.height / 2 - 128),
            value: responseTime,
            maxValue: responseTime
        });
        var behavior = {
            name: 'behaviorComponent',
            noThanksTween: null,
            counter: responseTime,
            start: function (data) {
                // remove default behavior
                fish.removeByName('behaviorComponent');
                fish.getComponent('spine', function (spine) {
                    spine.setAnimation('sad', function () {}, true);
                });
                fish.scale.x = 1.5;
                fish.scale.y = 1.5;

                // fade in 
                new Tween({
                    from: 0,
                    to: 1,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                });

                // tween in no thanks after 2 seconds
                behavior.noThanksTween = new Tween({
                    from: viewport.height,
                    to: noThanksButton.position.y,
                    in: 30,
                    delay: 60 * 2,
                    applyOnDelay: true,
                    ease: 'elastic',
                    decay: 5,
                    oscillations: 1,
                    onStart: function () {
                        entity.attach(noThanksButton);
                    },
                    onUpdate: function (v, t) {
                        noThanksButton.position.y = v;
                    }
                });
            },
            update: function () {
                // countdown
                if (!isWatchingVid) {
                    behavior.counter -= 1 / 60;
                    if (behavior.counter < 0) {
                        cancel();
                    } else {
                        circleCounter.setValue(behavior.counter);
                    }
                }
            }
        };
        /**
         * Countdown before reviving
         * @type {Object}
         */
        var countDownBehavior = {
            name: 'name',
            count: 3,
            update: function () {
                if (countDownText.ticker % 40 === 39) {
                    countDownBehavior.count -= 1;
                    countDownText.setText(countDownBehavior.count);
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_button');

                    if (countDownBehavior.count === 0) {
                        revive();
                    }
                }
            }
        };
        var countDownText = new GameText({
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            text: '3',
            fontSize: 100,
            components: [
                countDownBehavior
            ]
        });
        /**
         * Start revival countdown
         */
        var countDown = function () {
            entity.remove(fish);
            entity.remove(continueText);
            entity.remove(circleCounter);
            entity.remove(buttonAd);
            entity.remove(behavior);
            entity.remove(noThanksButton);
            entity.remove(behavior.noThanksTween);
            behavior.noThanksTween.removeSelf();

            entity.attach(countDownText);
        };
        /**
         * Revive player
         */
        var revive = function () {
            Bento.objects.get('player', function (player) {
                player.revive();
            });
            SaveManager.add('usedRevive', 1);
            entity.removeSelf();
        };
        /**
         * Remove self and show death overlay
         */
        var cancel = function () {
            entity.removeSelf();
            Bento.objects.attach(new DeathOverlay({
                tweenIn: false
            }));
        };
        var entity = new Entity({
            z: settings.z || Globals.Layers.OVERLAY,
            name: 'reviveForAd',
            position: position,
            updateWhenPaused: 0,
            alpha: 0,
            float: true,
            components: [
                fill,
                fish,
                behavior,
                continueText,
                circleCounter,
                buttonAd,
            ]
        });

        return entity;
    };
});