/**
 * Reviving overlay thanks to "second chance" exp item
 * @moduleName SecondChance
 * @snippet SecondChance.snippet
SecondChance({})
 */
bento.define('gui/overlays/secondchance', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/fill',
    'gui/gametext',
    'components/modal',
    'entities/player',
    'bento/components/eventlistener',
    'managers/save',
    'gui/overlays/dialog',
    'bento/components/nineslice',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Text,
    Utils,
    Tween,
    Globals,
    Fill,
    GameText,
    Modal,
    Player,
    EventListener,
    SaveManager,
    Dialog,
    NineSlice,
    Localization
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var fill = new Entity({
            name: 'fill',
            components: [
                new Fill({
                    color: [0, 0, 0, 0.75]
                })
            ]
        });

        var fxTada = new Entity({
            position: new Vector2(viewport.width*0.5, viewport.height*0.45),
            alpha: 0,
            components: [
                {
                    update: function () {
                        fxTada.rotation += 0.03;
                    }
                },
                new Sprite({
                    imageName: 'boss-ui/fx-tada',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });

        var shieldIcon = new Entity({
            z: Globals.Layers.GUI,
            name: 'shieldIcon',
            position: new Vector2(viewport.width*0.5, viewport.height*0.45),
            scale: new Vector2(0, 0),
            components: [
                {
                    start: function () {
                    }
                },
                new Sprite({
                    imageName: 'xp-ui/power-extralife',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });

        var blueBar = new Entity({
            z: Globals.Layers.GUI,
            position: new Vector2(viewport.width, viewport.height*0.45 + 300),
            scale: new Vector2(1, 1),
            components: [
                new NineSlice({
                    imageName: 'xp-ui/banner',
                    originRelative: new Vector2(0.5, 0.5),
                    width: viewport.width + 400,
                    height: 181
                }),
            ]
        });
        blueBar.position.x += blueBar.dimension.width * 0.52;

        var text = new GameText({
            position: blueBar.position.add(new Vector2(0, -55)),
            text: Localization.getText('secondChance').toUpperCase(),
            fontSize: 100,
        });

        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {

                // overlay alpha
                
                Utils.sequenceTweens([
                {
                    from: 0,
                    to: 1,
                    in: 10,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        entity.alpha = v;
                    }
                },
                {
                    delay: 120, 
                    to: 0,
                    in: 16
                },
                {
                    delay: 8,
                    onComplete: revive
                }
                ]);

                // icon scale + tada!

                Utils.sequenceTweens([
                {
                    from: 0, to: 1, in: 30,
                    delay: 20,
                    ease: 'easeInOutQuad',
                    onUpdate: function (v, t) {
                        shieldIcon.alpha = v;
                        shieldIcon.scale.x = v;
                        shieldIcon.scale.y = v;
                        fxTada.alpha = v*0.2;
                    }
                },
                {
                    delay: 50,
                    to: 0,
                    in: 30,
                    ease: 'easeInOutQuad',
                    onUpdate: function (v, t) {
                        shieldIcon.alpha = v;
                        shieldIcon.scale.x = v;
                        shieldIcon.scale.y = v;
                        shieldIcon.rotation = (1-v) * 5;
                        fxTada.alpha = v*0.2;
                        fxTada.scale.x = v;
                        fxTada.scale.y = v;
                    }
                }
                ]);

                // blue bar

                Utils.sequenceTweens([
                {
                    from: blueBar.position.x,
                    to: viewport.width/2,
                    in: 14,
                    delay: 10,
                    onUpdate: function (v, t) {
                        blueBar.position.x = v;
                    }
                },
                {
                    to: blueBar.dimension.width * -1.04,
                    in: 16,
                    delay: 80,
                }
                ]);

                // text flying along blue bar

                Utils.sequenceTweens([
                {
                    from: viewport.width * 2,
                    to: viewport.width * 0.6,
                    delay: 30,
                    in: 10,
                    onUpdate: function (v, t) {
                        text.position.x = v;
                    }
                },
                {
                    to: viewport.width * 0.45,
                    in: 40,
                },
                {
                    to: viewport.width * -1,
                    in: 10,
                    ease: 'easeInQuad'
                },
                ]);
            }
        };

        /**
         * Revive player
         */
        var revive = function () {
            Bento.objects.get('player', function (player) {
                player.revive();
            });
            SaveManager.add('usedSecondChance', 1);
            entity.removeSelf();
        };

        var entity = new Entity({
            z: settings.z || Globals.Layers.OVERLAY,
            name: 'secondChance',
            position: position,
            updateWhenPaused: 0,
            alpha: 0,
            float: true,
            components: [
                fill,
                behavior,
                fxTada,
                shieldIcon,
                blueBar,
                text
            ]
        });

        return entity;
    };
});