/**
 * Settings overlay
 * @moduleName SettingsOverlay
 * @snippet SettingsOverlay.snippet
SettingsOverlay({})
 */
bento.define('gui/overlays/settings', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/modal',
    'gui/buttons/togglemusic',
    'gui/buttons/togglesound',
    'gui/speaker',
    'gui/gametext',
    'gui/centeredlist',
    'gui/overlays/iap',
    'modules/localization',
    'gui/loadingspinner',
    'gui/buttons/togglehaptics',
    'gui/overlays/credits',
    'components/backbutton',
    'gui/buttons/home',
    'gui/overlays/dialog',
    'modules/cmplay',
    'modules/appsflyer',
    'modules/mopub',
    'components/debug',
    'managers/save',
    'gui/scrollinglist',
    'bento/components/eventlistener'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Modal,
    ToggleMusic,
    ToggleSound,
    Speaker,
    GameText,
    CenteredList,
    IapOverlay,
    Localization,
    LoadingSpinner,
    ToggleHaptics,
    Credits,
    BackButton,
    HomeButton,
    Dialog,
    CMPlay,
    AppsFlyer,
    MoPub,
    Debug,
    SaveManager,
    ScrollingList,
    EventListener
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var noAdsId = Utils.IapToId['noads'];
        var background = new Entity({
            components: [
                new Sprite({
                    imageName: 'settings/bg',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });

        var dotList = new CenteredList({
            spacing: 8,
            position: new Vector2(viewport.width * 0.5, viewport.height - 200),
            components: []
        });

        var scrollingList = new ScrollingList({
            z: 0,
            position: new Vector2(viewport.width / 2, 0),//viewport.height / 2),
            updateWhenPaused: 0,
            direction: 'x',
            spacing: viewport.width, // additional spacing between items
            maxOffset: 0, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: true, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 128,
            onSnap: function (item, index) {
                dotList.iterate(function (dot, i) {
                    if (i === index) {
                        dot.alpha = 1;
                    } else {
                        dot.alpha = 0.5;
                    }
                });
            },
            onLoseFocus: function (index) {}, // called when snap focus changed
            onScrollStart: function () {}, // callback when scrolling has started
            snapMinSlideSpeed: 100, // minimum slidespeed before starting to snap
            snapSpeed: 0.2, // speed of snapping
            damping: 0, // speed decrease, 
            components: [
                new EventListener({
                    eventName: 'cancelScroll',
                    onEvent: function (data) {
                        scrollingList.cancel();
                    }
                })
            ]
        });

        /**
         * List container
         */
        var mainPage = new CenteredList({
            spacing: 308,
            direction: 'y',
            position: new Vector2(viewport.width / 2, viewport.height / 2 - 80),
            useItemDimensions: false
        });
        var aboutPage = new CenteredList({
            spacing: 308,
            direction: 'y',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            useItemDimensions: false
        });
        var privacyContainer = new CenteredList({
            spacing: 330,
            direction: 'y',
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            useItemDimensions: false
        });

        /**
         * Buttons
         */
        var buttonHome = new HomeButton({
            position: new Vector2(128, 128),
            onClick: function () {
                entity.removeSelf();
                EventSystem.fire('GameAnalytics-custom', 'settings:home');
            }
        });
        buttonHome.attach(new BackButton({
            removeSelf: true,
            onPressed: function (data) {
                data.entity.doCallback();
            }
        }));
        var buttonSound = new ToggleSound({});
        var buttonMusic = new ToggleMusic({});
        var toggles = new CenteredList({
            spacing: 64,
            direction: 'x',
            position: new Vector2(0, 0),
            dimension: buttonSound.dimension.clone(),
            items: [
                buttonSound,
                buttonMusic
            ]
        });
        var noAdsContainer = new Entity({
            name: 'noAds',
            components: [
                new GameText({
                    position: new Vector2(100, 0),
                    text: Localization.getText('removeAds').toUpperCase() + ' ' + IapOverlay.getPrice(noAdsId, '$0.99'), // -> get real price asap
                    textBaseline: 'middle',
                    fontSize: 100,
                    maxWidth: 600,
                    maxHeight: 200
                }),
                new Entity({
                    name: 'shine',
                    position: new Vector2(-928 / 2 + 128, 0),
                    scale: new Vector2(0.5, 0.5),
                    components: [
                        new Sprite({
                            imageName: 'boss-ui/fx-tada',
                            originRelative: new Vector2(0.5, 0.5)
                        }),
                        {
                            name: 'rotator',
                            update: function (data) {
                                data.entity.rotation += 0.01;
                            }
                        }
                    ]
                }),
                new Entity({
                    position: new Vector2(-928 / 2 + 128, 0),
                    scale: new Vector2(0.9, 0.9),
                    components: [
                        new Sprite({
                            imageName: 'settings/button-noad',
                            originRelative: new Vector2(0.5, 0.5)
                        })
                    ]
                })
            ]
        });
        var buttonNoAds = new ClickButton({
            name: 'buttonNoAds',
            sfx: 'sfx_button',
            imageName: 'settings/banner-noad',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(0, 0),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                // purchase
                var overlay = IapOverlay.Purchase({
                    z: entity.z + 1,
                    id: noAdsId,
                    fontSettings: {
                        font: 'FredokaOne-Regular',
                        fontSize: 100,
                        maxWidth: viewport.width - 256
                    },
                    onPurchaseComplete: function (success, id, purchaseInfo) {
                        // purchased-com.luckykat.fishhop.noads is set to true in global event listener
                        if (success && id === noAdsId) {
                            buttonNoAds.removeSelf();
                        }
                        overlay.removeByName('loadingSpinner');
                    }
                });
                overlay.attach(new LoadingSpinner({
                    radius: 128,
                    fadeSpeed: 1 / 60,
                    ballsToSpawn: 8,
                    rotateBalls: false,
                    imageName: 'dot',
                    position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
                    scale: new Vector2(0.5, 0.5),
                    components: []
                }));

                EventSystem.fire('GameAnalytics-custom', 'settings:noAds');
            },
            components: [noAdsContainer]
        });
        var buttonRestore = new Entity({
            name: 'restoreContainer',
            components: [
                new ClickButton({
                    name: 'buttonRestore',
                    sfx: 'sfx_button',
                    imageName: 'settings/button-purchases',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        // restore
                        var overlay = IapOverlay.Restore({
                            z: entity.z + 1,
                            fontSettings: {
                                font: 'FredokaOne-Regular',
                                fontSize: 100,
                                maxWidth: viewport.width - 256
                            },
                            onPurchaseComplete: function () {
                                Utils.timeout(60, function () {
                                    if (Bento.saveState.load('purchased-' + noAdsId, false)) {
                                        buttonNoAds.removeSelf();
                                    }
                                });
                                overlay.removeByName('loadingSpinner');
                            }
                        });
                        overlay.attach(new LoadingSpinner({
                            radius: 128,
                            fadeSpeed: 1 / 60,
                            ballsToSpawn: 8,
                            rotateBalls: false,
                            imageName: 'dot',
                            position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
                            scale: new Vector2(0.5, 0.5),
                            components: []
                        }));
                        EventSystem.fire('GameAnalytics-custom', 'settings:restorePurchases');
                    }
                }),
                new GameText({
                    position: new Vector2(100, 0),
                    text: Localization.getText('restore').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4)
                })
            ]
        });
        var toggleHaptics = new Entity({
            name: 'hapticsContainer',
            components: [
                new ToggleHaptics({
                    position: new Vector2(-928 / 2 + 128, 0)
                }),
                new GameText({
                    position: new Vector2(100, 0),
                    text: Localization.getText('haptics').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4)
                })
            ]
        });
        var buttonCredits = new Entity({
            name: 'creditsContainer',
            components: [
                new ClickButton({
                    name: 'buttonCredits',
                    sfx: 'sfx_button',
                    imageName: 'settings/button-credits',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        var credits = new Credits({
                            z: entity.z + 1
                        });
                        Bento.objects.attach(credits);
                        EventSystem.fire('GameAnalytics-custom', 'settings:credits');
                    }
                }),
                new GameText({
                    position: new Vector2(100, 0),
                    text: Localization.getText('credits').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                    components: []
                })
            ]
        });

        /**
         * About page
         */

        var buttonTos = new Entity({
            name: 'tosContainer',
            components: [
                new ClickButton({
                    name: 'buttonTos',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-info',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Utils.openUrl('http://www.cmcm.com/protocol/site/tos.html');
                        EventSystem.fire('GameAnalytics-custom', 'settings:tos');
                    }
                }),
                new GameText({
                    position: new Vector2(160, 0),
                    text: Localization.getText('termsOfService').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                })
            ]
        });

        var buttonPrivacyPolicy = new Entity({
            name: 'privacyPolicyContainer',
            components: [
                new ClickButton({
                    name: 'buttonPrivacyPolicy',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-info',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Utils.openUrl('http://www.cmcm.com/protocol/site/privacy.html');
                        EventSystem.fire('GameAnalytics-custom', 'settings:privacyPolicy');
                    }
                }),
                new GameText({
                    position: new Vector2(160, 0),
                    text: Localization.getText('privacyPolicy').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                })
            ]
        });

        var buttonAdChoice = new Entity({
            name: 'adChoiceContainer',
            components: [
                new ClickButton({
                    name: 'buttonAdChoice',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-info',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Utils.openUrl('http://www.cmcm.com/protocol/site/ad-choice.html');
                        EventSystem.fire('GameAnalytics-custom', 'settings:adChoice');
                    }
                }),
                new GameText({
                    position: new Vector2(160, 0),
                    text: Localization.getText('adChoice').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                })
            ]
        });
        var buttonMyData = new Entity({
            name: 'myDataContainer',
            components: [
                new ClickButton({
                    name: 'buttonMyData',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-blank',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        entity.remove(scrollingList);
                        entity.remove(dotList);
                        entity.attach(privacyContainer);
                        EventSystem.fire('GameAnalytics-custom', 'settings:myData');
                    },
                    components: [
                        new Entity({
                            position: new Vector2(0, 0),
                            scale: new Vector2(0.9, 0.9),
                            components: [
                                new Sprite({
                                    imageName: 'title/button-settings',
                                    originRelative: new Vector2(0.5, 0.5),
                                    frameCountX: 1,
                                    frameCountY: 1,
                                    animations: {
                                        default: {
                                            speed: 0,
                                            frames: [0]
                                        }
                                    }
                                })
                            ]
                        })
                    ]
                }),
                new GameText({
                    position: new Vector2(160, 0),
                    text: Localization.getText('myData').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                }),
            ]
        });


        /**
         * Opens a page on the Cheetah site, allowing the
         *  user to request removal of their personal data.
         */
        var buttonDeleteData = new Entity({
            name: 'deleteDataContainer',
            components: [
                new ClickButton({
                    name: 'buttonDeleteData',
                    sfx: 'sfx_button',
                    imageName: 'settings/button-deletesave',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Utils.openUrl('http://cmcm.com/en-us/opt-out/index.html?pkg=com.luckykat.fishhop');
                        EventSystem.fire('GameAnalytics-custom', 'settings:myData:delete');
                    }
                }),
                new GameText({
                    position: new Vector2(140, 0),
                    text: Localization.getText('deleteData').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                    components: []
                })
            ]
        });
        
        var showConsentUpdated = function () {
            // updating two dialogs on the same frame can cause issues?
            Utils.timeout(1, function () {
                Bento.objects.attach(new Dialog({
                    text: Localization.getText('consentUpdated'),
                }));
            });
        };

        var buttonDontCollect = new Entity({
            name: 'dontCollectContainer',
            components: [
                new ClickButton({
                    name: 'buttonDontCollect',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-blank',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Bento.objects.attach(new Dialog({
                            name: 'dialog',
                            text: Localization.getText('askDontCollect'),
                            canClose: true,
                            type: 'yesno',
                            onYes: function () {
                                SaveManager.save('gdprConsent', false);
                                CMPlay.saveGDPRAgreedAdStayInformed(false);
                                AppsFlyer.setDeviceTrackingDisabled(true);
                                MoPub.revokeConsent();
                                EventSystem.fire('GameAnalytics-custom', 'settings:myData:revokeConsent');
                                showConsentUpdated();
                            },
                            onNo: function () {
                                SaveManager.save('gdprConsent', true);
                                CMPlay.saveGDPRAgreedAdStayInformed(true);
                                AppsFlyer.setDeviceTrackingDisabled(false);
                                MoPub.grantConsent();
                                EventSystem.fire('GameAnalytics-custom', 'settings:myData:grantConsent');
                                showConsentUpdated();
                            },
                            onClose: function () {},
                            onComplete: function () {}
                        }));
                    },
                    components: [
                        new GameText({
                            position: new Vector2(0, -8),
                            text: '!',
                            textBaseline: 'middle',
                            fontSize: 140,
                            maxWidth: 600,
                            maxHeight: 200,
                            shadow: true,
                            shadowColor: '#5e90c3',
                            shadowOffset: new Vector2(0, 4),
                            components: []
                        })
                    ]
                }),
                new GameText({
                    position: new Vector2(140, 0),
                    text: Localization.getText('dontCollect').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                    components: []
                }),
            ]
        });

        /**
         *  To go back to the main settings from the data management screen
         */

        var buttonBack = new Entity({
            name: 'backButtonContainer',
            scale: new Vector2(0.8, 0.8),
            components: [
                new ClickButton({
                    name: 'buttonBack',
                    sfx: 'sfx_button',
                    imageName: 'ui/button-blank',
                    frameCountX: 1,
                    frameCountY: 1,
                    position: new Vector2(-928 / 2 + 128, 0),
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        Bento.audio.stopSound('sfx_button');
                        Bento.audio.playSound('sfx_button');
                        entity.remove(privacyContainer);
                        entity.attach(scrollingList);
                        entity.attach(dotList);
                    },
                    components: [
                        new GameText({
                            position: new Vector2(0, -8),
                            text: '<',
                            textBaseline: 'middle',
                            fontSize: 180,
                            maxWidth: 600,
                            maxHeight: 200,
                        })
                    ]
                }),
                new GameText({
                    position: new Vector2(90, 0),
                    text: Localization.getText('backToSettings').toUpperCase(),
                    textBaseline: 'middle',
                    fontSize: 80,
                    maxWidth: 600,
                    maxHeight: 200,
                    shadow: true,
                    shadowColor: '#5e90c3',
                    shadowOffset: new Vector2(0, 4),
                }),
            ]
        });

        
        /**
         * Other components
         */
        var modal = new Modal({});
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                background.scale.x = viewport.width / background.dimension.width * 1.1;
                background.scale.y = viewport.height / background.dimension.height;
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var speaker = new Speaker({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63)
        });
        var versionText = new GameText({
            position: new Vector2(viewport.width / 2, viewport.height - 4),
            text: 'v' + window.VERSION + '-' + window.BUILD + '.' + (window.PATCHVERSION || 0) + 
                (Bento.isDev() || Bento.saveState.load('showDeveloperCheats', false) ? 'DEV' : '') + (Utils.isFB() ? 'FB' : ''),
            fontSize: 32,
            maxWidth: viewport.width - 64,
            linebreaks: false,
            textBaseline: 'bottom',
            shadow: true,
            shadowColor: '#5e90c3',
            shadowOffset: new Vector2(0, 2)
        });

        /**
         * Container entity
         */
        var entity = new Entity({
            z: settings.z || Globals.Layers.GUI,
            name: 'settingsOverlay',
            position: position,
            float: true,
            components: [
                modal,
                background,
                speaker,
                behavior,
                buttonHome,
                versionText,
                scrollingList
            ]
        });


        mainPage.addItem(toggles);
        if (Bento.saveState.load('purchased-' + noAdsId, false) === false) {
            mainPage.addItem(buttonNoAds);
        }
        mainPage.addItem(buttonRestore);
        mainPage.addItem(toggleHaptics);
        mainPage.addItem(buttonCredits);
        aboutPage.addItem(buttonTos);
        aboutPage.addItem(buttonPrivacyPolicy);
        aboutPage.addItem(buttonAdChoice);

        var addGdpr = function (eu) {
            if (eu) {
                aboutPage.addItem(buttonMyData);
            }
        };
        if (Utils.isDev()) {
            addGdpr(true);
        } else {
            CMPlay.checkIsGDPREnforcedCountry(addGdpr);
        }

        privacyContainer.addItem(buttonDeleteData);
        privacyContainer.addItem(buttonDontCollect);
        privacyContainer.addItem(buttonBack);

        scrollingList.addItem(mainPage);
        scrollingList.addItem(aboutPage);

        // populate dot list
        console.log(scrollingList.length);
        for (var i = 0; i < scrollingList.length; i++) {
            dotList.addItem(new Entity({
                name: 'dot',
                scale: new Vector2(0.75, 0.75),
                alpha: i === 0 ? 1 : 0.5,
                components: [ 
                    new Sprite({
                        originRelative: new Vector2(0.5, 0.5),
                        imageName: 'shop/dot'
                    })
                ]
            }));
        }
        entity.attach(dotList);

        return entity;
    };
});