/**
 * Shop overlay
 * @moduleName ShopOverlay
 * @snippet ShopOverlay.snippet
ShopOverlay({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/overlays/shop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'entities/player',
    'gui/scrollinglist',
    'gui/centeredlist',
    'gui/coincounter',
    'managers/save',
    'bento/components/eventlistener',
    'bento/components/spine',
    'components/modal',
    'globals',
    'gui/overlays/coins',
    'gui/buttons/coinsforad',
    'gui/gametext',
    'bento/components/fill',
    'entities/glassparticle',
    'gui/confetti',
    'gui/overlays/dialog',
    'modules/localization',
    'gui/loadingspinner',
    'modules/tapticengine',
    'components/bouncewithmusic',
    'gui/buttons/home',
    'modules/store',
    'modules/mopub'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Player,
    ScrollingList,
    CenteredList,
    CoinCounter,
    SaveManager,
    EventListener,
    Spine,
    Modal,
    Globals,
    CoinsOverlay,
    CoinsForAdButton,
    GameText,
    Fill,
    GlassParticle,
    Confetti,
    Dialog,
    Localization,
    LoadingSpinner,
    TapticEngine,
    BounceWithMusic,
    HomeButton,
    Store,
    MoPub
) {
    'use strict';
    var availableSkins = ['redfish'];
    var enumerateSkins = function () {
        if (availableSkins.length > 1) {
            // already enumerated
            return;
        }
        var fish = Bento.assets.getSpine('redfish');
        var skeleton = JSON.parse(fish.skeleton);
        var skins = skeleton.skins;
        Utils.forEach(skins, function (skin, skinName, l, breakLoop) {
            // ignore default
            if (skinName === 'default' || availableSkins.indexOf(skinName) >= 0) {
                return;
            }
            availableSkins.push(skinName);
        });
    };
    /**
     * Constants
     */
    var baseCosts = [500, 1000, 2000, 3000, 4000];
    var spacing = 128;
    var pageWidth = 1000 - spacing;
    var width = 3; // number of slots in the width
    var rarities = {
        "common": [
            "redfish",
            "bluefish",
            "coldfish",
            "greenfish",
            "koifish",
            "orangefish",
            "pufferfish",
            "silverfish",
            "taiyakifish"
        ],
        "rare": [
            "billybassfish",
            "chipsfish",
            "clownfish",
            "killerfish",
            "paperfish",
            "radfish",
            "rainbowfish",
            "sushifish",
            "tigerfish"
        ],
        "legendary": [
            "arowanafish",
            "cakefish",
            "deadfish",
            "flappyfish",
            "ghostfish",
            "hellfish",
            "napoleonfish",
            "robotfish",
            "spiderfish"
        ],
        "epic1": [
            "blockyfish",
            "godfish",
            "neonfish",
        ],
        "epic2": [
            "cartoonfish",
            "cyberfish",
            "maorifish",
            "narwhalfish",
            "retrofish",
            "toyfish"
        ],
    };

    var ShopOverlay = function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position;
        var target = settings.target;
        console.log('target:', target);

        var scrollIndexTarget = null;
        var listY = viewport.height * 0.54;
        var coinCounter = new CoinCounter({});
        var cost = baseCosts[0];
        var shiftUp = MoPub.isShowingBanner();
        var isUnlocking = false;
        /**
         * UI
         */
        var player = new Player({
            planet: null, // dummy player
            position: new Vector2(viewport.width * 0.5, listY - pageWidth / 2 - 256 + (Utils.isWideScreen ? 0 : 64) + (shiftUp ? -48 : 0))
        });
        var dotList = new CenteredList({
            spacing: 8,
            position: new Vector2(viewport.width * 0.5, listY + pageWidth / 2 + 64 + (shiftUp ? -96 : 0)),
            components: []
        });
        var pages = [];
        var scrollingList = new ScrollingList({
            position: new Vector2(viewport.width * 0.5, listY),
            updateWhenPaused: 0,
            // area: new Rectangle(viewport.width * 0.5 - pageWidth / 2, listY - pageWidth / 2, pageWidth, pageWidth), // area that user can grab
            direction: 'x', // 'x' or 'y'
            spacing: spacing, // additional spacing between items
            maxOffset: 0, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: true, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 128,
            onSnap: function (item, index) {
                dotList.iterate(function (dot, i) {
                    if (i === index) {
                        dot.alpha = 1;
                    } else {
                        dot.alpha = 0.5;
                    }
                });

                cost = baseCosts[index];
                // update cost text
                buttonRandom.getComponent('costText', function (costText) {
                    costText.setText(cost.toString());
                });

                item.select();

                TapticEngine.selection();

                // TODO: check if current page has any fish to unlock
            },
            onLoseFocus: function (index) {}, // called when snap focus changed
            onScrollStart: function () {}, // callback when scrolling has started
            snapMinSlideSpeed: 100, // minimum slidespeed before starting to snap
            snapSpeed: 0.2, // speed of snapping
            damping: 0, // speed decrease,
            components: [
                new EventListener({
                    eventName: 'cancelScroll',
                    onEvent: function (data) {
                        scrollingList.cancel();
                    }
                })
            ]
        });
        /**
         * Buttons
         */
        var bounceRandom = new BounceWithMusic({
            additionalScale: 0.1
        });
        var buttonRandom = new ClickButton({
            name: 'buttonRandom',
            sfx: 'sfx_button',
            imageName: 'shop/button-random',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width / 3, listY + pageWidth / 2 + 256 + (shiftUp ? -96 : 0)),
            onButtonDown: function () {
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                // var randomSlotIndex;
                var coins = SaveManager.load('coins');

                if (isUnlocking) {
                    return;
                }

                if (coins < cost) {
                    Bento.objects.attach(new Dialog({
                        name: 'notEnoughCoins',
                        text: 'Not enough coins!',
                        onComplete: function () {}
                    }));
                    return;
                }

                bounceRandom.active = false;

                isUnlocking = true;
                unlockSkinOnPage({
                    onDidSelect: function () {
                        var currentPageIndex = scrollingList.getSelection();
                        var page = scrollingList.get(currentPageIndex);

                        // spend the coins
                        coins -= cost;
                        SaveManager.save('coins', coins);
                        EventSystem.fire('updateCoins');

                        EventSystem.fire('GameAnalytics-sink', {
                            currency: 'coins',
                            cost: cost,
                            sinkType: 'randomSkin',
                            sinkId: page.getRarity()
                        });
                    },
                    onUnlock: function () {
                        isUnlocking = false;
                    }
                });
            },
            components: [
                new Text({
                    name: 'unlockRandomText',
                    position: new Vector2(0, -24),
                    text: 'UNLOCK\nRANDOM',
                    font: 'FredokaOne-Regular',
                    fontSize: 40,
                    fontColor: '#ffffff',
                    align: 'center',
                    textBaseline: 'middle',
                    sharpness: 1,
                    scale: new Vector2(4, 4),
                    shadow: true,
                    shadowColor: 'rgba(0,0,0,0.3)',
                    shadowOffset: new Vector2(4, 4),
                    maxWidth: 230,
                    maxHeight: 40 * 2,
                    components: []
                }),
                new Text({
                    name: 'costText',
                    position: new Vector2(0 + 8, 64),
                    text: cost.toString(),
                    font: 'FredokaOne-Regular',
                    fontSize: 50,
                    fontColor: '#ffffff',
                    align: 'right',
                    textBaseline: 'middle',
                    sharpness: 1,
                    scale: new Vector2(4, 4),
                    shadow: true,
                    shadowColor: 'rgba(0,0,0,0.3)',
                    shadowOffset: new Vector2(4, 4),
                    maxWidth: 80,
                    maxHeight: 50,
                    linebreaks: false,
                    components: []
                }),
                new Entity({
                    name: 'coin',
                    position: new Vector2(8 + 8, 66),
                    scale: new Vector2(0.5, 0.5),
                    components: [
                        new Sprite({
                            imageName: 'coin-icon',
                            originRelative: new Vector2(0, 0.5),
                        })
                    ]
                }),
                new EventListener({
                    name: 'eventListener',
                    eventName: 'updateCoins',
                    ignorePause: true,
                    onEvent: function (data) {
                        // enough coins to unlock?
                        bounceRandom.active = (SaveManager.load('coins') >= cost);
                    }
                }),
                bounceRandom
            ]
        });
        var buttonCoinsForAd = new CoinsForAdButton({
            position: new Vector2(viewport.width / 3 * 2, listY + pageWidth / 2 + 256 + (shiftUp ? -96 : 0)),
        });
        var adsRemainingText = new Text({
            name: 'ads',
            position: new Vector2(128, 0),
            text: '0 / 3',
            font: 'FredokaOne-Regular',
            fontSize: 100,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            sharpness: 1,
            scale: new Vector2(4, 4),
            maxWidth: 256,
            maxHeight: 100,
            linebreaks: false
        });
        var buttonAds = new ClickButton({
            name: 'ads',
            sfx: 'sfx_button',
            imageName: 'shop/button-ads',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width / 2, listY + pageWidth / 2 + 256 + (shiftUp ? -96 : 0)),
            onButtonDown: function () {
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                var onComplete = function (didComplete) {
                    // restore mute state
                    Bento.audio.muteMusic(Bento.saveState.load('muteMusic', false), true);
                    Bento.audio.muteSound(Bento.saveState.load('muteSound', false));
                    EventSystem.fire('onMute', {
                        muted: Bento.saveState.load('muteMusic', false)
                    });

                    if (!didComplete) {
                        // video was turned off early
                        return;
                    }

                    SaveManager.add('watchedAdsForSkin', 1);
                    if (SaveManager.load('watchedAdsForSkin') >= Globals.getTotalAdsForSkin(getLegendarySkinsUnlocked())) {
                        buttonAds.active = false;
                        buttonAds.visible = false;
                        // unlock skin
                        unlockSkinOnPage({
                            onUnlock: function () {
                                buttonAds.active = true;
                                buttonAds.visible = true;
                                // reset
                                SaveManager.save('watchedAdsForSkin', 0);
                                updateAdsRemaining();
                            }
                        });
                    } else {
                        updateAdsRemaining();
                    }
                    
                    EventSystem.fire('GameAnalytics-custom', {
                        id: 'shop:skinForAd',
                        value: SaveManager.load('watchedAdsForSkin')
                    });
                };
                if (Utils.isDev()) {
                    window.alert('You are now watching an ad');
                    onComplete(true);
                    return;
                }
                if (MoPub.canShowRewardVideo()) {
                    // mute before showing
                    Bento.audio.muteMusic(true);
                    MoPub.showRewardVideo(onComplete);
                } else {
                    Bento.objects.attach(new Dialog({
                        name: 'noAds',
                        text: 'Ads not loaded. Try again later.',
                        onComplete: function () {}
                    }));
                }
            },
            components: [
                adsRemainingText
            ]
        });

        var priceText = new Text({
            name: 'priceText',
            position: new Vector2(0, 0),
            text: '$9.99',
            font: 'FredokaOne-Regular',
            fontSize: 100,
            fontColor: '#ffffff',
            align: 'center',
            textBaseline: 'middle',
            sharpness: 1,
            scale: new Vector2(4, 4),
            maxWidth: 256,
            maxHeight: 100,
            linebreaks: false
        });
        var buttonPurchase = new ClickButton({
            name: 'buttonPurchase',
            sfx: 'sfx_button',
            imageName: 'shop/button-purchase',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width / 2, listY + pageWidth / 2 + 256 + (shiftUp ? -96 : 0)),
            onButtonDown: function () {
                this.transform.y = 8;
                EventSystem.fire('cancelScroll');
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                // purchase the current page
                var currentPageIndex = scrollingList.getSelection();
                var page = scrollingList.get(currentPageIndex);
                var id = page.getRarity();
                var productId = Utils.IapToId[id];

                if (SaveManager.load('purchased-' + productId)) {
                    // already purchased this
                    return;
                }

                // using bento.require here so I can refresh this module
                bento.require(['gui/overlays/iap'], function (IapOverlay) {
                    var overlay = IapOverlay.Purchase({
                        z: entity.z,
                        id: productId,
                        fontSettings: {
                            font: 'FredokaOne-Regular',
                            fontSize: 100,
                            maxWidth: viewport.width - 256
                        },
                        onPurchaseComplete: function (success, pid, purchaseInfo) {
                            if (success && pid === productId) {
                                page.unlock();
                                // make button invisible
                                buttonPurchase.visible = false;
                                buttonPurchase.active = false;
                            }
                            overlay.removeByName('loadingSpinner');
                        }
                    });
                    overlay.attach(new LoadingSpinner({
                        radius: 128,
                        fadeSpeed: 1 / 60,
                        ballsToSpawn: 8,
                        rotateBalls: false,
                        imageName: 'dot',
                        position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
                        scale: new Vector2(0.5, 0.5),
                        components: []
                    }));
                });

                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName() + ':purchase:' + page.getRarity()
                });
            },
            components: [
                priceText
            ]
        });
        var buttonHome = new HomeButton({});
        /**
         * Other components
         */
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                player.scale.x = 1.25;
                player.scale.y = 1.25;

                makePage({
                    rarity: 'common'
                });
                makePage({
                    rarity: 'rare'
                });
                makePage({
                    rarity: 'legendary'
                });

                makePage({
                    rarity: 'epic1',
                    banner: 'legendary',
                    lockPage: true,
                    dummyPrice: '$2.99',
                });
                makePage({
                    rarity: 'epic2',
                    banner: 'legendary',
                    lockPage: true,
                    dummyPrice: '$5.99',
                });

                updateAdsRemaining();

                if (scrollIndexTarget !== null) {
                    scrollingList.snapTo(scrollIndexTarget);
                }
            },
            destroy: function (data) {},
            update: function (data) {},
            draw: function (data) {}
        };
        var skinsAttached = 0;
        /**
         * Attach button to slot
         * Note: performance can be improved by disabling clickables and drawing if there are too many
         */
        var attachSkinToSlot = function (slot, type, skinIndex) {
            var skinName = rarities[type][skinIndex];
            if (!skinName) {
                // ran out of skins!
                return;
            }
            var didUnlock = isSkinUnlocked(skinName);
            var skin = new ClickButton({
                name: 'button',
                sfx: 'sfx_button',
                imageName: 'shop/skins' + (didUnlock ? '' : '-darken') + '/' + skinName,
                frameCountX: 1,
                frameCountY: 1,
                position: new Vector2(0, 0),
                updateWhenPaused: 0,
                float: false,
                onButtonDown: function () {
                    this.transform.y = 8;
                },
                onButtonUp: function () {
                    this.transform.y = 0;
                },
                onClick: function () {
                    // query again
                    if (!isSkinUnlocked(skinName)) {
                        // select bowl
                        return;
                    }
                    if (!scrollingList.didMove(2)) {
                        SaveManager.save('skin', skinName);
                        player.setSkin(skinName);
                    }
                }
            }).extend({
                unlock: function () {
                    var unlockedSkins = SaveManager.load('unlockedSkins');
                    if (unlockedSkins.indexOf(skinName) < 0) {
                        unlockedSkins.push(skinName);
                    }
                    SaveManager.save('unlockedSkins', unlockedSkins);

                    // change button sprite
                    skin.getComponent('sprite', function (sprite) {
                        sprite.setup({
                            imageName: 'shop/skins/' + skinName,
                            frameCountX: 1,
                            frameCountY: 1,
                            animations: {
                                up: {
                                    frames: [0]
                                },
                                down: {
                                    frames: [0]
                                },
                                inactive: {
                                    frames: [0]
                                }
                            }
                        });
                    });

                    // simple tween animation
                    new Tween({
                        from: 0,
                        to: 1,
                        in: 60,
                        ease: 'elastic',
                        decay: 5,
                        oscillations: 3,
                        onUpdate: function (v, t) {
                            skin.scale.x = v;
                            skin.scale.y = v;
                        },
                        onComplete: function () {
                            skin.doCallback();
                        }
                    });

                }
            });
            slot.attach(skin);

            // retrieve skin name by slot
            slot.extend({
                getSkinName: function () {
                    return skinName;
                }
            });

            // remove bowl
            if (didUnlock) {
                slot.getComponent('background', function (slotBackground) {
                    slotBackground.visible = false;
                });
            }

            // add shadow
            var shadowSprite = new Sprite({
                name: 'shadowSprite',
                imageName: 'shop/skins-shadow/' + skinName,
                originRelative: new Vector2(0.5, 0.5),
            });
            skin.attach(shadowSprite);
            skin.moveComponentTo(shadowSprite, 0);

        };
        /**
         * Create a page with a slots grid in the scrolling list
         */
        var makePage = function (pageSettings) {
            // a page is a container of 9 or fewer items
            var lockPage = pageSettings.lockPage || false;
            var pageSize = new Rectangle(0, 0, pageWidth, pageWidth);
            var slots = [];
            var banner = new Entity({
                name: 'banner',
                position: new Vector2(0, -544),
                components: [
                    new Sprite({
                        imageName: 'shop/banner-' + (pageSettings.banner || pageSettings.rarity),
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new GameText({
                        position: new Vector2(0, Utils.isCocoonJs() ? -8 : 0), // for some reason, there's an offset in native
                        text: Localization.getText(pageSettings.rarity).toUpperCase(),
                        textBaseline: 'middle',
                        fontSize: 64,
                        maxWidth: 400,
                        maxHeight: 64,
                        linebreaks: false
                    })
                ]
            });
            var skinsContainer = new Entity({
                name: 'skinsContainer',
                components: []
            });
            var page = new Entity({
                name: 'page',
                position: new Vector2(0, 32 + (shiftUp ? -96 : 0)),
                dimension: pageSize,
                scale: shiftUp ? new Vector2(0.9, 0.9) : null,
                components: [
                    banner
                ]
            }).extend({
                getSlots: function () {
                    return slots;
                },
                select: function () {
                    setBottomButtons(pageSettings);
                },
                getRarity: function () {
                    return pageSettings.rarity;
                },
                unlock: function () {
                    page.getComponent('iapPage', function (iapPage) {
                        page.remove(iapPage);
                        page.attach(skinsContainer);
                    });

                    // unlock all fish in page if needed
                    Utils.forEach(slots, function (slot, i, l, breakLoop) {
                        slot.getComponent('button', function (button) {
                            button.unlock();
                        });
                        slot.getComponent('background', function (slotBackground) {
                            // remove the bowl
                            slotBackground.visible = false;
                        });
                    });
                }
            });
            var dotSprite = new Sprite({
                originRelative: new Vector2(0.5, 0.5),
                imageName: 'shop/dot'
            });
            var addSlot = function (i, l) {
                var spacingX = pageSize.width / width;
                var spacingY = pageSize.height / width;
                var slotBackground = new Entity({
                    name: 'background',
                    // alpha: 0.75,
                    // scale: new Vector2(0.9, 0.9),
                    components: [
                        new Sprite({
                            originRelative: new Vector2(0.5, 0.5),
                            imageName: 'shop/slot'
                        })
                    ]
                });
                var slot = new Entity({
                    name: 'slot',
                    position: new Vector2(
                        (i % width) * spacingX - spacingX * (width - 1) / 2,
                        Math.floor(i / width) * spacingY - spacingY * (width - 1) / 2
                    ),
                    scale: new Vector2(1, 1),
                    components: [
                        slotBackground
                    ]
                });
                slots.push(slot);
                skinsContainer.attach(slot);

                attachSkinToSlot(slot, pageSettings.rarity, i);
                skinsAttached += 1;
            };

            /**
             * IAP
             */
            var makeIapPage = function () {
                // the page itself is also clickable
                return new ClickButton({
                    name: 'iapPage',
                    position: new Vector2(0, 0),
                    sfx: 'sfx_button',
                    imageName: 'shop/pack-' + pageSettings.rarity,
                    frameCountX: 1,
                    frameCountY: 1,
                    onButtonDown: function () {
                        this.transform.y = 8;
                    },
                    onButtonUp: function () {
                        this.transform.y = 0;
                    },
                    onClick: function () {
                        // the page must be selected
                        var currentPageIndex = scrollingList.getSelection();
                        var currentPage = scrollingList.get(currentPageIndex);
                        if (page !== currentPage) {
                            scrollingList.snapTo(pages.indexOf(page));
                            return;
                        }
                        if (!scrollingList.didMove(2)) {
                            buttonPurchase.doCallback();
                        }
                    }
                });
            };

            var pageSkins = rarities[pageSettings.rarity];
            Utils.repeat(pageSkins.length, addSlot);

            // centre the fish container vertically when there are less than 9 fish
            var spacingY = pageSize.height / width;
            var bottomY = 2 * spacingY;
            var lastFishY = Math.floor((pageSkins.length-1) / width) * spacingY;
            var diff = bottomY - lastFishY;
            skinsContainer.position.y += diff/2;


            dotList.addItem(new Entity({
                name: 'dot',
                scale: new Vector2(0.75, 0.75),
                alpha: scrollingList.length === 0 ? 1 : 0.5,
                components: [
                    dotSprite
                ]
            }));

            if (lockPage) {
                if (SaveManager.load('purchased-' + Utils.IapToId[pageSettings.rarity])) {
                    // purchased before, call the unlock method to be sure
                    page.unlock();
                    page.attach(skinsContainer);
                } else {
                    page.attach(makeIapPage());
                }
            } else {
                page.attach(skinsContainer);
            }

            pages.push(page);
            scrollingList.addItem(page);

            // scroll target?
            if (target === pageSettings.rarity) {
                scrollIndexTarget = pages.length - 1;
            }

            return page;
        };
        /**
         * Inspect locked/unlocked skins per page 
         */
        var inspectPage = function (page) {
            var slots = page.getSlots();
            var skinsInPage = [];
            var lockedSkins = [];

            // loop through its slots
            Utils.forEach(slots, function (slot, i, l, breakLoop) {
                // retrieve skin name and slot
                var skinName;
                var isUnlocked;
                if (slot.getSkinName) {
                    skinName = slot.getSkinName();
                    isUnlocked = isSkinUnlocked(skinName);

                    // collect in arrays
                    skinsInPage.push({
                        skinName: skinName,
                        slot: slot,
                        isUnlocked: isUnlocked
                    });

                    if (!isUnlocked) {
                        lockedSkins.push({
                            skinName: skinName,
                            slot: slot
                        });
                    }
                }
            });

            return {
                skinsInPage: skinsInPage,
                lockedSkins: lockedSkins
            };
        };
        /**
         * Start animation to unlock skin on a certain page
         */
        var unlockSkinOnPage = function (unlockSettings) {
            var onNoSkins = unlockSettings.onNoSkins;
            var onDidSelect = unlockSettings.onDidSelect;
            var onUnlock = unlockSettings.onUnlock;
            var onComplete = unlockSettings.onComplete;

            // unlock on current page
            var currentPageIndex = scrollingList.getSelection();
            var page = scrollingList.get(currentPageIndex);
            var pageData = inspectPage(page);
            var lockedSkins = pageData.lockedSkins;
            var onSelectedSkin = function (randomSlotIndex) {
                var slot = lockedSkins[randomSlotIndex].slot;
                var slotParent = slot.parent;
                var originalPosition = slot.position.clone();
                var worldPosition = slot.getWorldPosition();
                var originalScale = slot.scale.clone();
                var overlayFill = new Entity({
                    name: 'fill',
                    alpha: 0,
                    components: [
                        new Fill({
                            color: [0, 0, 0, 1]
                        })
                    ]
                });
                var overlay = new Entity({
                    z: entity.z + 1,
                    name: 'unlockOverlay',
                    position: new Vector2(0, 0),
                    components: [
                        new Modal({}),
                        overlayFill
                    ]
                });
                var fadeIn = function () {
                    new Tween({
                        from: 0,
                        to: 0.75,
                        in: 30,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            overlayFill.alpha = v;
                        }
                    });
                };
                var moveSlotToMiddle = function () {
                    new Tween({
                        from: slot.position,
                        to: viewport.getExtents(),
                        in: 40,
                        ease: 'easeInOutBack',
                        onUpdate: function (v, t) {
                            slot.position = v;
                        },
                        onComplete: function () {

                        }
                    });
                    new Tween({
                        from: slot.scale,
                        to: new Vector2(2, 2),
                        in: 40,
                        onStart: function () {
                            Bento.audio.playSound('sfx_zoom');
                        },
                        onUpdate: function (v, t) {
                            slot.scale = v;
                        },
                        onComplete: shake
                    });
                };
                var shake = function () {
                    new Tween({
                        from: 0,
                        to: 32,
                        in: 60,
                        ease: 'quadratic',
                        onStart: function () {
                            Bento.audio.playSound('sfx_unlock');
                        },
                        onUpdate: function (v, t) {
                            slot.transform.x = Utils.getRandomRange(-v, v);
                            slot.transform.y = Utils.getRandomRange(-v, v);
                        },
                        onComplete: function () {
                            slot.transform.x = 0;
                            slot.transform.y = 0;
                            breakOpen();
                        }
                    });
                };
                var breakOpen = function () {
                    var popSprite = new Sprite({
                        imageName: 'balloon-pop',
                        originRelative: new Vector2(0.5, 0.5)
                    });
                    var pop = new Entity({
                        name: 'pop',
                        components: [
                            popSprite
                        ]
                    });
                    slot.attach(pop);
                    new Tween({
                        from: 0,
                        to: 2,
                        in: 30,
                        ease: 'exponential',
                        decay: 5,
                        onUpdate: function (v, t) {
                            pop.scale.x = v;
                            pop.scale.y = v;
                        },
                        onComplete: function () {
                            pop.removeSelf();
                            Utils.timeout(30, revert);
                        }
                    });
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 30,
                        ease: 'exponential',
                        decay: 5,
                        onUpdate: function (v, t) {
                            pop.alpha = v;
                        }
                    });

                    // Bento.audio.playSound('sfx_bump');
                    // Bento.audio.playSound('sfx_glass');

                    slot.getComponent('button', function (button) {
                        button.unlock();
                    });
                    slot.getComponent('background', function (slotBackground) {
                        // remove the bowl
                        slotBackground.visible = false;
                    });

                    overlay.attach(new Confetti({
                        speedY: 4,
                        particleScale: 0.5,
                        duration: 40
                    }));

                    // glass
                    Utils.repeat(10, function () {
                        overlay.attach(new GlassParticle(slot));
                    });

                    TapticEngine.impact({
                        type: "heavy"
                    });

                    if (onUnlock) {
                        onUnlock();
                    }
                };
                var revert = function () {
                    // remove overlay
                    new Tween({
                        from: overlayFill.alpha,
                        to: 0,
                        in: 30,
                        onUpdate: function (v, t) {
                            overlayFill.alpha = v;
                        }
                    });
                    // put slot back
                    new Tween({
                        from: slot.scale,
                        to: originalScale,
                        in: 30,
                        ease: 'linear',
                        onUpdate: function (v, t) {
                            slot.scale = v;
                        }
                    });
                    new Tween({
                        from: slot.position,
                        to: worldPosition,
                        in: 30,
                        ease: 'easeOutBack',
                        onUpdate: function (v, t) {
                            slot.position = v;
                        },
                        onComplete: function () {
                            slot.removeSelf();
                            slotParent.attach(slot);
                            slot.position = originalPosition.clone();

                            overlay.removeSelf();

                            // reactivate
                            slot.getComponent('button', function (button) {
                                button.active = true;
                            });
                            scrollingList.active = true;

                            if (onComplete) {
                                onComplete();
                            }
                        }
                    });
                };

                Bento.objects.attach(overlay);

                // detach slot from parent
                slot.removeSelf();
                // attach to slot
                overlay.attach(slot);
                slot.position = worldPosition.clone();

                slot.getComponent('button', function (button) {
                    button.active = false;
                });

                fadeIn();
                moveSlotToMiddle();

                if (onDidSelect) {
                    onDidSelect();
                }
            };

            // is there anything to unlock?
            if (lockedSkins.length) {
                if (lockedSkins.length === 1) {
                    // no point in doing the random thing, there is only 1 skin left
                    onSelectedSkin(0);
                } else {
                    // do random selection animation
                    randomSelectionAnimation(lockedSkins, onSelectedSkin);
                    // lock scrolling
                    scrollingList.active = false;
                }

                EventSystem.fire('GameAnalytics-custom', {
                    id: 'shop:unlockSkin:'+page.getRarity()
                });

            } else {
                // Note: should have removed this button before
                if (onNoSkins) {
                    onNoSkins();
                }
            }
        };
        /**
         * Slot random selection animation
         */
        var randomSelectionAnimation = function (skinDataArray, callback) {
            var length = skinDataArray.length;
            var startOffset = Utils.getRandom(length);
            var lockedSkins = skinDataArray;
            var index;
            var glow = new Entity({
                name: 'selection',
                components: [
                    new Sprite({
                        imageName: 'shop/slot-glow',
                        originRelative: new Vector2(0.5, 0.5),
                    })
                ]
            });
            var previousSlot = null;
            var onUpdate = function (v, t) {
                var newIndex = Math.floor(v) % length;
                if (index !== newIndex) {
                    index = newIndex;
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_button');
                }
                var slot = skinDataArray[index].slot;
                glow.removeSelf();
                slot.attach(glow);

                if (slot !== previousSlot) {
                    slot = previousSlot;
                    // TapticEngine.selection();
                }

                // keep the fish moving
                if (Bento.isPaused(player)) {
                    var gameData = Bento.getGameData();
                    gameData.deltaT = 16;
                    player.update(gameData);
                }
            };

            // start a tween
            new Tween({
                from: startOffset,
                to: startOffset + 16 + Utils.getRandom(16),
                in: 60 * 2,
                ease: 'cuberoot',
                onUpdate: onUpdate,
                onComplete: function () {
                    this.removeByName('modal');
                    // a few more steps, slowly!
                    new Tween({
                        from: index,
                        to: index + 3 + Utils.getRandom(2),
                        in: 90,
                        ease: 'linear',
                        onUpdate: onUpdate,
                        onComplete: function () {
                            this.removeByName('modal');
                            new Tween({
                                from: glow.alpha,
                                to: 0,
                                in: 30,
                                onUpdate: function (v, t) {
                                    glow.alpha = v;
                                },
                                onComplete: function () {
                                    glow.removeSelf();
                                    callback(index);
                                }
                            });
                        }
                    }).attach(new Modal({}));
                }
            }).attach(new Modal({}));
        };
        /**
         * Query unlocked skin
         */
        var isSkinUnlocked = function (skinName) {
            var unlocked = SaveManager.load('unlockedSkins');
            return unlocked.indexOf(skinName) >= 0;
        };
        /**
         * Buttons may change depending on the situation
         */
        var setBottomButtons = function (pageSettings) {
            var rarity = pageSettings.rarity;
            var dummyPrice = pageSettings.dummyPrice || 'n/a';

            var removeAllButtons = function () {
                buttonRandom.visible = false;
                buttonRandom.active = false;

                buttonCoinsForAd.visible = false;
                buttonCoinsForAd.active = false;

                buttonAds.visible = false;
                buttonAds.active = false;

                buttonPurchase.visible = false;
                buttonPurchase.active = false;
            };
            switch (rarity) {
            case 'rare':
            case 'common':
                removeAllButtons();
                if (!buttonRandom.active) {
                    buttonRandom.active = true;
                    buttonRandom.visible = true;
                }
                if (!buttonCoinsForAd.active) {
                    buttonCoinsForAd.active = true;
                    buttonCoinsForAd.visible = true;
                }

                // enough coins to unlock?
                bounceRandom.active = (SaveManager.load('coins') >= cost);

                break;
            case 'legendary':
                removeAllButtons();

                if (!buttonAds.active) {
                    buttonAds.active = true;
                    buttonAds.visible = true;
                }
                break;
            case 'epic1':
            case 'epic2':
                removeAllButtons();

                var currentPageIndex = scrollingList.getSelection();
                var page = scrollingList.get(currentPageIndex);
                var id = page.getRarity();
                var productId = Utils.IapToId[id];

                if (SaveManager.load('purchased-' + productId)) {
                    // already purchased this
                    return;
                }

                var product = Store.getProduct(productId);
                priceText.setText(product ? product.localizedPrice : dummyPrice);

                if (!buttonPurchase.active) {
                    buttonPurchase.active = true;
                    buttonPurchase.visible = true;
                }
                break;
            default:
                // no buttons at all 
                removeAllButtons();
                break;
            }
        };
        /**
         * Update ads remaining text
         */
        var updateAdsRemaining = function () {
            var adsWatched = SaveManager.load('watchedAdsForSkin');
            var legendarySkinsUnlocked = getLegendarySkinsUnlocked();
            var adsNeeded = Globals.getTotalAdsForSkin(legendarySkinsUnlocked);

            var adsLeft = adsNeeded - (adsWatched % adsNeeded);

            adsRemainingText.setText(
                Localization.getText('XadsLeft').replace(
                    '{NUMBER}',
                    adsLeft.toString()
                )
            );
        };
        var getLegendarySkinsUnlocked = function () {
            var unlocked = 0;
            var skins = rarities.legendary;
            Utils.forEach(skins, function (skin, i, l, breakLoop) {
                if (isSkinUnlocked(skin)) {
                    unlocked += 1;
                }
            });
            return unlocked;
        };
        /**
         * Container entity
         */
        var entity = new Entity({
            z: 0,
            name: 'shopOverlay',
            position: position,
            float: true,
            components: [
                behavior,
                player,
                scrollingList,
                dotList,
                coinCounter,
                buttonRandom,
                buttonCoinsForAd,
                buttonHome,
                buttonAds,
                buttonPurchase
            ]
        });

        enumerateSkins();
        setBottomButtons({
            rarity: 'common'
        });

        return entity;
    };

    // expose skins
    ShopOverlay.rarities = rarities;
    ShopOverlay.getCost = function (rarity) {
        if (rarity === 'common') {
            return baseCosts[0];
        } else if ('rare') {
            return baseCosts[1];
        }
    };

    ShopOverlay.getRarityBySkin = function (skin) {
        return skinToRarity[skin];
    };

    var skinToRarity = {};
    for (var k in rarities) {
        var skinList = rarities[k];
        for (var i in skinList) {
            skinToRarity[skinList[i]] = k;
        }
    }

    ShopOverlay.hasPurchased = function (rarity) {
        return SaveManager.load('purchased-' + Utils.IapToId[rarity]);
    };

    return ShopOverlay;
});