/**
 * Pulsing speaker!
 * @moduleName Speaker
 * @snippet Speaker.snippet
Speaker({
    position: new Vector2(0, 0)
})
 */
bento.define('gui/speaker', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position || new Vector2(viewport.width * 0.5, viewport.height / 3 * 2);
        var behavior = {
            start: function () {
                EventSystem.on('songBeat', onSongBeat);
            },
            destroy: function () {
                EventSystem.off('songBeat', onSongBeat);
            }
        };
        var speaker = new Entity({
            z: settings.z || 0,
            position: position,
            scale: new Vector2(1.3, 1.3),
            alpha: 0.08,
            components: [
                behavior,
                new Sprite({
                    originRelative: Vector2(0.5, 0.5),
                    imageName: settings.imageName || 'title/speaker',
                })
            ]
        });
        var onSongBeat = function () {
            new Tween({
                from: 1.4,
                to: 1.3,
                in: 32,
                ease: 'linear',
                onUpdate: function (v, t) {
                    speaker.scale.x = speaker.scale.y = v;
                }
            });
        };

        return speaker;
    };
});