/**
 * Text that says tap to start!
 * @moduleName TapToStart
 * @snippet TapToStart.snippet
TapToStart({})
 */
bento.define('gui/taptostart', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'bento/components/eventlistener',
    'modules/localization',
    'components/bouncewithmusic'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    EventListener,
    Localization,
    BounceWithMusic
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position || Vector2(0, 0);

        var onPlayerDead = function () {
            // text.visible = true;
        };

        var onTap = function () {
            if (text.visible) {
                text.visible = false;
                EventSystem.fire('startLevel');
            }
        };
        
        var behavior = {
            name: 'behaviorComponent',
            start: function (data) {
                EventSystem.on('playerDead', onPlayerDead);
            },
            destroy: function (data) {
                EventSystem.off('playerDead', onPlayerDead);
            },
            update: function (data) {},
            draw: function (data) {}
        };
        var text = new Text({
            z: Globals.Layers.GUI,
            position: position,
            name: 'tapToStart',
            text: Localization.getText('touchToStart'),
            font: 'FredokaOne-Regular',
            fontSize: 100,
            fontColor: '#fff',
            align: 'center',
            textBaseline: 'middle',
            sharpness: 1,
            scale: Vector2(4, 4),
            ySpacing: 0,
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(8, 8),
            maxWidth: undefined,
            maxHeight: undefined,
            linebreaks: true,
            drawDebug: false,
            float: true,
            // global: true,
            components: [
                behavior,
                new EventListener({
                    name: 'eventListener',
                    eventName: 'pointerDown',
                    ignorePause: false,
                    onEvent: onTap
                }),
                new EventListener({
                    name: 'eventListener',
                    eventName: 'buttonDown-enter',
                    ignorePause: false,
                    onEvent: onTap
                }),
                new BounceWithMusic({
                    additionalScale: 0.1
                })
            ]
        });
        return text;
    };
});