/**
 * Star to be shown on the win screen
 * If the starNumber <= playerScore, it will play a highlight animation
 * If it's the final star to be highlighted, it will call the onFinalStar callback
 * @moduleName WinStar
 * @snippet WinStar.snippet
WinStar({
    starNumber: 1,
    playerScore: score,
    position: new Vector2(0, 0),
    rotation: 0,
    onHighlight: function () {}
})
 */
bento.define('gui/winstar', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var score = settings.playerScore;
        var starNumber = settings.starNumber;
        var onFinalStar = settings.onFinalStar;
        var onHighlight = settings.onHighlight;

        var star = new Entity({
            z: 0,
            position: settings.position,
            rotation: settings.rotation,
            components: [
                new Sprite({
                    imageName: 'ui/star-grey',
                    originRelative: new Vector2(0.5, 0.5),
                })
            ]
        });

        // light up the star 0.5 seconds after the previous
        if (score >= starNumber) {
            Utils.timeout(
                30 * starNumber,
                function () {
                    star.getComponent('sprite', function (sprite) {
                        sprite.setup({
                            imageName: 'ui/star-yellow',
                        });
                    });
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 30,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            star.scale.x = star.scale.y = 1 + v*0.8;
                            // star.rotation = angle + Math.PI * v * 2/3;
                        }
                    });
                    Bento.audio.playSound('sfx_addstar'+starNumber);
                    if (onHighlight) {
                        onHighlight();
                    }
                }
            );
        }

        return star;
    };
});
