/**
 * AppsFlyer Cordova SDK wrapper for Bento
 * For documentation, see 
 *    https://github.com/AppsFlyerSDK/cordova-plugin-appsflyer-sdk/tree/releases/4.x.x/4.3.x/4.3.0_cordova_android_6
 * @moduleName AppsFlyer
 */
bento.define('modules/appsflyer', [
    'bento',
    'bento/eventsystem',
    'bento/utils'
], function (
    Bento,
    EventSystem,
    Utils
) {
    'use strict';

    var doNothing = function () {};

    var module = {
        init:                          null,
        trackEvent:                    doNothing,
        setDeviceTrackingDisabled:     doNothing,
        setCurrencyCode:               doNothing,
        setAppUserId:                  doNothing,
        enableUninstallTracking:       doNothing,
        updateServerUninstallToken:    doNothing,
        getAppsFlyerUID:               doNothing,
        setAppInviteOneLinkID:         doNothing,
        generateInviteLink:            doNothing,
        trackCrossPromotionImpression: doNothing,
        trackAndOpenStore:             doNothing,
        handleOpenUrl:                 doNothing,
    };

    var appsFlyer;

    // settings:
    //   devKey: string   // your AppsFlyer devKey 
    //   appId: string    // only needed for iOS
    //   isDebug: boolean
    //   onInstallConversionDataListener: boolean

    module.init = function (settings) {

        if (!(Utils.isNativeIos() || Utils.isNativeAndroid())) {
            console.log('Skipping AppsFlyer initialization');
            return;
        }

        appsFlyer = window.plugins && window.plugins.appsFlyer;
        if (!appsFlyer) {
            Utils.log('ERROR: Could not get appsFlyer SDK!');
            return;
        }

        // appsflyer options
        var options = {
            devKey: settings.devKey,
            isDebug: settings.isDebug || false,
            onInstallConversionDataListener: settings.onInstallConversionDataListener || false
        };

        if (Utils.isNativeIos()) {
            options.appId = settings.appId;
        }

        appsFlyer.initSdk(options, onInitSuccess, onInitError);
    };

    var onInitSuccess = function (message) {

        module.trackEvent = function (eventName, eventValues) {
            appsFlyer.trackEvent(eventName, eventValues);
        };

        // pass true to opt-out user from appsflyer tracking
        module.setDeviceTrackingDisabled = function (flag) {
            appsFlyer.setDeviceTrackingDisabled(flag);
        };

        // set currency e.g. "USD" "GBP"
        module.setCurrencyCode = function (currency) {
            appsFlyer.setCurrencyCode(currency);
        };

        // custom ID allows you to match with your own unique ID, for cross-referencing
        // if you are using this, be sure to call as early as possible during intialization
        module.setAppUserId = function (customUserId) {
            appsFlyer.setAppUserId(customUserId);
        };

        // enables app uninstall tracking
        // - token     : string   // GCM/FCM ProjectNumber
        // - onSuccess : function(string)
        // - onError   : function(string)
        module.enableUninstallTracking = function (token, onSuccess, onError) {
            appsFlyer.enableUninstallTracking(token, onSuccess, onError);
        };

        // allows to pass GCM/FCM tokens collected by third party plugins to the AppsFlyer server
        // - token : GCM/FCM ProjectNumber
        module.updateServerUninstallToken = function (token) {
            appsFlyer.updateServerUninstallToken(token);
        };

        // (advanced) get proprietary appsFlyer device ID
        // - onSuccess : function(string)
        module.getAppsFlyerUID = function (onSuccess) {
            appsFlyer.getAppsFlyerUID(onSuccess);
        };

        // set this to enable shorter User Invite links
        // - OneLinkID : string  // can be obtained from AppsFlyer dashboard
        module.setAppInviteOneLinkID = function (OneLinkID) {
            appsFlyer.setAppInviteOneLinkID(OneLinkID);
        };

        // Invite link allows users to invite their friends as new users to your app.
        // - options   : object  // see https://github.com/AppsFlyerSDK/cordova-plugin-appsflyer-sdk#-generateinvitelinkoptions-onsuccess-onerror-void--user-invite
        // - onSuccess : function(string)
        // - onError   : function(string)
        module.generateInviteLink = function (options, onSuccess, onError) {
            appsFlyer.generateInviteLink(options, onSuccess, onError);
        };

        // Track an impression
        // - appID    : string  // promoted application ID
        // - campaign : string  // promoted campaign 
        module.trackCrossPromotionImpression = function (appID, campaign) {
            appsFlyer.trackCrossPromotionImpression(appID, campaign);
        };

        // Launch the app's appstore page via browser
        // - appID    : string  // promoted application ID
        // - campaign : string  // promoted campaign
        // - options  : object  // see https://github.com/AppsFlyerSDK/cordova-plugin-appsflyer-sdk#-trackandopenstoreappidcampaign-options-void--cross-promotion
        module.trackAndOpenStore = function (appID, campaign, options) {
            appsFlyer.trackAndOpenStore(appID, campaign, options);
        };


        // call this on iOS to track deep links.
        module.handleOpenUrl = function (url) {
            appsFlyer.handleOpenUrl(url);
        };
    };

    var onInitError = function (message) {
        Utils.log('ERROR: Could not initialize AppsFlyer SDK. Message: ' + message);
    };

    return module;
});