/**
 * @moduleName GameCenter
 */
bento.define('modules/gamecenter', [], function () {
    'use strict';
    var device = null,
        isLoggedIn = function () {
            return window.Cocoon.Social.GameCenter.isLoggedIn();
        },
        nothing = function () {
            return;
        },
        /**
         * @snippet GameCenter.login()|snippet
GameCenter.login(function (loggedIn, error) {
    if (error) {
        console.error("Error: " + error.message);
    }
    $1
});
         */
        login = function (onComplete) {
            window.Cocoon.Social.GameCenter.login(function (response, error) {
                if (error) {
                    // alert(error);
                    console.log('GC error', error.message);
                    if (onComplete) {
                        onComplete(false, error.message);
                    }
                    return;
                } else {
                    console.log('GC response', JSON.stringify(response));
                    if (onComplete) {
                        onComplete(response.isAuthenticated, response);
                    }
                }
            });
        },
        /**
         * @snippet GameCenter.post()|snippet
        GameCenter.post(${1:0}, '${2:leaderboardId}');
         */
        post = function (score, leaderboardId, onComplete) {
            if (!isLoggedIn()) {
                return;
            }
            var socialService = window.Cocoon.Social.GameCenter.getSocialInterface();
            socialService.submitScore(score, onComplete, {
                leaderboardID: leaderboardId
            });

            // var scoreGC = new window.Cocoon.Social.GameCenter.Score(score, leaderboardId);
            // window.Cocoon.Social.GameCenter.submitScore(scoreGC, onComplete);
        },
        /**
         * @snippet GameCenter.show()|snippet
        GameCenter.show('${1:leaderboardId}');
         */
        show = function (id) {
            window.Cocoon.Social.GameCenter.showLeaderboards(nothing, {
                identifier: id,
                category: id
            });
        },
        /**
         * @snippet GameCenter.postAchievement()|snippet
        GameCenter.postAchievement('${1:achievementId}');
         */
        postAchievement = function (id, onComplete) {
            var socialService = window.Cocoon.Social.GameCenter.getSocialInterface();
            socialService.submitAchievement(id, function (response) {
                if (onComplete) {
                    onComplete(response);
                }
            });
        },
        /**
         * @snippet GameCenter.showAchievement()|snippet
        GameCenter.showAchievement(function () {});
         */
        showAchievement = function (onComplete) {
            window.Cocoon.Social.GameCenter.showAchievements(onComplete);
        };

    // ios?
    if (
        navigator.isCocoonJS &&
        window.Cocoon &&
        window.Cocoon.Social &&
        window.Cocoon.Social.GameCenter &&
        window.Cocoon.getPlatform() === 'ios'
    ) {
        return {
            isLoggedIn: isLoggedIn,
            login: login,
            post: post,
            show: show,
            postAchievement: postAchievement,
            showAchievement: showAchievement,
            loadSavedGame: function (name, onSuccess, onFail) {
                if (!isLoggedIn()) {
                    return;
                }
                window.Cocoon.Social.GameCenter.loadSavedGame(name, function (snapshot) {
                    if (onSuccess) {
                        if (snapshot.data) {
                            onSuccess(snapshot);
                        } else {
                            onFail(snapshot);
                        }
                    }
                }, function (error) {
                    if (onFail) {
                        onFail(error);
                    }
                });
            },
            writeSavedGame: function (snapshot, onSuccess, onFail) {
                if (!isLoggedIn()) {
                    return;
                }
                /*snapshot {
                    identifier: string,
                    title: string,
                    description: string,
                    data: string //this is the JSON you want to load/store
                }*/
                window.Cocoon.Social.GameCenter.writeSavedGame(snapshot.identifier, snapshot.data, function () {
                    if (onSuccess) {
                        onSuccess();
                    }                    
                }, function (error) {
                    if (error) {
                        onFail(error);
                    }
                });
            },
        };
    } else {
        return {
            isLoggedIn: function () {
                return false;
            },
            login: nothing,
            post: nothing,
            show: nothing,
            postAchievement: nothing,
            showAchievement: nothing
        };
    }
});