/**
 * @moduleName GooglePlay
 */
bento.define('modules/googleplay', [
    'bento'
], function (
    Bento
) {
    'use strict';
    var gp;
    var clientId = '1090872060639-jng9fid41eqdcfktm7pop963b3jo30m1.apps.googleusercontent.com';
    var initialized = false;
    var isInitializing = false;
    var nothing = function () {};
    var init = function (onComplete) {
        if (isInitializing) {
            // already trying to initialize
            return;
        }
        isInitializing = true;
        // set default player stats if none are available (eg. on first bootup or iOS)
        if (!Bento.saveState.load('playerStats', null)) {
            Bento.saveState.save('playerStats', {
                averageSessionLength: -1, // -1 is the default value when there's not enough data
                churnProbability: -1,
                daysSinceLastPlayed: -1,
                highSpenderProbability: -1,
                numberOfPurchases: -1,
                numberOfSessions: -1,
                sessionPercentile: -1,
                spendPercentile: -1,
                spendProbability: -1,
                totalSpendNext28Days: -1
            });
        }

        gp = window.Cocoon.Social.GooglePlayGames;
        gp.init({
            clientId: clientId,
            defaultLeaderboard: 'CgkI39Wq6N8fEAIQAA',
            showAchievementNotifications: true,
            /**
             * WARNING: Be sure to also enable the scopes on console.developers.google.com,
             * otherwise you will get an 'INVALID_SCOPE' login error.
             */
            scopes: [
                // 'https://www.googleapis.com/auth/drive.appdata',
                // 'appfolder',
                // 'drive.appfolder',
                // 'drive.appdata'
            ]
        }, function (error) {
            isInitializing = false;
            initialized = true;
            if (onComplete) {
                onComplete(error);
            }
        });
    };
    var isLoggedIn = function () {
        if (!initialized) {
            return false;
        }
        var socialService = gp.getSocialInterface();
        console.log('Google Play: is logged in', socialService.isLoggedIn());
        return socialService.isLoggedIn();
    };
    var login = function (onComplete) {
        var socialService;
        onComplete = onComplete || function () {};
        socialService = gp.getSocialInterface();
        socialService.login(onComplete);
    };
    if (!window.Cocoon || !window.Cocoon.Social || !window.Cocoon.Social.GooglePlayGames) {
        return {
            init: nothing,
            isLoggedIn: nothing,
            login: nothing,
            post: nothing,
            show: nothing,
            postAchievement: nothing,
            showAchievement: nothing,
            submitEvent: nothing,
            loadSavedGame: nothing,
            writeSavedGame: nothing,
            showSavedGame: nothing,
            loadPlayerStats: nothing
        };
    }

    return {
        init: function (onComplete) {
            if (!initialized) {
                init(onComplete);
            }
        },
        isLoggedIn: isLoggedIn,
        login: function (onComplete) {
            var socialService,
                self = this;

            if (!initialized) {
                init(function (error) {
                    self.login(onComplete);
                });
                return;
            }

            socialService = gp.getSocialInterface();
            if (socialService.isLoggedIn()) {
                // dont log in twice
                if (onComplete) {
                    onComplete(false);
                }
                return;
            }
            socialService.login(function (loggedIn, error) {
                if (error) {
                    if (onComplete) {
                        onComplete(false, error);
                    }
                    console.error("login error: " + error.message + " " + error.code);
                } else {
                    if (onComplete) {
                        onComplete(loggedIn);
                    }
                }
            });
        },
        post: function (v, leaderboardId, onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitScore(v, onComplete, {
                    leaderboardID: leaderboardId
                });
            }
        },
        show: function (param) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();

            if (!isLoggedIn()) {
                login(function () {
                    console.log('logged in');
                    socialService.showLeaderboard(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    }, param);
                });
            } else {
                socialService.showLeaderboard(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                }, param);
            }

        },
        postAchievement: function (id, onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitAchievement(id, function (response) {
                    console.log('achievement unlocked', response);
                    if (response && response.newlyUnlocked && onComplete) {
                        // show message
                        onComplete();
                    }
                });
            }
        },
        showAchievement: function (onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (!isLoggedIn()) {
                login(function () {
                    socialService.showAchievements(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    });
                });
            } else {
                socialService.showAchievements(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                });

            }
        },
        submitEvent: function (name, amount) {
            if (!initialized) {
                return;
            }
            if (window.Cocoon.Social.GooglePlayGames.submitEvent) {
                window.Cocoon.Social.GooglePlayGames.submitEvent(name, amount);
            } else {
                console.log('submitEvent not available (cocoon devapp?)');
            }
        },
        loadSavedGame: function (name, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.loadSavedGame(name, function (snapshot, error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    // console.log('load snapshot', JSON.stringify(snapshot));
                    if (snapshot.data) {
                        onSuccess(snapshot);
                    } else {
                        onFail(snapshot);
                    }
                }
            });
        },
        writeSavedGame: function (snapshot, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            /*snapshot {
                identifier: string,
                title: string,
                description: string,
                data: string //this is the JSON you want to load/store
            }*/
            window.Cocoon.Social.GooglePlayGames.writeSavedGame(snapshot, function (error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    onSuccess();
                }
            });
        },
        showSavedGame: function (snapshot) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.showSavedGames(snapshot, function (error) {

            });
        },
        loadPlayerStats: function (onComplete) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.loadPlayerStats(function (stats, error) {
                if (stats) {
                    Bento.saveState.save('playerStats', stats);

                    if (onComplete) {
                        onComplete(stats);
                    }
                }
                if (error) {
                    console.error(error);

                    if (onComplete) {
                        onComplete(null);
                    }
                }
            });
        },
        /**
         * Loads the scores around a player for the specified leaderboard id.
         *
         * @param {String} leaderboardID - The leaderboard identifier to get score from.
         * @param {Boolean} topScores - If true, will load top scores. If false, will load scores centered around user. Defaults to true.
         * @param {Number} timeSpan - For which time period to get scores. Use 0 for daily scores, 1 for weekly scores, 2 for scores since the start of the leaderboard. Defaults to 2.
         * @param {Boolean} friends - If true, will load scores from user's social circles. If false, will load general scores. Defaults to false.
         * @param {Function} onComplete - The callback function. It receives the following parameters:
         * @param {Object[]} scores - Array of objects containing leaderboard info about players. Example:
         *                            [{
         *                                "id": 123456789098765432101,
         *                                "name": "Johnny B. Goode",
         *                                "rank": 1,
         *                                "score": 1234
         *                            }, {
         *                                ...
         *                            },
         *                            ...
         *                            ]
         */
        loadScores: function (leaderboardId, topScores, timeSpan, friends, onComplete) {
            if (!initialized) {
                return;
            }

            topScores = topScores === false ? false : true;
            timeSpan = timeSpan === 0 ? 0 : (timeSpan || 2);
            friends = friends ? true : false;

            var cordovaCallback = function (scores, error) {
                if (scores) {

                    // TODO: save the leaderboard using Bento.saveState?

                    if (onComplete) {
                        onComplete(scores);
                    }
                }
                if (error) {
                    if (onComplete) {
                        onComplete(null);
                    }
                }
            };

            if (topScores) {
                window.Cocoon.Social.GooglePlayGames.loadTopScores(leaderboardId, timeSpan, friends, cordovaCallback);
            } else {
                window.Cocoon.Social.GooglePlayGames.loadPlayerCenteredScores(leaderboardId, timeSpan, friends, cordovaCallback);
            }
        }
    };
});