/**
 * Shows exp and unlocked items
 */
bento.define('screens/expscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'bento/components/spine',
    'gui/speaker',
    'modules/expitems',
    'gui/gametext',
    'globals',
    'bento/components/fill',
    'gui/experiencebar',
    'entities/scoremanager',
    'gui/coincounter',
    'modules/localization',
    'gui/buttons/home'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Spine,
    Speaker,
    ExpItems ,
    GameText,
    Globals,
    Fill,
    ExperienceBar,
    ScoreManager,
    CoinCounter,
    Localization,
    HomeButton
) {
    'use strict';
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var background = new Entity({
            z: 0,
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'shop/background',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });

        var speaker = new Speaker({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5)
        });

        var buttonHome = new HomeButton({});

        var ItemPanel = function (panelSettings) {
            var item = panelSettings.item;
            var position = panelSettings.position;

            var text, iconImage, bannerImage;
            if (item.isUnlocked()) {
                text = item.values[item.timesUnlocked-1] + '\n' + item.text;
                iconImage = item.icon;
                bannerImage = 'xp-ui/banner';
            } else {
                text = Localization.getText('levelToUnlock').replace('{NUMBER}', nextUnlockLevel).toUpperCase();
                iconImage = 'xp-ui/button-lock';
                bannerImage = 'xp-ui/banner-locked';
            }

            var panel = new Entity({
                position: position,
                components: [
                    new Sprite({
                        imageName: bannerImage,
                        originRelative: Vector2(0.5, 0)
                    }),
                    new Entity({
                        position: new Vector2(-380, 90),
                        scale: new Vector2(0.80, 0.80),
                        components: [
                            new Sprite({
                                imageName: iconImage,
                                originRelative: Vector2(0.5, 0.5)
                            })
                        ]
                    }),
                    new GameText({
                        position: new Vector2(66, 80),
                        text: text,
                        fontSize: 36,
                        maxWidth: 700,
                        maxHeight: 160,
                        textBaseline: 'middle',
                        // drawDebug: true
                    })
                ]
            });
            return panel;
        };

        var coinCounter = new CoinCounter({});

        var experienceBar = new ExperienceBar({
            position: new Vector2(viewport.width/2, 320),
            showLevel: true
        });

        var panelContainer = new Entity({
            z: Globals.Layers.GUI,
            name: 'panelContainer',
            position: new Vector2(viewport.width / 2 + 22, 580),
            components: []
        });

        var itemNum = 0;
        var nextUnlockLevel = ScoreManager.getExpLevel();

        ExpItems.forEach(function (item) {
            if (!item.isUnlocked()) {
                nextUnlockLevel++;
            }
            if (item.hidden) {
                return;
            }
            var panel = new ItemPanel({
                position: new Vector2(0, itemNum++ * 240),
                item: item,
            });
            panelContainer.attach(panel);
        });

        background.scale.x = viewport.width / background.dimension.width * 1.1;
        background.scale.y = viewport.height / background.dimension.height;
        Bento.objects.attach(background);
        Bento.objects.attach(speaker);
        Bento.objects.attach(buttonHome);
        Bento.objects.attach(coinCounter);
        Bento.objects.attach(experienceBar);
        Bento.objects.attach(panelContainer);
    };

    var onHide = function () {

    };

    return new Screen({
        onShow: onShow,
        onHide: onHide
    });
});