/**
 * Victory! Next level, home, retry, etc.
 */
bento.define('screens/failscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/player',
    'gui/gametext',
    'globals',
    'gui/speaker'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Player,
    GameText,
    Globals,
    Speaker
) {
    'use strict';

    var onShow = function (settings) {

        var viewport = Bento.getViewport();
        
        var background = new Entity({
            z: -3,
            position: new Vector2(0, 0),
            scale: new Vector2(100, 1),
            components: [
                new Sprite({
                    imageName: 'ui/bg-failscreen',
                })
            ]
        });
        Bento.objects.attach(background);

        Bento.objects.attach(new Speaker({ 
            z: -2,
            imageName: 'ui/failscreen-speaker'
        }));

        var makeStar = function (x, y, angle, n) {
            var star = new Entity({
                z: 0,
                position: new Vector2(x, y),
                rotation: angle,
                components: [
                    new Sprite({
                        imageName: 'ui/star-grey',
                        originRelative: new Vector2(0.5, 0.5),
                    })
                ]
            });
            Bento.objects.attach(star);
        };

        makeStar(0.5*viewport.width - 250, 0.5*viewport.height - 310, -0.7, 1);
        makeStar(0.5*viewport.width, 0.5*viewport.height - 390, 0, 2);
        makeStar(0.5*viewport.width + 250, 0.5*viewport.height - 310, 0.7, 3);

        var player = new Player({
            planet: null, // dummy player
            sad: true,
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5)
        });
        Bento.objects.attach(player);

        Bento.objects.attach(new GameText({
            position: Vector2(0.5 * viewport.width, 0.1 * viewport.height),
            text: 'fail',
            fontSize: 140,
        }));

        var btnReplay = new ClickButton({
            z: 0,
            name: 'replay',
            sfx: 'sfx_button',
            imageName: 'ui/button-replay',
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.74),
            float: true,
            scale: Vector2(0.9, 0.9),
            onClick: function () {
                Bento.screens.show('screens/main');
            }
        });
        Bento.objects.attach(btnReplay);

        var btnHome = new ClickButton({
            z: 0,
            name: 'home',
            sfx: 'sfx_button',
            imageName: 'ui/button-home',
            position: new Vector2(viewport.width * 0.15, viewport.height * 0.92),
            float: true,
            scale: Vector2(0.5, 0.5),
            onClick: function () {
                Bento.screens.show('screens/titlescreen');
            }
        });
        Bento.objects.attach(btnHome);

    };
    
    var onHide = function (data) {
        EventSystem.off('pointerUp', onTap);
    };
    
    var onTap = function (event) {
        Globals.currentLevel++;
        Bento.screens.show('screens/main');
    };
    
    return new Screen({
        onShow: onShow,
        onHide: onHide
    });
});