/**
 * Screen description
 */
bento.define('screens/preloader', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/components/fill',
    'bento/gui/text',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'globals',
    'modules/localization',
    'entities/musicmanager',
    'managers/save',
    'gui/iphonexoverlay',
    'managers/time',
    'modules/cmplay',
    'gui/overlays/dialog',
    'modules/mopub'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Fill,
    Text,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    Globals,
    Localization,
    MusicManager,
    SaveManager,
    IPhoneXOverlay,
    TimeManager,
    CMPlay,
    Dialog,
    MoPub
) {
    'use strict';
    var initPostPreloader = function () {
        // hide cordova splashscreen if exists
        if (navigator.splashscreen) {
            navigator.splashscreen.hide();
        }

        // Localization language is set after localized json files are loaded
        Localization.setLanguage();

        if (Globals.SOFTLAUNCH) {
            // force to english during softlaunch
            Localization.setLanguage('en');
        }

        CMPlay.pullCloudConfigData();

        //Initialize time manager
        TimeManager.initialize();

        catchExit();
    };

    var catchExit = function () {
        EventSystem.on('exitApp', function () {
            var dialog = new Dialog({
                name: 'exitAppStandard',
                text: Localization.getText('askQuit'),
                type: 'yesno',
                onYes: function () {
                    // notify CMPlay, then exit regardless of success or failure.
                    CMPlay.onGameExit(exit, exit);
                }
            });
            Bento.objects.attach(dialog);
        });
    };
    
    var exit = function () {
        if (navigator.app) {
            navigator.app.exitApp();
        }
    }; 

    var onShow = function () {
        /* Screen starts here */
        var viewport = Bento.getViewport();
        var loaded = false;
        var background = new Entity({
            z: 0,
            name: 'background',
            components: [
                new Fill({
                    color: [0x38 / 255, 0x8b / 255, 0xff / 255, 1]
                })
            ]
        });
        var text = Text({
            z: 1,
            name: 'loadingText',
            font: 'font',
            fontSize: 48,
            fontColor: '#fff',
            text: '',
            align: 'left',
            textBaseline: 'top',
            position: new Vector2(0, 0)
        });
        var end = function () {
            if (loaded) {
                Bento.objects.attach(new MusicManager({
                    visible: false
                }));

                // show iPhone X overlay if we are in dev mode and roughly the correct aspect ratio
                if (Utils.isDev()) {
                    // Bento.objects.attach(new IPhoneXOverlay({}));
                }

                Bento.screens.show('screens/titlescreen');
                // Bento.screens.show('screens/main');
                // Bento.screens.show('screens/walldebug');
                // Bento.screens.show('screens/testnativeaudio');
                // Bento.screens.show('screens/testyanap');
                // Bento.screens.show('screens/testads');
            }
        };
        var loadFonts = function () {
            var fonts = Bento.assets.getAssetGroups().preloader.fonts;
            var font;
            if (!fonts) {
                return;
            }
            for (font in fonts) {
                if (!fonts.hasOwnProperty(font)) {
                    continue;
                }
                new Text({
                    position: new Vector2(1000, 1000),
                    text: '.',
                    font: font
                });
            }
        };

        initPostPreloader();

        Bento.objects.attach(background);
        Bento.objects.attach(text);

        // preload fonts
        loadFonts();

        // DEMO: reset tutorial
        if (Globals.DEMO) {
            SaveManager.save('tutorialHoldToJump', 0);
        }

        // we will now load all other assets
        Bento.assets.loadAllAssets({
            exceptions: ['preloader'], // preloader was already loaded
            onLoaded: function (current, total) {
                // show how many assets still to be loaded
                if (!Utils.isCocoonJs()) {
                    text.setText('Loading ' + current + '/' + total);
                }
            },
            onReady: function () {
                loaded = true;
                end();

                // hide cordova splashscreen if exists
                if (navigator.splashscreen) {
                    navigator.splashscreen.hide();
                }
                if (window.UserSplash) {
                    window.UserSplash.hideSplash();
                }
            }
        });
    };

    var onHide = function () {
        if (SaveManager.load('bootups') === 0) {
            EventSystem.fire('GameAnalytics-custom', 'preloader:firstBoot');
        }
        SaveManager.add('bootups', 1);
        EventSystem.fire('GameAnalytics-custom', 'preloader:boot');
    };

    return new Screen({
        onShow: onShow,
        onHide: onHide
    });
});