/**
 * Shop screen
 */
bento.define('screens/shop', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/overlays/shop',
    'bento/components/spine',
    'gui/speaker'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    ShopOverlay,
    Spine,
    Speaker
) {
    'use strict';
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var background = new Entity({
            z: 0,
            position: new Vector2(0, 0),
            components: [
                new Sprite({
                    imageName: 'shop/background',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var speaker = new Speaker({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5)
        });
        var shopOverlay = new ShopOverlay(settings || {});

        background.scale.x = viewport.width / background.dimension.width * 1.1;
        background.scale.y = viewport.height / background.dimension.height;
        Bento.objects.attach(background);
        Bento.objects.attach(speaker);
        Bento.objects.attach(shopOverlay);

        Bento.setGameSpeed(1);
    };

    return new Screen({
        onShow: onShow,
        onHide: function () {
            // clear spine cache
            Spine.cleanLazyLoadedImages();
        }
    });
});