/**
 * World skin screen
 */
bento.define('screens/shopworld', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/coincounter',
    'gui/scrollinglist',
    'modules/planetskins',
    'managers/save',
    'gui/gametext',
    'globals',
    'gui/centeredlist',
    'gui/buttons/coinsforad',
    'gui/overlays/dialog',
    'gui/confetti',
    'bento/components/eventlistener',
    'gui/buttons/home',
    'modules/store',
    'gui/loadingspinner',
    'modules/localization',
    'modules/mopub'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    CoinCounter,
    ScrollingList,
    PlanetSkins,
    SaveManager,
    GameText,
    Globals,
    CenteredList,
    CoinsForAdButton,
    Dialog,
    Confetti,
    EventListener,
    HomeButton,
    Store,
    LoadingSpinner,
    Localization,
    MoPub
) {
    'use strict';
    var onShow = function (settings) {
        var target = settings && settings.target;
        var viewport = Bento.getViewport();
        var skins = Globals.getPlanetSkins();
        var shiftUp = MoPub.isShowingBanner();
        var background = new Entity({
            z: -100,
            name: 'bg',
            components: [
                new Sprite({
                    imageName: 'title/bg',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var planetScale = new Vector2(1, 1);
        var coinCounter = new CoinCounter({});
        var scrollingList = new ScrollingList({
            z: 0,
            position: new Vector2(viewport.width / 2, viewport.height / 2),
            updateWhenPaused: 0,
            direction: 'x',
            spacing: 720, // additional spacing between items
            maxOffset: 0, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: true, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 128,
            onSnap: function (item, index) {
                // show/hide purchase button
                var planet = item.getItem(0);
                setButton(planet);
                setBackground(planet);

                // immediately select
                if (isUnlocked(planet.name)) {
                    SaveManager.save('planetSkin', planet.name);
                }

                if (scrollingList.timer > 60) {
                    Bento.audio.stopSound('sfx_button');
                    Bento.audio.playSound('sfx_button');
                }
            },
            onLoseFocus: function (index) {}, // called when snap focus changed
            onScrollStart: function () {}, // callback when scrolling has started
            snapMinSlideSpeed: 100, // minimum slidespeed before starting to snap
            snapSpeed: 0.2, // speed of snapping
            damping: 0, // speed decrease, 
            components: [
                new EventListener({
                    eventName: 'cancelScroll',
                    onEvent: function (data) {
                        scrollingList.cancel();
                    }
                })
            ]
        });
        var buttonHome = new HomeButton({});
        
        var coinText = new GameText({
            position: new Vector2(-64, 0),
            text: '100',
            fontSize: 100,
            textBaseline: 'middle',
            shadow: true,
            shadowColor: 'rgba(0,0,0,0.3)',
            shadowOffset: new Vector2(8, 8),
            drawDebug: false,
            maxWidth: 256,
            maxHeight: 100
        });
        var coinIcon = new Entity({
            name: 'coin',
            position: new Vector2(128, 0),
            components: [
                new Sprite({
                    imageName: 'coin-icon',
                    originRelative: new Vector2(0.5, 0.5)
                })
            ]
        });
        var buttonPurchase = new ClickButton({
            name: 'buttonPurchase',
            sfx: 'sfx_button',
            imageName: 'shop/button-purchase2',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width / 2 - 128, viewport.height - 256 + (shiftUp ? -48 : 0)),
            visible: false,
            active: false,
            onButtonDown: function () {
                this.transform.y = 8;
                scrollingList.cancel();
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                var selected = scrollingList.getSelection();
                var planet = scrollingList.get(selected).getItem(0);
                var skin = planet.name;
                var cost = planet.cost;
                var coins = SaveManager.load('coins');
                if (planet.isIap) {
                    doPurchase(planet);
                } else if (!isUnlocked(skin)) {
                    if (coins >= cost) {
                        // unlock animation
                        unlock(planet);

                        // spend the coins
                        coins -= cost;
                        SaveManager.save('coins', coins);
                        EventSystem.fire('updateCoins');

                        EventSystem.fire('GameAnalytics-sink', {
                            currency: 'coins',
                            cost: cost,
                            sinkType: 'unlockWorldSkin',
                            sinkId: skin
                        });
                    } else {
                        // warn user
                        Bento.objects.attach(new Dialog({
                            name: 'notEnoughCoins',
                            text: 'Not enough coins!',
                            onComplete: function () {}
                        }));
                        return;
                    }
                }
            },
            components: [
                coinText,
                coinIcon
            ]
        });
        var buttonCoinsForAd = new CoinsForAdButton({
            position: new Vector2(viewport.width / 2 + 320, viewport.height - 256 + (shiftUp ? -48 : 0))
        });
        var currentSkin = SaveManager.load('planetSkin');
        var initialPage = 0;
        var isUnlocked = function (worldName) {
            var unlockedPlanets = SaveManager.load('unlockedPlanetSkins');
            return unlockedPlanets.indexOf(worldName) >= 0;
        };
        var createPlanet = function (worldName, cost) {
            var button = new ClickButton({
                name: worldName,
                sfx: 'sfx_button',
                position: new Vector2(0, 0),
                imageName: 'ui/button-play',
                frameCountX: 1,
                frameCountY: 1,
                updateWhenPaused: 0,
                float: false,
                dimension: new Rectangle(-100, -100, 200, 200),
                onButtonDown: function () {
                    this.transform.y = 8;
                },
                onButtonUp: function () {
                    this.transform.y = 0;
                },
                onClick: function () {
                    if (isUnlocked(worldName) && !scrollingList.didMove(5)) {
                        SaveManager.save('planetSkin', worldName);
                        Bento.screens.show('screens/titlescreen');
                    }
                }
            }).extend({
                cost: cost
            });
            var planet = PlanetSkins.create(worldName);
            var darken = new Entity({
                name: 'darkenOverlay',
                alpha: 0.75,
                components: [
                    new Sprite({
                        imageName: 'planet-darken',
                        originRelative: new Vector2(0.5, 0.5)
                    })
                ]
            });

            planet.scale = planetScale.clone();
            button.removeByName('sprite');
            button.attach(planet);

            if (!isUnlocked(worldName)) {
                if (cost === 0) {
                    // auto-unlock any world which costs 0 coins (such as purchased IAP worlds)
                    var unlockedPlanets = SaveManager.load('unlockedPlanetSkins');
                    unlockedPlanets.push(worldName);
                    SaveManager.save('unlockedPlanetSkins', unlockedPlanets);
                } else {
                    button.attach(darken);
                }
            }

            // start at currently selected world, unless settings.target was given
            if (button.name === (target ? target : currentSkin)) {
                initialPage = scrollingList.getItems().length;
            }

            // scrollingList.addItem(button);
            addToPage(button);

            // button.scale.x = 0.6;
            // button.scale.y = 0.6;
        };
        var createIapPack = function (iap, dummyPrice) {
            var productId = Utils.IapToId[iap];
            var product = Store.getProduct(productId);
            var price = product ? product.localizedPrice : dummyPrice;

            var iapBanner = new Entity({
                name: 'iapBanner',
                position: new Vector2(8, -544),
                scale: new Vector2(1.2, 1.2),
                components: [
                    new Sprite({
                        imageName: 'shop/banner-legendary',
                        originRelative: new Vector2(0.5, 0.5)
                    }),
                    new GameText({
                        position: new Vector2(0, Utils.isCocoonJs() ? -8 : 0),
                        text: Localization.getText(iap).toUpperCase(),
                        textBaseline: 'middle',
                        fontSize: 64,
                        maxWidth: 400,
                        maxHeight: 64,
                        linebreaks: false
                    })
                ]
            });

            var iapImage = new Entity({
                name: 'pack-'+iap,
                position: new Vector2(0, 0),
                scale: new Vector2(0.8, 0.8),
                components: [
                    new Sprite({
                        imageName: 'pack-'+iap,
                        originRelative: new Vector2(0.5, 0.5),
                    }),
                    iapBanner
                ]
            });
            Utils.extend(iapImage, {
                isIap: true,
                iapName: iap,
                productId: productId,
                cost: price,
            });
            addToPage(iapImage);
        };
        var doPurchase = function (planet) {
            var productId = planet.productId;
            var iapName = planet.iapName;

            if (SaveManager.load('purchased-' + productId)) {
                // already purchased this
                return;
            }

            var unlockIapPlanets = function () {
                // select a purchased planet (hard coded for now)
                SaveManager.save('planetSkin', 'world-cog');

                // planets are automatically unlocked when we refresh the screen.
                Bento.screens.show('screens/shopworld');
            };

            if (Utils.isDev()) {
                SaveManager.save('purchased-' + productId, true);
                unlockIapPlanets();
                return;
            }

            // using bento.require here so I can refresh this module
            bento.require(['gui/overlays/iap'], function (IapOverlay) {
                var overlay = IapOverlay.Purchase({
                    z: Globals.Layers.DIALOG,
                    id: productId,
                    fontSettings: {
                        font: 'FredokaOne-Regular',
                        fontSize: 100,
                        maxWidth: viewport.width - 256
                    },
                    onPurchaseComplete: function (success, pid, purchaseInfo) {
                        if (success && pid === productId) {
                            unlockIapPlanets();
                        }
                        overlay.removeByName('loadingSpinner');
                    }
                });
                overlay.attach(new LoadingSpinner({
                    radius: 128,
                    fadeSpeed: 1 / 60,
                    ballsToSpawn: 8,
                    rotateBalls: false,
                    imageName: 'dot',
                    position: new Vector2(viewport.width / 2, viewport.height * 2 / 3),
                    scale: new Vector2(0.5, 0.5)
                }));
            });

            EventSystem.fire('GameAnalytics-custom', {
                id: Utils.currentScreenName() + ':purchase:' + iapName
            });
        };
        var currentPage;
        var createPage = function () {
            var columns = 1;
            var rows = 1;
            var page = new Entity({
                name: 'page'
            }).extend({
                isFull: function () {
                    return page.components.length >= rows * columns;
                },
                addItem: function (item) {
                    var componentCount = page.components.length;
                    page.attach(item);

                    // item.position.x = -320 + 320 * (componentCount % columns);
                    // item.position.y = -320 + 320 * Math.floor(componentCount / columns);
                    // item.position.x = -256 + 512 * (componentCount % columns);
                    // item.position.y = -256 + 512 * Math.floor(componentCount / columns);
                    item.position.x = (componentCount % columns);
                    item.position.y = Math.floor(componentCount / columns);
                },
                getItem: function (index) {
                    return page.components[0];
                }
            });
            scrollingList.addItem(page);
            return page;
        };
        var addToPage = function (button) {
            if (!currentPage || currentPage.isFull()) {
                currentPage = createPage();
            }
            currentPage.addItem(button);
        };
        var setButton = function (planet) {
            if (isUnlocked(planet.name)) {
                buttonPurchase.active = false;
                buttonPurchase.visible = false;
                buttonCoinsForAd.active = false;
                buttonCoinsForAd.visible = false;
            } else if (planet.isIap) {
                buttonPurchase.active = true;
                buttonPurchase.visible = true;
                buttonCoinsForAd.active = false;
                buttonCoinsForAd.visible = false;
                coinIcon.visible = false;
                coinText.position.x = 0;
                buttonPurchase.position.x = viewport.width / 2;
                coinText.setText(planet.cost.toString());
            } else {
                buttonPurchase.active = true;
                buttonPurchase.visible = true;
                buttonCoinsForAd.active = true;
                buttonCoinsForAd.visible = true;
                coinIcon.visible = true;
                coinText.position.x = -64;
                buttonPurchase.position.x = viewport.width / 2 - 128;
                // set price
                coinText.setText(planet.cost.toString());
            }
        };
        var setBackground = function (planet) {
            Bento.objects.get('background', function (planetBackground) {
                planetBackground.removeSelf();
            });
            if (isUnlocked(planet.name)) {
                PlanetSkins.createBackground(planet.name);
            }
        };
        var unlock = function (planet) {
            // unlock immediately
            var unlockedPlanets = SaveManager.load('unlockedPlanetSkins');
            unlockedPlanets.push(planet.name);
            SaveManager.save('unlockedPlanetSkins', unlockedPlanets);

            // immediately select
            SaveManager.save('planetSkin', planet.name);

            var shrink = function () {
                // play animation
                new Tween({
                    from: planet.scale,
                    to: planet.scale.scalarMultiply(0.75),
                    in: 20,
                    ease: 'linear',
                    onUpdate: function (v, t) {
                        planet.scale = v;
                    },
                    onComplete: shake
                });
                Bento.audio.playSound('sfx_unlock');
            };
            var shake = function () {
                new Tween({
                    from: 0,
                    to: 32,
                    in: 20,
                    ease: 'quadratic',
                    onUpdate: function (v, t) {
                        planet.transform.x = Utils.getRandomRange(-v, v);
                        planet.transform.y = Utils.getRandomRange(-v, v);
                    },
                    onComplete: function () {
                        planet.transform.x = 0;
                        planet.transform.y = 0;
                        toNormal();
                    }
                });
            };
            var toNormal = function () {
                new Tween({
                    from: planet.scale,
                    to: planetScale,
                    in: 40,
                    ease: 'elastic',
                    decay: 3,
                    oscillations: 3,
                    onStart: function () {
                        planet.removeByName('darkenOverlay');

                        setButton(planet);
                        setBackground(planet);

                        Bento.objects.attach(new Confetti({
                            speedY: 4,
                            particleScale: 0.5,
                            duration: 40
                        }));
                    },
                    onUpdate: function (v, t) {
                        planet.scale = v;
                    },
                    onComplete: function () {}
                });
            };
            shrink();
        };
        background.scale.x = viewport.width / background.dimension.width * 1.1;
        background.scale.y = viewport.height / background.dimension.height;

        Bento.objects.attach(background);
        Bento.objects.attach(scrollingList);
        Bento.objects.attach(buttonHome);
        Bento.objects.attach(coinCounter);
        Bento.objects.attach(buttonPurchase);
        Bento.objects.attach(buttonCoinsForAd);

        buttonCoinsForAd.active = false;
        buttonCoinsForAd.visible = false;

        Utils.forEach(skins, function (skinData, i, l, breakLoop) {
            if (skinData[0].indexOf('iap-') === 0) {
                createIapPack(skinData[0].substring(4), skinData[1]);
            } else {
                createPlanet(skinData[0], skinData[1]);
            }
        });

        // activate scrollingList if we have more worlds
        if (scrollingList.getItems().length <= 1) {
            scrollingList.active = false;
        }

        // set background to grass
        setBackground(scrollingList.get(0).getItem(0));

        scrollingList.forceTo(2000);
        // force to current background
        scrollingList.snapTo(initialPage);

    };

    return new Screen({
        onShow: onShow
    });
});