/**
 * Bento YANAP test
 */
bento.define('screens/testyanap', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/gametext',
    'bento/components/fill'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    GameText,
    Fill
) {
    'use strict';

    var onShow = function (settings) {
        var viewport = Bento.getViewport();

        var background = new Entity({
            z: 0,
            name: 'background',
            components: [
                new Fill({
                    color: [0, 0, 0, 1]
                })
            ]
        });
        Bento.objects.attach(background);

        var text = GameText({
            position: new Vector2(viewport.width*0.5, viewport.height*0.08),
            text: 'YANAP test',
            fontSize: 100,
        });
        Bento.objects.attach(text);


        var btnx = 220;
        var btny = 400;
        var makeBtn = function (name, onClick) {
            Bento.objects.attach(new Entity({
                position: new Vector2(btnx, btny),
                scale: new Vector2(0.5, 0.5),
                components: [
                    new Sprite({
                        imageName: 'shop/button-purchase',
                        originRelative: new Vector2(0.5, 0.5),
                    }),
                    new GameText({
                        position: new Vector2(0, 0),
                        text: name,
                        fontSize: 100,
                        textBaseline: 'middle'
                    }),
                    new Clickable({
                        onClick: function () {
                            onClick();
                        }
                    })
                ]
            }));
            btny += 170;
        };

        var Yanap = window.cordova.plugins.Yanap;

        var onError = function (msg) {
            console.log('YANAP Error:', msg);
        };


        var suspendedInstances = {};

        var pauseAllPlaying = function () {
            for (var uid in Yanap.audioInstances) {
                var sound = Yanap.audioInstances[uid];
                var status = sound.status;
                var type = sound.audioType;
                if (status === 'playing' || status === 'looping') {
                    sound.stop();
                    if (type === 'music' || type === 'loop') {
                        suspendedInstances[uid] = sound;
                    }
                }
            }
        };
        var resumeAllPlaying = function () {
            for (var uid in suspendedInstances) {
                var sound = suspendedInstances[uid];
                sound.play();
            }
            suspendedInstances = {};
        };

        var bgm, sfxHit, sfxBossIntro;

        var onStatusUpdate = function (status, info) {
            console.log('YANAP', status, info ? '"'+info+'"' : '');
        };

        makeBtn('init', function () {
            Yanap.disableGaplessLoop();

            bgm = new Yanap.AudioInstance('loop', onStatusUpdate);
            bgm.load('file:///android_asset/www/assets/main/audio/bgm_test_loop.ogg');

            sfxHit = new Yanap.AudioInstance('sound', onStatusUpdate);
            sfxHit.load('file:///android_asset/www/assets/main/audio/sfx_hit.ogg');

            sfxBossIntro = new Yanap.AudioInstance('sound', onStatusUpdate);
            sfxBossIntro.load('file:///android_asset/www/assets/main/audio/sfx_boss_intro.ogg');
        });

        makeBtn('play bgm', function () {
            bgm.play();
        });
        makeBtn('sfx hit', function () {
            sfxHit.play();
        });
        makeBtn('sfx boss intro', function () {
            sfxBossIntro.play();
        });

        window.Cocoon.App.on('suspended', pauseAllPlaying);
        window.Cocoon.App.on('activated', resumeAllPlaying);

        window.Cocoon.App.on('suspended', function () {
            console.log('cocoon suspended');
        });
        window.Cocoon.App.on('activated', function () {
            console.log('cocoon activated');
        });
        document.addEventListener("pause", function () {
            console.log('document pause');
        });
        document.addEventListener("resume", function () {
            console.log('document resume');
        });
    };

    return new Screen({
        onShow: onShow
    });
});