/**
 * Title screen
 */
bento.define('screens/titlescreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/player',
    'gui/speaker',
    'globals',
    'bento/components/spine',
    'gui/centeredlist',
    'gui/coincounter',
    'gui/overlays/settings',
    'managers/time',
    'gui/buttons/gift',
    'gui/buttons/worldselect',
    'modules/gamecenter',
    'managers/save',
    'gui/buttons/shop',
    'gui/buttons/xp',
    'components/bouncewithmusic',
    'modules/gameanalytics',
    'modules/crashlytics',
    'components/backbutton',
    'modules/mopub',
    'modules/cmplay',
    'modules/googleplay',
    'modules/googleplaylogin'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Player,
    Speaker,
    Globals,
    Spine,
    CenteredList,
    CoinCounter,
    SettingsOverlay,
    TimeManager,
    GiftButton,
    WorldSelectButton,
    GameCenter,
    SaveManager,
    ShopButton,
    XPButton,
    BounceWithMusic,
    GameAnalytics,
    Crashlytics,
    BackButton,
    MoPub,
    CMPlay,
    GooglePlay,
    GooglePlayLogin
) {
    'use strict';
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var title = new Entity({
            z: 0,
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.23),
            components: [
                new Sprite({
                    originRelative: Vector2(0.5, 0.5),
                    imageName: 'title/title',
                })
            ]
        });
        var speaker = new Speaker({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63)
        });
        var background = new Entity({
            components: [
                new Sprite({
                    imageName: 'title/bg',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var player = new Player({
            planet: null, // dummy player
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63 - 256)
        });
        var coinCounter = new CoinCounter({});
        /**
         * Buttons
         */
        var buttonPlay = new ClickButton({
            name: 'play',
            sfx: 'sfx_button',
            imageName: 'title/button-play',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63),
            scale: new Vector2(1.1, 1.1),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                Globals.currentLevel = 0;
                EventSystem.fire('GameAnalytics-custom', Utils.currentScreenName()+':gui:play');
                Bento.screens.show('screens/main');
            }
        });
        var buttonSettings = new ClickButton({
            name: 'settings',
            sfx: 'sfx_button',
            imageName: 'title/button-settings',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(100, 100),
            scale: new Vector2(0.8, 0.8),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                var settingsOverlay = new SettingsOverlay({});
                Bento.objects.attach(settingsOverlay);
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName() + ':gui:settings'
                });
            }
        });
        var buttonLeaderboards = new ClickButton({
            name: 'leaderboards',
            sfx: 'sfx_button',
            imageName: 'title/button-leaderboards',
            frameCountX: 1,
            frameCountY: 1,
            position: new Vector2(100, 100 + 160),
            scale: new Vector2(0.8, 0.8),
            onButtonDown: function () {
                this.transform.y = 8;
            },
            onButtonUp: function () {
                this.transform.y = 0;
            },
            onClick: function () {
                if (Utils.isNativeIos()) {
                    GameCenter.show('com.luckykat.fishhop.leaderboards.grass');
                } else if (Utils.isNativeAndroid()) {
                    // log in first
                    if (GooglePlay.isLoggedIn()) {
                        GooglePlay.show('CgkI39Wq6N8fEAIQAA');
                    } else {
                        GooglePlayLogin(function (hasLoggedIn) {
                            if (hasLoggedIn) {
                                // try showing again
                                GooglePlay.show('CgkI39Wq6N8fEAIQAA');
                            }
                        });
                    }
                }
                EventSystem.fire('GameAnalytics-custom', {
                    id: Utils.currentScreenName() + ':gui:leaderboards'
                });
            }
        });
        var scale = 0.8;
        var buttonContainer = new CenteredList({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.63 + 320),
            spacing: -32
        });
        var buttonXp = new XPButton({
            scale: new Vector2(scale, scale),
            position: new Vector2(0, 0),
        });
        var buttonLevelSelect = new WorldSelectButton({
            scale: new Vector2(scale, scale),
            position: new Vector2(0, 100)
        });
        var buttonShop = new ShopButton({
            position: new Vector2(0, 100),
            scale: new Vector2(scale, scale)
        });
        var buttonGift = new GiftButton({
            scale: new Vector2(scale, scale),
            position: new Vector2(0, 0)
        });

        background.scale.x = viewport.width / background.dimension.width * 1.1;
        background.scale.y = viewport.height / background.dimension.height;

        Bento.objects.attach(background);
        Bento.objects.attach(speaker);


        Bento.objects.attach(title);

        Bento.objects.attach(player);
        Bento.objects.attach(buttonPlay);
        Bento.objects.attach(coinCounter);
        Bento.objects.attach(buttonSettings);
        Bento.objects.attach(buttonLeaderboards);
        
        // button container
        buttonContainer.addItem(buttonXp);
        buttonContainer.addItem(buttonLevelSelect);
        buttonContainer.addItem(buttonShop);
        buttonContainer.addItem(buttonGift);

        // show buttons after 1st gameover
        if (SaveManager.load('gameOvers') > 0) {
            Bento.objects.attach(buttonContainer);
        }

        // show banner ad
        if (
            SaveManager.load('showBanner') &&
            !MoPub.isShowingBanner() &&
            !SaveManager.load('purchased-' + Utils.IapToId['noads'])
        ) {
            MoPub.showBanner();
        }

        Bento.setGameSpeed(1);

        if (!SaveManager.load('hasShownGDPRView')) {
            CMPlay.showGDPRView(function () {
                // callback when user chooses 'continue'
                SaveManager.save('hasShownGDPRView', true);
                SaveManager.save('gdprConsent', true);
                MoPub.grantConsent();
            });
        }
    };

    return new Screen({
        onShow: onShow
    });
});