/**
 * Main screen
 */
bento.define('screens/walldebug', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'globals',
    'entities/planet',
    'components/planetwrap',
    'bento/components/fill',
    'entities/testplayer',
    'modules/maploader',
    'entities/background',
    'entities/musicmanager',
    'entities/zoom',
    'bento/components/eventlistener',
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    Utils,
    Screen,
    Tween,
    Globals,
    Planet,
    PlanetWrap,
    Fill,
    TestPlayer,
    MapLoader,
    Background,
    MusicManager,
    Zoom,
    EventListener
) {
    'use strict';

    var player;
    var planets;
    var currentPlanet;

    // this is used to tilt the world slightly ahead of where the player is looking
    var desiredAngle;

    // multiplier to specify how quickly the planet should reach the desired angle
    // 0.0 = don't rotate planet at all
    // 1.0 = instantly snap to desired angle
    // tween this from 0 to prevent jitters when the player teleports.
    var desiredAngleSpeed = 0.1;

    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var level = Globals.levels[Globals.currentLevel];
        var mapLoader = new MapLoader({});
        var zoom = new Zoom({
            zoom: 0.8
        });
        var background = new Background({});

        /**
         * Aligns planet with player position
         */
        var planetMover = {
            update: function () {
                if (Globals.DISABLE_PLANET_ROTATION) {
                    return;
                }
                player.getComponent('planetWrap', function (playerWrap) {
                    var diff = desiredAngle - playerWrap.angle;
                    var diff2 = diff - Math.PI*2;
                    var diff3 = diff + Math.PI*2;
                    if (Math.abs(diff2) < Math.abs(diff)) {
                        diff = diff2;
                    }
                    if (Math.abs(diff3) < Math.abs(diff)) {
                        diff = diff3;
                    }
                    currentPlanet.angle += diff * desiredAngleSpeed;
                });
            }
        };

        this.dimension.width = Infinity;
        this.dimension.height = Infinity;

        loadPlanets(mapLoader);
        // set up chunkloader (depedency on the player module)
        if (mapLoader.loadChunks) {
            mapLoader.chunkLoader.getCurrentX = function () {
                return Math.floor(player.position.x / mapLoader.chunkLoader.gridScale / mapLoader.chunkLoader.tiledSize);
            };
            Bento.objects.attach(mapLoader.chunkLoader);
        }

        desiredAngle = 0;
        player = new TestPlayer({
            planet: currentPlanet,
            position: currentPlanet.spawnPosition.clone()
        });

        Bento.objects.attach(background);
        Bento.objects.attach(player);
        Bento.objects.attach(planetMover);
        Bento.objects.attach(zoom);
    };

    var loadPlanets = function (mapLoader) {
        var viewport = Bento.getViewport();
        var i;
        planets = mapLoader.load({
            mapName: 'collision-test'
        });
        currentPlanet = planets[0];
        viewport.x = currentPlanet.position.x - viewport.width / 2;
        viewport.y = currentPlanet.position.y - viewport.height / 2;
        EventSystem.fire('setActivePlanet', currentPlanet);
    };

    return new Screen({
        onShow: onShow,
    });
});