/**
 * Victory! Next level, home, retry, etc.
 */
bento.define('screens/winscreen', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'entities/player',
    'gui/gametext',
    'globals',
    'gui/confetti',
    'gui/speaker',
    'gui/winstar',
    'gui/clickableregion'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Utils,
    Screen,
    Tween,
    Player,
    GameText,
    Globals,
    Confetti,
    Speaker,
    WinStar,
    ClickableRegion
) {
    'use strict';

    var onShow = function (settings) {

        settings = settings || {
            score: 3
        };

        var score = settings.score;

        var viewport = Bento.getViewport();
        
        // don't attach background yet, because we want it to handle clicks last of all
        var background = new Entity({
            z: -3,
            position: new Vector2(0, 0),
            scale: new Vector2(100, 1),
            components: [
                new Sprite({
                    imageName: 'ui/bg-victoryscreen',
                })
            ]
        });
        Bento.objects.attach(background);

        Bento.objects.attach(new Speaker({ 
            z: -2,
            imageName: 'ui/winscreen-speaker'
        }));

        var spawnConfetti = function () {
            var confetti = new Confetti({
                speedY: 4,
                particleScale: 0.2,
                z: -1,
            });
            Bento.objects.attach(confetti);
            Bento.audio.playSound('sfx_cheer');
        };

        var makeStar = function (x, y, angle, n) {
            Bento.objects.attach(new WinStar({
                starNumber: n,
                playerScore: settings.score,
                position: new Vector2(x, y),
                rotation: angle,
                onHighlight: function () {
                    if (settings.score === n) readyToContinue();
                }
            }));
        };

        makeStar(0.5*viewport.width - 250, 0.5*viewport.height - 310, -0.7, 1);
        makeStar(0.5*viewport.width,       0.5*viewport.height - 390,  0,   2);
        makeStar(0.5*viewport.width + 250, 0.5*viewport.height - 310,  0.7, 3);

        if (settings.score == 0) {
            // edge case, we can continue immediately if we didn't get any stars
            readyToContinue();
        } else {
            // spawn confetti as long as they scored something
            spawnConfetti();
        }

        var player = new Player({
            planet: null, // dummy player
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5)
        });
        Bento.objects.attach(player);

        Bento.objects.attach(new GameText({
            position: Vector2(0.5 * viewport.width, 0.1 * viewport.height),
            text: 'COMPLETED',
            fontSize: 140,
        }));

        var btnReplay = new ClickButton({
            z: 0,
            name: 'replay',
            sfx: 'sfx_button',
            imageName: 'ui/button-replay',
            position: new Vector2(viewport.width * 0.85, viewport.height * 0.92),
            float: true,
            scale: Vector2(0.5, 0.5),
            onClick: function () {
                Bento.screens.show('screens/main');
            }
        });
        if (settings.score !== 3) {
            Bento.objects.attach(btnReplay);
        }

        var btnHome = new ClickButton({
            z: 0,
            name: 'home',
            sfx: 'sfx_button',
            imageName: 'ui/button-home',
            position: new Vector2(viewport.width * 0.15, viewport.height * 0.92),
            float: true,
            scale: Vector2(0.5, 0.5),
            onClick: function () {
                Bento.screens.show('screens/titlescreen');
            }
        });
        Bento.objects.attach(btnHome);

    };

    var readyToContinue = function () {
        var viewport = Bento.getViewport();
        var position = Vector2(0.5 * viewport.width, viewport.height + 100);
        var text, size, duration, delay;

        if (Globals.levels[Globals.currentLevel+1]) {
            size = 90;
            text = 'tap to continue';
            duration = 100;
            delay = 20;
            Bento.objects.attach(new ClickableRegion({
                dimension: new Rectangle(0, 0, 1, 0.88),
                onClickUp: onTap
            }));
            Bento.objects.attach(new ClickableRegion({
                dimension: new Rectangle(0.3, 0.88, 0.4, 0.22),
                onClickUp: onTap
            }));
        } else {
            size = 110;
            text = 'thank you\nfor playing!';
            duration = 160;
            delay = 60;
        }

        Bento.objects.attach(new GameText({
            position: position,
            text: text,
            fontSize: size,
        }));

        new Tween({
            from: position.y,
            to: 0.66 * viewport.height,
            in: duration,
            delay: delay,
            ease: 'easeOutElastic',
            oscillations: 0.5,
            onUpdate: function (v, t) {
                position.y = v;
            }
        });

    };

    var onHide = function (data) {
        EventSystem.off('pointerUp', onTap);
    };

    var onTap = function (event) {
        Globals.currentLevel++;
        Bento.screens.show('screens/main');
    };

    return new Screen({
        onShow: onShow,
        onHide: onHide
    });
});