/**
 * World select screen
 */
bento.define('screens/worldselect', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/screen',
    'bento/tween',
    'gui/speaker',
    'gui/coincounter',
    'gui/scrollinglist',
    'modules/planetskins',
    'managers/save',
    'gui/gametext',
    'globals',
    'gui/buttons/home'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Screen,
    Tween,
    Speaker,
    CoinCounter,
    ScrollingList,
    PlanetSkins,
    SaveManager,
    GameText,
    Globals,
    HomeButton
) {
    'use strict';
    var onShow = function (settings) {
        var viewport = Bento.getViewport();
        var speaker = new Speaker({
            position: new Vector2(viewport.width * 0.5, viewport.height * 0.5)
        });
        var background = new Entity({
            components: [
                new Sprite({
                    imageName: 'title/bg',
                    frameCountX: 3,
                    animations: {
                        default: {
                            frames: [1] // take the middle frame for edge bleeding
                        }
                    }
                })
            ]
        });
        var coinCounter = new CoinCounter({});
        var scrollingList = new ScrollingList({
            z: 0,
            position: new Vector2(viewport.width / 2, viewport.height * 0.25),
            updateWhenPaused: 0,
            direction: 'y',
            spacing: 160, // additional spacing between items
            maxOffset: 0, // additional max scrolling distance
            minOffset: 0, // additional min scrolling distance
            snap: false, // snap items (cancels maxOffset and minOffset)
            snapNextOffset: 0, // distance to scroll to snap to next, in screen pixels. Used with snap:true, best with damping:0
            onSnap: function (item, index) {}, // called when snap focus changed
            onLoseFocus: function (index) {}, // called when snap focus changed
            onScrollStart: function () {}, // callback when scrolling has started
            snapMinSlideSpeed: 1, // minimum slidespeed before starting to snap
            snapSpeed: 0.3, // speed of snapping
            damping: 0.9 // speed decrease,
        });
        var buttonHome = new HomeButton({});
        
        var createPlanet = function (worldName) {
            var button = new ClickButton({
                name: worldName,
                sfx: 'sfx_button',
                position: new Vector2(0, 0),
                imageName: 'ui/button-play',
                frameCountX: 1,
                frameCountY: 1,
                updateWhenPaused: 0,
                float: false,
                onButtonDown: function () {
                    this.transform.y = 8;
                },
                onButtonUp: function () {
                    this.transform.y = 0;
                },
                onClick: function () {
                    Globals.currentWorld = worldName;
                    SaveManager.save('currentWorld', worldName);
                    Bento.screens.show('screens/titlescreen');
                }
            });
            var planet = PlanetSkins.create(worldName);
            var highscore = SaveManager.load('highScore-' + worldName);
            var highscoreText = new GameText({
                position: new Vector2(0, 256),
                text: 'Best: ' + highscore,
                fontSize: 64,
                fontColor: '#ffffff',
                textBaseline: 'middle',
                shadow: true,
                shadowColor: 'rgba(0,0,0,0.3)',
                shadowOffset: new Vector2(4, 4),
            });
            planet.scale = new Vector2(0.8, 0.8);

            button.attach(planet);
            button.attach(highscoreText);

            scrollingList.addItem(button);
        };
        background.scale.x = viewport.width / background.dimension.width * 1.1;
        background.scale.y = viewport.height / background.dimension.height;

        Bento.objects.attach(background);
        // Bento.objects.attach(speaker);
        Bento.objects.attach(scrollingList);
        Bento.objects.attach(buttonHome);
        Bento.objects.attach(coinCounter);

        createPlanet('world-grass');
        createPlanet('world-forest');
        createPlanet('world-disco');

        // activate scrollingList if we have more worlds
        scrollingList.active = false;
    };

    return new Screen({
        onShow: onShow
    });
});