cordova.define("cordova-plugin-mopubad.MoPub", function(require, exports, module) {
/**
 * Cordova MoPub Ad Network Plugin. Please make sure you open .xcworkspace file in Xcode rather than .xcodeproj
 * @class 
 * @example
 *  cordova plugin add -d PathToPluginFolder
 *  cd platforms/ios
 *  pod install 
 */

var exec = require('cordova/exec');

/**
 * This function is used to initialize the MoPub SDK with any Ad unit id banner/interstitial/reward. SDK should be initialize first before making any other call.
 *
 * @method initSdk
 * @summary USAGE - cordova.plugins.MoPub.initSdk(adId, successCallback, errorCallback)
 * @param {string} adId - Ad unit id.
 * @param {boolean} consent - Personalised advertisement consent status
 * @param {function} successCallback - Callback function which will be called when sdk is initialized successfully.
 * @param {function} errorCallback - Callback function which will be called if there is on ad unit id passed.
 * @example
 * var adId = "0ac59b0996d947309c33f59d6676399f"
 *
 * cordova.plugins.MoPub.initSdk(adId, successCallback, errorCallback)
 */
exports.initSdk = function (adId, consent, success, error) {
    exec(success, error, 'MoPubAdPlugin', 'initSdk', [adId, consent]);
};


/**
 * This function gives you the device size, which can be used to set the Y position of native ad banner
 *
 * @method getDeviceSize
 * @summary USAGE - cordova.plugins.MoPub.getDeviceSize(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.getDeviceSize(successCallback, errorCallback)
 */
exports.getDeviceSize = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'getDeviceSize', []);
};

/**
 * This function is used to initialize the MoPub Ad Banner View. It doesnt load, it just prepare the ad and ad view. To load the ad call showAd method.
 *
 * @method createBannerView
 * @summary USAGE - cordova.plugins.MoPub.createBannerView(options, successCallback, errorCallback)
 * @param {Object} options - See example for options available.
 * @param {function} successCallback - Callback function which will be called banner view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *    var options = {
 *           adUnitId:"0ac59b0996d947309c33f59d6676399f",
 *           interstitialAdId:"4f117153f5c24fa6a3a92b818a5eb630",
 *           rewardVideoId:"8f000bd5e00246de9c789eed39ff6096",
 *           adSize:"BANNER", //SKYSCRAPER, LEADERBOARD, MEDIUM_RECTANGLE
 *           bannerAtTop:true, //true
 *           bannerOverlap:true,
 *           autoRefreshBanner: true,
 *           offsetTopBar:true,
 *           autoShow: false
 *      };
 *
 *  cordova.plugins.MoPub.createBannerView(options, successCallback, errorCallback)
 */

exports.createBannerView = function (options, success, error) {
    console.log('Options plugin', options);
    exec(success, error, 'MoPubAdPlugin', 'createBannerView', [options]);
};

/**
 * This function is used to remove the MoPub Ad Banner sub view.
 *
 * @method removeBannerView
 * @summary USAGE - cordova.plugins.MoPub.removeBannerView(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called banner view is removed.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *  cordova.plugins.MoPub.removeBannerView(successCallback, errorCallback)
 */

exports.removeBannerView = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'removeBannerView', []);
};


exports.enableAutoRefresh = function () {
    exec(null, null, 'MoPubAdPlugin', 'enableAutoRefresh', []);
};

exports.disableAutoRefresh = function () {
    exec(null, null, 'MoPubAdPlugin', 'disableAutoRefresh', []);
};

/**
 * This function is used to display the banner ad which is intialized first using the create banner view.
 *
 * @method showAd
 * @summary USAGE - cordova.plugins.MoPub.showAd(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.showAd(successCallback, errorCallback)
 */

exports.showAd = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'showAd', [true]);
};

/**
 * This function hides the currently displaying ad, but doesn't remove it completely like removeBannerView.
 *
 * @method hideAd
 * @summary USAGE - cordova.plugins.MoPub.hideAd(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.hideAd(successCallback, errorCallback)
 */
exports.hideAd = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'showAd', [false]);
};

/**
 * This function is used to refressh the banner ads and load the ad based on certain keyword.
 *
 * @method refreshBannerAds
 * @summary USAGE - cordova.plugins.MoPub.refreshBannerAds(keyword, successCallback, errorCallback)
 * @param {String} keyword - Any keyword to target the ads
 * @example
 *  var keyword = "icecream";
 *  cordova.plugins.MoPub.refreshBannerAds(keyword, successCallback, errorCallback)
 */
exports.refreshBannerAds = function (keyword, success, error) {
    exec(success, error, 'MoPubAdPlugin', 'refreshBannerAds', [keyword]);
};

/**
 * This function is used to initialize the MoPub Interstitial ads. It doesnt load, it just prepare the ad. To load the ad call showInterstitialAd method.
 *
 * @method createInterstitialView
 * @summary USAGE - cordova.plugins.MoPub.createInterstitialView(options, successCallback, errorCallback)
 * @param {Object} options - See example for options available. You can pass empty object if already passed the interstitialAdId in createbannerview method.
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *    var options = {
 *           interstitialAdId:"4f117153f5c24fa6a3a92b818a5eb630",
 *      };
 *
 *  cordova.plugins.MoPub.createInterstitialView(options, successCallback, errorCallback)
 */

exports.createInterstitialView = function (options, success, error) {
    console.log('Options plugin', options);
    exec(success, error, 'MoPubAdPlugin', 'createInterstitialView', [options]);
};

/**
 * This function is used to display the interstitial ad which is intialized first using the create interstial view method. Please use isInterstialReady method to check if the ad is * ready to get displayed.
 *
 * @method showInterstitialAd
 * @summary USAGE - cordova.plugins.MoPub.showInterstitialAd(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.showInterstitialAd(successCallback, errorCallback)
 */
exports.showInterstitialAd = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'showInterstitialAd', []);
};

/**
 * This function is used to check if the interstitial ad is ready to get displayed.
 * Returns true if ready otherwise false
 *
 * @method isInterstitialReady
 * @summary USAGE - cordova.plugins.MoPub.isInterstitialReady(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.isInterstitialReady(successCallback, errorCallback)
 */
exports.isInterstitialReady = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'isInterstitialReady', []);
};

/**
 * This function is used to initialize the MoPub Reward Video ads. It doesnt load video ad, it just prepare the ad. To load the ad call showRewardVideo method.
 *
 * @method createRewardVideo
 * @summary USAGE - cordova.plugins.MoPub.createRewardVideo(options, successCallback, errorCallback)
 * @param {Object} options - See example for options available. You can pass empty object if already passed the rewardVideoId in createbannerview method.
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *    var options = {
 *           rewardVideoId:"8f000bd5e00246de9c789eed39ff6096",
 *      };
 *
 *  cordova.plugins.MoPub.createRewardVideo(options, successCallback, errorCallback)
 */
exports.createRewardVideo = function (options, success, error) {
    console.log('Options plugin', options);
    exec(success, error, 'MoPubAdPlugin', 'createRewardVideo', [options]);
};

/**
 * This function is used to display the reward video ad which is intialized first using the create reward video method. Please use isRewardVideoReady method to check if the ad is 
 * ready to get displayed.
 *
 * @method showRewardVideo
 * @summary USAGE - cordova.plugins.MoPub.showRewardVideo(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.showRewardVideo(successCallback, errorCallback)
 */
exports.showRewardVideo = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'showRewardVideo', []);
};

/**
 * This function is used to check if the reward video ad is ready to get displayed.
 * Returns true if ready otherwise false
 *
 * @method isRewardVideoReady
 * @summary USAGE - cordova.plugins.MoPub.isRewardVideoReady(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.isRewardVideoReady(successCallback, errorCallback)
 */
exports.isRewardVideoReady = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'isRewardVideoReady', []);
};

/**
 * This function is used to prepare the native ads.
 *
 * @method prepareNativeAd
 * @summary USAGE - cordova.plugins.MoPub.prepareNativeAd(options, successCallback, errorCallback)
 * @example
 *  var options = {
 *       nativeAdId: "76a3fefaced247959582d2d2df6f4757",
 *       autoshow: false, //display ad automatically or not
 *       position: "0" //0 means center position by default, you can change it to any number you want. Get the max Y position using getDeviceSize method.
 *       //Native ad Height is 312, so make sure to substract it from the actual *device size, when setting up the position otherwise ad will go below the screen. If device height is 500, then to show at the center you have to do (500 - 312)/2. It applies for other position too.
 *  };
 *  cordova.plugins.MoPub.prepareNativeAd(options, successCallback, errorCallback)
 */
exports.prepareNativeAd = function (options, success, error) {
    if (!options.positionY) options.positionY = "0";
    if (!options.autoshow) options.autoshow = false;
    exec(success, error, 'MoPubAdPlugin', 'prepareNativeAd', [options]);
};

/**
 * This function is used to display the native ads. Make sure you call isNativeAdReady before to check if ad is loaded or not before calling this method.
 *
 * @method showNativeAd
 * @summary USAGE - cordova.plugins.MoPub.showNativeAd(options, successCallback, errorCallback)
 * @example
 *  var positionY: "0" //0 means center position by default, you can change it to any number you want. Get the max Y position using getDeviceSize method.
 *       //Native ad Height is 312, so make sure to substract it from the actual *device size, when setting up the position otherwise ad will go below the screen. If device height is 500, then to show at the center you have to do (500 - 312)/2. It applies for other position too.
 *  };
 *  cordova.plugins.MoPub.showNativeAd(positionY, successCallback, errorCallback)
 */
exports.showNativeAd = function (positionY, success, error) {
    if (!positionY) positionY = "0";
    exec(success, error, 'MoPubAdPlugin', 'showNativeAd', [positionY]);
};

/**
 * This function is used to check if the native ad is ready to get displayed.
 * Returns true if ready otherwise false
 *
 * @method isNativeAdReady
 * @summary USAGE - cordova.plugins.MoPub.isNativeAdReady(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.isNativeAdReady(successCallback, errorCallback)
 */
exports.isNativeAdReady = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'isNativeAdReady', []);
};
/**
 * This function hides the currently displaying native ad and remove the view
 *
 * @method hideNativeAd
 * @summary USAGE - cordova.plugins.MoPub.hideNativeAd(successCallback, errorCallback)
 * @example
 *  cordova.plugins.MoPub.hideNativeAd(successCallback, errorCallback)
 */
exports.hideNativeAd = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'hideNativeAd', []);
};

exports.destroyNativeAd = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'destroyNativeAd', []);
};

/**
 * Following listeners are fired for banner ads
 * @event Banner Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('adViewDidFailToLoadAd', d, false);
 * document.addEventListener('willPresentModalViewForAd', d, false);
 * document.addEventListener('didDismissModalViewForAd', d, false);
 * document.addEventListener('willLeaveApplicationFromAd', d, false);
 * document.addEventListener('adViewDidLoadAd', d, false);
 */

/**
 * Following listeners are fired for Interstial ads
 * @event Interstial Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('interstitialDidLoadAd', d, false);
 * document.addEventListener('interstitialDidFailToLoadAd', d, false);
 * document.addEventListener('interstitialDidFailToLoadAd', d, false);
 * document.addEventListener('interstitialWillAppear', d, false);
 * document.addEventListener('interstitialDidAppear', d, false);
 * document.addEventListener('interstitialWillDisappear', d, false);
 * document.addEventListener('interstitialDidDisappear', d, false);
 * document.addEventListener('interstitialDidReceiveTapEvent', d, false);
 */

/**
 * Following listeners are fired for Reward Video ads
 * @event Reward Video Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('rewardedVideoAdDidLoadForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdDidFailToPlayForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdWillAppearForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdDidAppearForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdWillDisappearForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdDidDisappearForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdDidExpireForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdDidReceiveTapEventForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdShouldRewardForAdUnitID', d, false);
 * document.addEventListener('rewardedVideoAdWillLeaveApplicationForAdUnitID', d, false);
 */

/**
 * Following listeners are fired for native ads
 * @event Native Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('willPresentModalForNativeAd', d, false);
 * document.addEventListener('didDismissModalForNativeAd', d, false);
 * document.addEventListener('willLeaveApplicationFromNativeAd', d, false);
 */
exports.fireDocumentAdEvent = function (eventName, data) {
    var event = document.createEvent('Event');
    event.initEvent(eventName, true, true);
    event.msg = data;
    document.dispatchEvent(event);
};


/**
 * GDPR Stuff
 */
exports.shouldShowConsentDialog = function (success) {
    exec(success, null, 'MoPubAdPlugin', 'shouldShowConsentDialog', []);
};

exports.showConsentDialog = function (success, error) {
    exec(success, error, 'MoPubAdPlugin', 'showConsentDialog', []);
};

exports.grantConsent = function () {
    exec(null, null, 'MoPubAdPlugin', 'grantConsent', []);
};

exports.revokeConsent = function () {
    exec(null, null, 'MoPubAdPlugin', 'revokeConsent', []);
};
});
